<?php

/**
 * AutoTweet plugin for JEvents Component
 *
 * @version     1.0
 * @copyright   Copyright (C) 2008-2009 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://www.jevents.net
 */
defined('_JEXEC') or die('Restricted access');
jimport('joomla.error.error');
JLoader::register('JevJoomlaVersion',JPATH_ADMINISTRATOR."/components/com_jevents/libraries/version.php");

// check for AutoTweet component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	\Joomla\CMS\Factory::getApplication()->enqueueMessage( 'AutoTweet NG JEvents Plugin - AutoTweet NG Component is not installed or not enabled.', "warning");
	return;
}

// check for JEvents component
if (!JComponentHelper::getComponent('com_jevents', true)->enabled)
{
	\Joomla\CMS\Factory::getApplication()->enqueueMessage( 'AutoTweet NG JEvents Plugin - JEvents Component is not installed or not enabled.', "warning");
	return;
}

if (!defined('AUTOTWEET_API') && !@include_once(JPATH_ADMINISTRATOR.'/components/com_autotweet/api/autotweetapi.php')) {
	return;
}

/**
 * JEvents extension plugin for AutoTweet. Handles events as new tweets.
 */
class plgsystemAutotweetJEvents extends plgAutotweetBase
{

	protected $categories = '';
	protected $excluded_categories = '';
	protected $users = '';
	protected $excluded_users = '';
	protected $post_modified = 0;
	protected $add_description = 0;
	protected $add_description_count = 80;
	protected $leadtime = 86400;
	protected $post_repeats = 0;
	protected $post_modified_repeats = 0;
	private $jevent = null;

	/**
	 * constructor
	 */
	function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$pluginParams = $this->params;

		// plugin specific params
		$this->categories = $pluginParams->get('categories', '');
		$this->excluded_categories = $pluginParams->get('excluded_categories', '');
		$this->users = $pluginParams->get('authors', '');
		$this->excluded_users = $pluginParams->get('excluded_authors', '');
		$this->post_modified = (int) $pluginParams->get('post_modified', 0);
		$this->add_description = (int) $pluginParams->get('add_description', 0);
		$this->add_description_count = (int) $pluginParams->get('add_description_count', 80);
		$this->leadtime = (int) $pluginParams->get('whentopost2', 86400);
		$this->post_repeats = (int) $pluginParams->get('post_repeats', 0);
		$this->post_modified_repeats = (int) $pluginParams->get('post_modified_repeats', 0);

		// check if limit is allright
		$this->add_description_count = $this->getTextcount($this->add_description_count);

		$this->show_info = (int) $pluginParams->get('show_info', 0);
		$this->info_position = (int) $pluginParams->get('info_position', 0);
		$this->info_format = $pluginParams->get('info_format', '');
		$this->date_format = $pluginParams->get('date_format', "Y-m-d");

		$this->whentopost = (int) $pluginParams->get('whentopost', 0);
		include_once(JPATH_SITE."/components/com_jevents/libraries/jevdate.php");

	}

	/**
	 * Trigger called when new/edited event is published
	 */
	function onAfterSaveEvent($event, $dryrun =false)
	{
		if ($dryrun)
			return true;
		$this->postMessage($event);
		return true;

	}

	/**
	 * Trigger called when new/edited event is processed
	 */
	function onPublishEvent($cid, $newstate)
	{
		// get the data and query models
		$dataModel = new JEventsDataModel();
		$queryModel = new JEventsDBModel($dataModel);

		JPluginHelper::importPlugin("jevents");

		foreach ($cid as $id){
			// get event by event id - include unpublished events

			$this->jevent = $queryModel->getEventById(intval($id), 1, "icaldb");

			if (!$this->jevent || $newstate<=0)
			{
				return;
			}

			$this->processEvent($id);
		}
	}

	/*
	 * Specific repeat is saved 
	 */
	function onStoreCustomRepeat(&$repeat)
	{

		// Are we saving a repeat - then notify of a change to this repeat
		$task = JFactory::getApplication()->input->getString("task", "");
		if ($task == "icalrepeat.save" || $task == "icalrepeat.apply"  || $task == "icals.reload")
		{
			$repeat_id = intval(JFactory::getApplication()->input->getInt("rp_id", 0));

			// get the data and query models
			$dataModel = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// get event by repeat id
			$therepeat = $queryModel->listEventsById($repeat_id, 1, "icaldb");

			if (!$therepeat || !$therepeat->published())
				return;


			if ($therepeat)
			{
				$this->jevent = $therepeat;
				$this->processEvent($repeat_id, true);
			}
		}

	}

	protected function postMessage($event)
	{

		// get the data and query models
		$dataModel = new JEventsDataModel();
		$queryModel = new JEventsDBModel($dataModel);

		JPluginHelper::importPlugin("jevents");

		// get event by event id - include unpublished events
		$id = $event->ev_id;
		$this->jevent = $queryModel->getEventById(intval($id), 1, "icaldb");

		if (!$this->jevent)
		{
			return;
		}
		/* Make this a configurable option
		if ($this->jevent->priority()<4) {
			return;
		}
		 * 
		 */
		if (!$this->jevent->state())  {
			return;
		}

		$this->processEvent($id);
	}

	/*
	 * By default processing whole event as opposed to modification to  specific repeat
	 */
	private function processEvent($id=0, $isChangedRepeat = false) {

		$user = JFactory::getUser();
		$userid = $user->id;
		$catid = $this->jevent->_catid;

		if ($this->post_modified_repeats == 0 && $isChangedRepeat)
		{ // if disabled posting modified REPEAT , then end
			return;
		}

		// check access level
		$accesslevel = $this->params->get("accesslevel",array());
		if (is_array($accesslevel) && count($accesslevel)!=0){
			JArrayHelper::toInteger($accesslevel);
			$access = $this->jevent->access();
			if (!in_array($access, $accesslevel)){
				return;
			}
		}
		$category_ok = $this->checkCategoryValidity($catid);
		$user_ok = $this->checkUserValidity($userid);

		if ($category_ok && $user_ok)
		{
			if (JFactory::getApplication()->input->getInt("evid") != 0)
			{
				if ($this->post_modified == 0 && !$isChangedRepeat)
				{ // if disabled posting modified EVENT and this isn't a changed repeat then end
					return;
				}
			}

			// separate tweet for each repeat
			// If its a changed repeat then don't create a separate POST for each of the other repeats
			if ($this->post_repeats && !$isChangedRepeat)
			{
				$query = "SELECT ev.*, rpt.*, rr.*, det.*"
					. "\n , YEAR(rpt.startrepeat) as yup, MONTH(rpt.startrepeat ) as mup, DAYOFMONTH(rpt.startrepeat ) as dup"
					. "\n , YEAR(rpt.endrepeat  ) as ydn, MONTH(rpt.endrepeat   ) as mdn, DAYOFMONTH(rpt.endrepeat   ) as ddn"
					. "\n , HOUR(rpt.startrepeat) as hup, MINUTE(rpt.startrepeat ) as minup, SECOND(rpt.startrepeat ) as sup"
					. "\n , HOUR(rpt.endrepeat  ) as hdn, MINUTE(rpt.endrepeat   ) as mindn, SECOND(rpt.endrepeat   ) as sdn"
					. "\n FROM #__jevents_vevent as ev"
					. "\n LEFT JOIN #__jevents_icsfile as icsf ON icsf.ics_id=ev.icsid"
					. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid = ev.ev_id"
					. "\n LEFT JOIN #__jevents_vevdetail as det ON det.evdet_id = rpt.eventdetail_id"
					. "\n LEFT JOIN #__jevents_rrule as rr ON rr.eventid = ev.ev_id"
					. "\n WHERE ev.ev_id=" . intval($id)
					. "\n GROUP BY rpt.rp_id"
					. "\n ORDER BY rpt.startrepeat";
				$db = JFactory::getDbo();
				$db->setQuery($query);
				$icalrows = $db->loadObjectList();
				$icalcount = count($icalrows);
				for ($i = 0; $i < $icalcount; $i++)
				{
					// convert rows to jIcalEvents
					$icalrows[$i] = new jIcalEventRepeat($icalrows[$i]);
					// event may be deleted
					if (!$icalrows[$i]) continue;

					$postevent = $icalrows[$i];

					// attach location etc.
					
					JFactory::getApplication()->triggerEvent('onDisplayCustomFields', array(&$postevent));

					if (isset($postevent->_imageurl1))
					{
						$image_url = $postevent->_imageurl1;
					}
					else
					{
						$image_url = '';
					}
					$url = $postevent->viewDetailLink($postevent->yup(), $postevent->mup(), $postevent->dup(), true);
					$id = $postevent->rp_id();

					/*
					 * $data = array (
					 * 	'title'	=> 'Title of the article, event, forum post, ...',
					 * 	'text'	=> 'Text for the message.',
					 * 	'hashtags'	=> 'hashtags (They are inserted automatically by AutoTweet for channels supporting hashtags.',
					 * 	'url'		=> 'The URL of the article. (not shortened)'
					 * 	'image_url'	=> 'URL for the image to show (if there is an image)'
					 * 	'fulltext'	=> 'Enhanced text for the description (e. g. Facebook). Not available for all channels.'
					 * 	'catids'	=> 'array with category, container, etc. ids for the article, post...'
					 * 	'author'	=> 'creator or editor of the article'
					 * 	'event'	=> array (
					 * 		'location'		=> 'Location of the event'
					 * 		'street'		=> 'Street is optional'
					 * 		'city'			=> 'Must be a valid City for Facebook'
					 * 		'privacy'		=> 'OPEN, CLOSED or SECRET'
					 * 		'start_time'	=> 'yyyy-mm-dd hh:ss'
					 * 		'end_time'	=> 'yyyy-mm-dd hh:ss'
					 * 	);
					 * );
					 */

					$data = array();
					$data["title"] = $postevent->title();
					$data["text"] = $postevent->content();
					$data["url"] = $url;
					$data["image_url"] = $image_url;
					$data["catids"] = array($postevent->catid());
					$data["author"] = $postevent->creatorName();
					$data["event"] = array();
					$data["event"]["location"] = $postevent->location();
					$data["event"]["street"] = "";
					$data["event"]["city"] = "";
					$data["event"]["privacy"] = "OPEN";

					// http://developers.facebook.com/blog/post/2012/08/01/platform-migration--events-timezone-support/
					$data["event"]["start_time"] = JFactory::getDate($postevent->getUnixStartTime())->toISO8601();
					$data["event"]["end_time"] = JFactory::getDate($postevent->getUnixEndTime())->toISO8601();
					//$data["event"]["start_time"] = $posttime = strftime('%Y-%m-%d %H:%M', $postevent->getUnixStartTime());
					//$data["event"]["end_time"] = $posttime = strftime('%Y-%m-%d %H:%M', $postevent->getUnixEndTime());

					if ($this->whentopost == 0)
					{
						// post immediately
						$now = new JevDate();
						//echo "posting event now = ".$now->toFormat()."<br/>";
						//var_dump($data);
						//echo "<br/>";
						//exit();
						// Joocial now respects timezones
						$this->postStatusMessage($id, $now->toISO8601(), $postevent->title(), 1, $url, $image_url);
						//AutotweetAPI::postMessage($postevent, $data, "JEvents Plugin");
					}
					else if ($this->whentopost == 1)
					{
						// post 1 day before start of the event
						//$posttime = strftime('%Y-%m-%d %H:%M:%S', $postevent->getUnixStartTime() - $this->leadtime);
						// Joocial now respects timezones

						$posttime = JFactory::getDate($postevent->getUnixStartTime() - $this->leadtime)->toISO8601();
						//echo "posting event at post time = ".$posttime."<br/>";
						//var_dump($data);
						//echo "<br/>";
						//exit();

						$this->postStatusMessage($id, $posttime, $postevent->title(), 1, $url, $image_url);
						//AutotweetAPI::postMessage($id, $data, "JEvents Plugin");
					}
					else if ($this->whentopost == 2)
					{

						// post 1 day before close of event registrations
						$db = JFactory::getDbo();
						static $hasRsvpPro;
						if (!isset($hasRsvpPro)) {
							// RSVP Pro
							$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_rsvppro' AND type='component' ");
							$hasRsvpPro = $db->loadResult();
							if (is_null($hasRsvpPro)) $hasRsvpPro = false;
						}
						// RSVP Pro doesn't create this repeatedly in the same request to 'cache' the information here
						static $regCloseData;
						if (!isset($regCloseData)){
							$regCloseData = isset($postevent->rsvpdata->regclose) ? $postevent->rsvpdata->regclose : false;
						}
						if (!$hasRsvpPro ){
							// fall back to event start date
							$posttime = JFactory::getDate($postevent->getUnixStartTime() - $this->leadtime)->toISO8601();
							$this->postStatusMessage($id, $posttime, $postevent->title(), 1, $url, $image_url);
						}
						else {
							$regclose = new JevDate(JevDate::strtotime($regCloseData));
							$regclose = $regclose->toUnix();
							$eventstart = isset($postevent->_fixed_dtstart)? $postevent->_fixed_dtstart : $postevent->dtstart();
							$repeatstart = $postevent->getUnixStartTime();

							$adjustedregclose = new JevDate($regclose + ($repeatstart - $eventstart));

							$posttime = $adjustedregclose->sub(new DateInterval("PT".$this->leadtime."S"))->toISO8601();
							$this->postStatusMessage($id, $posttime, $postevent->title(), 1, $url, $image_url);
						}
					}
					else {

					}
				}
			}
			else
			{
				$postevent = $this->jevent;

				// attach location etc.
				
				JFactory::getApplication()->triggerEvent('onDisplayCustomFields', array(&$postevent));

				if (isset($postevent->_imageurl1))
				{
					$image_url = $postevent->_imageurl1;
				}
				else
				{
					$image_url = '';
				}
				$url = $postevent->viewDetailLink($postevent->yup(), $postevent->mup(), $postevent->dup(), true);

				/*
				 * $data = array (
				 * 	'title'	=> 'Title of the article, event, forum post, ...',
				 * 	'text'	=> 'Text for the message.',
				 * 	'hashtags'	=> 'hashtags (They are inserted automatically by AutoTweet for channels supporting hashtags.',
				 * 	'url'		=> 'The URL of the article. (not shortened)'
				 * 	'image_url'	=> 'URL for the image to show (if there is an image)'
				 * 	'fulltext'	=> 'Enhanced text for the description (e. g. Facebook). Not available for all channels.'
				 * 	'catids'	=> 'array with category, container, etc. ids for the article, post...'
				 * 	'author'	=> 'creator or editor of the article'
				 * 	'event'	=> array (
				 * 		'location'		=> 'Location of the event'
				 * 		'street'		=> 'Street is optional'
				 * 		'city'			=> 'Must be a valid City for Facebook'
				 * 		'privacy'		=> 'OPEN, CLOSED or SECRET'
				 * 		'start_time'	=> 'yyyy-mm-dd hh:ss'
				 * 		'end_time'	=> 'yyyy-mm-dd hh:ss'
				 * 	);
				 * );
				 */


				$data = array();
				$data["title"] = $postevent->title();
				$data["text"] = $postevent->content();
				$data["url"] = $url;
				$data["image_url"] = $image_url;
				$data["catids"] = array($postevent->catid());
				$data["author"] = $postevent->creatorName();
				$data["event"] = array();
				$data["event"]["location"] = $postevent->location();
				$data["event"]["street"] = "";
				$data["event"]["city"] = "";
				$data["event"]["privacy"] = "OPEN";

				// http://developers.facebook.com/blog/post/2012/08/01/platform-migration--events-timezone-support/
				$data["event"]["start_time"] = JFactory::getDate($postevent->getUnixStartTime())->toISO8601();
				$data["event"]["end_time"] = JFactory::getDate($postevent->getUnixEndTime())->toISO8601();

				// post immediately
				$now = new JevDate();
				$posttime0 = $now->toISO8601();
				// post "1 day" before start of the event
				//$posttime1 = strftime('%Y-%m-%d %H:%M:%S', $postevent->getUnixStartTime() - $this->leadtime);
				// Joocial now respects timezones
				$posttime1 = JFactory::getDate($postevent->getUnixStartTime() - $this->leadtime )->toISO8601();

				// Before registrations close!
				$posttime2 = 0;
				if (isset($postevent->rsvpdata)){
					$posttime2 = JFactory::getDate($postevent->rsvpdata->regclose)->sub(new DateInterval("PT".$this->leadtime."S"))->toISO8601();
				}

				// Either post NOW or if modifying repeat and already past the leadtime then post NOW
				if ($this->whentopost == 0)
				{
					// post immediately
					$posttime  = $posttime0;
				}
				else if ($this->whentopost ==1)
				{
					// if post time is in the past
					if ($posttime0 > $posttime1 && $isChangedRepeat){
						// post immediately
						$posttime  = $posttime0;
					}
					else {
						// post "1 day" before start of the event
						$posttime  = $posttime1;
					}
				}
				else if ($this->whentopost ==2)
				{
					// post "1 day" before close of registrations
					$posttime  = $posttime2;

				}

				//echo "posting event at post time 2 = ".$posttime."<br/>";
				//var_dump($data);
				//echo "<br/>";
				//exit();
				if ($isChangedRepeat){
					// if a changed repeat then the status message needs to be the correct type
					$this->postStatusMessage($postevent->rp_id(), $posttime, $postevent->title(), 1, $url, $image_url);
				}
				else {
					$this->postStatusMessage($postevent->ev_id(), $posttime, $postevent->title(), 0, $url, $image_url);
				}

			}
		}

	}

	/**
	 * Function checks, if category is valid due settings restrictions
	 *
	 * @param $catid Category ID to check
	 * @return true if category accepted, false if denied by rules
	 */
	private function checkCategoryValidity($catid)
	{
		$acc_cat = is_array($this->categories) ? $this->categories : explode(',', str_replace(' ', '', $this->categories));
		$excl_cat = is_array($this->excluded_categories) ? $this->excluded_categories : explode(',', str_replace(' ', '', $this->excluded_categories));

		$isCategoryIncluded = true;
		if ($this->categories != '')
		{
			if (!in_array($catid, $acc_cat))
			{
				$isCategoryIncluded = false;
			}
		}

		$isCategoryExcluded = false;
		if ($this->excluded_categories != '')
		{
			if (in_array($catid, $excl_cat))
			{
				$isCategoryExcluded = true;
			}
		}

		if ($isCategoryIncluded && !$isCategoryExcluded)
			return true;
		else
			return false;

	}

	/**
	 * Function checks, if user is valid due settings restrictions
	 *
	 * @param $userid User ID to check
	 * @return true if user accepted, false if denied by rules
	 */
	private function checkUserValidity($userid)
	{
		$acc_usr = explode(',', str_replace(' ', '', $this->users));
		$excl_usr = explode(',', str_replace(' ', '', $this->excluded_users));

		$isUserIncluded = true;
		if ($this->users != '')
		{
			if (!in_array($catid, $acc_cat))
			{
				$isUserIncluded = false;
			}
		}

		$isUserExcluded = false;
		if ($this->excluded_users != '')
		{
			if (in_array($catid, $excl_cat))
			{
				$isUserExcluded = true;
			}
		}

		if ($isUserIncluded && !$isUserExcluded)
			return true;
		else
			return false;

	}

	/**
	 * Implemented abstract method to get data for tweet.
	 * Database used because object is recreated and method called
	 * from frontend later.
	 *
	 * @param $id ID of the event
	 * @param $typeinfo not used, only one type here
	 * @return array of mandatory parameters (title, text, hashtags, url)
	 */
	public function getData($id, $typeinfo)
	{

		include_once(JPATH_SITE . "/components/com_jevents/jevents.defines.php");

		$params = JComponentHelper::getParams(JEV_COM_COMPONENT);
		// See http://www.php.net/manual/en/timezones.php
		$tz=$params->get("icaltimezonelive","");
		if ($tz!="" && is_callable("date_default_timezone_set")){
			$timezone= date_default_timezone_get();
			date_default_timezone_set($tz);
			$registry	= JRegistry::getInstance("jevents");
			$registry->set("jevents.timezone",$timezone);
		}

		// get the data and query models
		$dataModel = new JEventsDataModel();
		$queryModel = new JEventsDBModel($dataModel);

		// This is a repeat!
		if ($typeinfo == 1)
		{
			// get event by repeat id
			$this->jevent = $queryModel->listEventsById(intval($id), 1, "icaldb");
		}
		else
		{
			// get event by event id
			$this->jevent = $queryModel->getEventById(intval($id), 1, "icaldb");
		}
		if (!$this->jevent)
		{
			$data = array ('is_valid' => false);
			return $data;
		}

		// process the new plugins
		JPluginHelper::importPlugin("jevents");
		
		JFactory::getApplication()->triggerEvent('onGetEventData', array(& $this->jevent));

		JFactory::getApplication()->triggerEvent('onDisplayCustomFields', array(& $this->jevent));

		/*
		  if (isset($this->jevent->_jevtags) && is_array($this->jevent->_jevtags) &&  count($this->jevent->_jevtags)>0){
		  $tags = array();
		  foreach ($this->jevent->_jevtags as $tag){
		  $tags[] = $tag->title;
		  }
		  }
		 */

		if ($this->show_info == 1)
		{
			$info = $this->info_format;
			$info = str_replace("%C", $this->jevent->getCategoryName(), $info);
			if (isset($this->jevent->_jevlocation))
			{
				$info = str_replace("%L", $this->jevent->_jevlocation->title, $info);
			}
			else
			{
				$info = str_replace("%L", $this->jevent->location(), $info);
			}
			$info = str_replace("%D", $this->eventDateFormat($this->jevent), $info);
		}

		$msgtext = "";

		if ($this->show_info == 1 && $this->info_position == 0)
		{
			$msgtext .= $info . " ";
		}

		if ($this->add_description == 1)
		{
			$msgtext .= $this->jevent->title() . " - " . substr(strip_tags($this->jevent->content()), 0, $this->add_description_count);
		}
		else
		{
			$msgtext .= $this->jevent->title();
		}

		if ($this->show_info == 1 && $this->info_position == 1)
		{
			$msgtext .= $info;
		}

		//$msgtext .= "(".implode(",",$tags).")";

		$Itemid = JEVHelper::getItemid($this->jevent);
		$Itemid  = $this->params->get("target_itemid", $Itemid);

		$url = str_replace('&amp;', '&', $this->jevent->viewDetailLink($this->jevent->yup(), $this->jevent->mup(), $this->jevent->dup(), false, $Itemid));

		$postevent = $this->jevent;
		$data = array ('is_valid' => true);

		if (isset($postevent->_imageurl1))
		{
			$image_url = $postevent->_imageurl1;
		}
		else
		{
			$image_url = '';
		}

		$data["title"] = $postevent->title();
		$data["text"] = $msgtext;
		$data["fulltext"] = $postevent->content();
		$data["url"] = $url;
		$data["image_url"] = $image_url;
		$data["catids"] = array($postevent->catid());
		$data["author"] = $postevent->creatorName();
		$data["event"] = array();
		$data["event"]["location"] = $postevent->location();
		$data["event"]["street"] = "";
		$data["event"]["city"] = "";
		$data["event"]["privacy"] = "OPEN";

		$data["event"]["start_time"] = $posttime = strftime('%Y-%m-%d %H:%M', $postevent->getUnixStartTime());
		$data["event"]["end_time"] = $posttime = strftime('%Y-%m-%d %H:%M', $postevent->getUnixEndTime());

		$startdate = new JevDate($postevent->getUnixStartTime());
		$data["event"]["start_time"] = $posttime = $startdate->toFormat('%Y-%m-%d %H:%M');
		$enddate = new JevDate($postevent->getUnixEndTime());
		$data["event"]["end_time"] = $posttime = $enddate->toFormat('%Y-%m-%d %H:%M');

		// http://developers.facebook.com/blog/post/2012/08/01/platform-migration--events-timezone-support/
		$data["event"]["start_time"] = JFactory::getDate($postevent->getUnixStartTime())->toISO8601();
		$data["event"]["end_time"] = JFactory::getDate($postevent->getUnixEndTime())->toISO8601();

		// Must reset the timezone back!!
		if ($tz && is_callable("date_default_timezone_set")){
			date_default_timezone_set($timezone);
		}

		return $data;

	}

	/**
	 * Function to get date string format
	 *
	 * @param $dtstart Start date of the event
	 * @param $dtend End date of the event
	 * @return formated string with the start and the end date
	 */
	private function eventDateFormat($event)
	{
		if ($event->getUnixStartDate() == $event->getUnixEndDate())
		{
			if (version_compare(JVERSION, "1.6.0", 'ge')){
				// use time otherwise we get midnight and then date format can change the date !
				$sdate =  new JevDate($event->publish_up());
				//$sdate =  JFactory::getDate($event->publish_up());
				return $sdate->format($this->date_format, true);
			}
			else {
				return date($this->date_format,$event->getUnixStartDate());
			}
		}
		else
		{
			if (version_compare(JVERSION, "1.6.0", 'ge')){
				//$sdate = new JevDate($event->getUnixStartDate());
				//$edate = new JevDate($event->getUnixEndtDate());
				//$sdate =  JFactory::getDate($event->getUnixStartDate());
				//$edate =  JFactory::getDate($event->getUnixEndDate());
				// use time otherwise we get midnight and then date format can change the date !
				$sdate =  new JevDate($event->publish_up());
				$edate =  new JevDate($event->publish_down());
				//$sdate =  JFactory::getDate($event->publish_up());
				//$edate =  JFactory::getDate($event->publish_down());
				return $sdate->format($this->date_format, true) . "->" . $edate->format($this->date_format, true);
			}
			else {
				return date($this->date_format,$event->getUnixStartDate()) . "->" . date($this->date_format,$event->getUnixEndDate()) ;
			}
		}

	}

}

?>