<?php

defined('JPATH_BASE') or die();

use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Factory;
use Joomla\String\StringHelper;

jimport('joomla.application.component.helper');
jimport('joomla.application.component.model');
jimport('joomla.html.parameter');

JLoader::register('JEVHelper', JPATH_SITE . "/components/com_jevents/libraries/helper.php");

// Based on joomla core model with added security filters on section, category and article access levels

/**
 * Content Component Article Model
 *
 * @package		Joomla
 * @subpackage	Content
 * @since		1.5
 */
class contentModelFrontelement extends BaseDatabaseModel
{

	/**
	 * Content data in category array
	 *
	 * @var array
	 */
	var
		$_list = null;
	var
		$_page = null;

	/**
	 * Method to get content article data for the frontpage
	 *
	 * @since 1.5
	 */
	function getList()
	{
		$mainframe = Factory::getApplication();  // RSH 11/09/10 - Make J!1.6 compatible

		$user = Factory::getUser();

		if (!empty($this->_list))
		{
			return $this->_list;
		}

		// Initialize variables
		$db =  $this->getDBO();
		$filter = null;

		// Get some variables from the request
		$sectionid = Factory::getApplication()->input->getInt('sectionid', -1);
		$redirect = $sectionid;
		$option = Factory::getApplication()->input->getCmd('option');
		$filter_order = $mainframe->getUserStateFromRequest('articleelement.filter_order', 'filter_order', '', 'cmd');
		$filter_order_Dir = $mainframe->getUserStateFromRequest('articleelement.filter_order_Dir', 'filter_order_Dir', '', 'word');
		$catid = $mainframe->getUserStateFromRequest('articleelement.catid', 'catid', 0, 'int');
		$filter_authorid = $mainframe->getUserStateFromRequest('articleelement.filter_authorid', 'filter_authorid', 0, 'int');
		$filter_sectionid = $mainframe->getUserStateFromRequest('articleelement.filter_sectionid', 'filter_sectionid', -1, 'int');
		$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
		$limitstart = Factory::getApplication()->input->getInt('limitstart', 0);
		$search = $mainframe->getUserStateFromRequest('articleelement.search', 'search', '', 'string');
		$search = StringHelper::strtolower($search);

		//$where[] = "c.state >= 0";
		$where[] = "c.state != -2";

		if (!$filter_order)
		{
			if (!version_compare(JVERSION, "1.6.0", 'ge'))
			{
				$filter_order = 'section_name';
			}
			else
			{
				$filter_order = 'c.title';
			}
		}
		if (version_compare(JVERSION, "1.6.0", 'ge'))
		{
			$order = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir . ', cc.title, c.ordering';
		}
		else
		{
			$order = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir . ', section_name, cc.name, c.ordering';
		}
		$all = 1;

		if ($filter_sectionid >= 0 && !version_compare(JVERSION, "1.6.0", 'ge'))
		{
			$filter = ' WHERE cc.section = ' . $db->Quote($filter_sectionid);
		}
		$section = new stdClass();
		$section->title = 'All Articles';
		$section->id = 0;

		/*
		 * Add the filter specific information to the where clause
		 */
		// Section filter
		if ($filter_sectionid >= 0 && !version_compare(JVERSION, "1.6.0", 'ge'))
		{
			$where[] = 'c.sectionid = ' . (int) $filter_sectionid;
		}
		// Category filter
		if ($catid > 0)
		{
			$where[] = 'c.catid = ' . (int) $catid;
		}
		// Author filter
		if ($filter_authorid > 0)
		{
			$where[] = 'c.created_by = ' . (int) $filter_authorid;
		}

		// Only published articles
		$where[] = 'c.state = 1';

		// Keyword filter
		if ($search)
		{
			$where[] = 'LOWER( c.title ) LIKE ' . $db->Quote('%' . $db->escape($search, true) . '%', false);
		}

		if (version_compare(JVERSION, "1.6.0", 'ge'))
		{
			include_once(JPATH_SITE . "/components/com_jevents/jevents.defines.php");
			$aid = JEVHelper::getAid($user);
			$where[] = " cc.access IN (" . $aid . ")";
			$where[] = " c.access IN (" . $aid . ")";
		}
		else
		{
			$where[] = " cc.access<=" . $user->aid;
			$where[] = " s.access<=" . $user->aid;
			$where[] = " c.access<=" . $user->aid;
		}

		$plugin =  PluginHelper::getPlugin('jevents', 'jevcck');
		$params = new Registry($plugin->params);
		$onlyown = $params->get("onlyown", 0);
		if ($onlyown)
		{
			$where[] = " (c.created_by=" . $user->id . " OR c.modified_by=" . $user->id . ")";
		}

		// Build the where clause of the content record query
		$where = (count($where) ? ' WHERE ' . implode(' AND ', $where) : '');

		// Get the total number of records
		$query = 'SELECT COUNT(*)' .
			' FROM #__content AS c' .
			' LEFT JOIN #__categories AS cc ON cc.id = c.catid' .
			(version_compare(JVERSION, "1.6.0", 'ge') ? '' : ' LEFT JOIN #__sections AS s ON s.id = c.sectionid' ) .
			$where;
		$db->setQuery($query);
		$total = $db->loadResult();

		// Create the pagination object
		jimport('joomla.html.pagination');
		$this->_page = new Pagination($total, $limitstart, $limit);

		// Get the articles
		$query = 'SELECT c.*, cc.title as cctitle, u.name AS editor, f.content_id AS frontpage, v.name AS author ' .
			(version_compare(JVERSION, "1.6.0", 'ge') ? '' : ', s.title AS section_name, g.name AS groupname ' ) .
			' FROM #__content AS c' .
			' LEFT JOIN #__categories AS cc ON cc.id = c.catid' .
			(version_compare(JVERSION, "1.6.0", 'ge') ? '' : ' LEFT JOIN #__sections AS s ON s.id = c.sectionid' ) .
			(version_compare(JVERSION, "1.6.0", 'ge') ? '' : ' LEFT JOIN #__groups AS g ON g.id = c.access' ) .
			' LEFT JOIN #__users AS u ON u.id = c.checked_out' .
			' LEFT JOIN #__users AS v ON v.id = c.created_by' .
			' LEFT JOIN #__content_frontpage AS f ON f.content_id = c.id' .
			$where .
			$order;
		$db->setQuery($query, $this->_page->limitstart, $this->_page->limit);
		$this->_list = $db->loadObjectList();

		return $this->_list;

	}

	function getPagination()
	{
		if (is_null($this->_list) || is_null($this->_page))
		{
			$this->getList();
		}
		return $this->_page;

	}

}
