<?php

use Joomla\CMS\Language\Language;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Factory;


/**
 * @copyright	Copyright (C) 2015-JEVENTS_COPYRIGHT GWESystems Ltd. All rights reserved.
 * @license		By negotiation with author via http://www.gwesystems.com
 */

function gwejson_skiptoken()
{
	// Skip token for non-logged in users
	$user = Factory::getUser();

	if ($user->get('guest') === 1)
	{
		return true;
	}
}
function ProcessJsonRequest(&$requestObject, $returnData)
{
	$debug = false;
	$conf = Factory::getConfig();
	$joomladebug = $conf->get('debug');

	if ($debug || $joomladebug)
	{
		Log::addLogger(array('text_file' => 'getfloatdata.php'), Log::ALL, array('float'));
		Log::add("*** New run *** ", Log::INFO, 'float');

		$db = Factory::getDbo();

	}

	list($usec, $sec) = explode(" ", microtime());
	$starttime = (float) $usec + (float) $sec;

	JLoader::register('JevIsotope', JPATH_LIBRARIES . "/jevents/jevisotope/jevisotope.php");

	ini_set("display_errors", 0);
	$returnData->eventdata = false;

	// we can't rely on the language filter plugin to do this for us since it is executed later
	if (isset($requestObject->lang)) {
		$language_new = Language::getInstance($requestObject->lang);
		Factory::$language = $language_new;
		Factory::getApplication()->loadLanguage($language_new);
	}
	$lang = Factory::getLanguage();
	$lang->load("com_jevents", JPATH_SITE);
	$lang->load("com_jevents", JPATH_ADMINISTRATOR);

	include_once JPATH_SITE . "/components/com_jevents/jevents.defines.php";

	// TODO need to take account of Menu item constraints, logged in status etc. and filters !!!

	if (!JEVHelper::isEventCreator())
	{
		//	PlgSystemGwejson::throwerror(Text::_sprintf('JEW_GWE_JSON_NOT_CREATOR'));
	}

	if (!isset($requestObject->task) || !isset($requestObject->params)  || !isset($requestObject->Itemid))
	{
		PlgSystemGwejson::throwerror(Text::_sprintf('JEV_GWE_JSON_INVALID_TASK'));
	}

	// Set the Itemid for the request
	Factory::getApplication()->input->set("Itemid", intval( $requestObject->Itemid));

	$menu = Factory::getApplication()->getMenu();
	$menu->setActive(intval( $requestObject->Itemid));

	$menuparams = Factory::getApplication()->getMenu()->getParams(intval($requestObject->Itemid));

	if ($requestObject->filters){
		foreach ($requestObject->filters as $key => $val){
			Factory::getApplication()->input->set($key, $val);
		}
		if (isset($requestObject->filters->task) && $requestObject->filters->task!= ""){
			if ($requestObject->filters->task == "day.listevents" && $requestObject->task = "RangeData"){
				// reset the dates selected
				if (isset($requestObject->filters->year) && isset($requestObject->filters->month) && isset($requestObject->filters->day)){
					$requestObject->params[0] = $requestObject->params[1] = $requestObject->filters->year."-".$requestObject->filters->month."-".$requestObject->filters->day;
				}
			}
		}
	}

	JEVHelper::getFilterValues();

	// force menu parameters to be active
	$component =  ComponentHelper::getComponent("com_jevents");

	// handle global menu item parameter for com_showrepeats
	$com_showrepeats = $menuparams->get('com_showrepeats', "");
	if ($com_showrepeats === "-1" || $com_showrepeats === "")
	{
		$tempparams = ComponentHelper::getParams(JEV_COM_COMPONENT);
		$menuparams->set('com_showrepeats', $tempparams->get('com_showrepeats'));
	}

	// Override the global params if menu params have these values
	foreach($menuparams->getIterator() as $key => $val) {
		$component->params->set($key,$val);
	}
	$params = ComponentHelper::getParams("com_jevents");

	$float_columns = $params->get('float_columns', 4);

	$iso_params = array('microdata' => 1);

	$db = Factory::getDbo();
	$dataModel = new JEventsDataModel();
	$dataModel->setupComponentCatids();

	$r = 0;
	// Ok lets go get this data!
	if (is_callable(array($dataModel, "get" . $requestObject->task)))
	{
		$method = "get" . $requestObject->task;
		$returnData->ev_pdata = "";

		// TODO should really have try/catch around this!
		$returnData->ev_data = call_user_func_array(array($dataModel, $method), $requestObject->params);

		// Hard coded params so not ideal
		$limitstart = $requestObject->params[3];
		if ($returnData->ev_data["total"] <= $limitstart){
			$returnData->ev_data['rows'] = array();
		}

		$numEvents = count($returnData->ev_data['rows']);

		if ($numEvents >= 1)
		{
			$document = Factory::getDocument();
			// Generate script file links
			$headData = array();

			foreach ($document->_scripts as $strSrc => $strAttr)
			{
				$strSrc = str_replace(STRIPPATH, "", $strSrc);
				if (!isset($headData["scriptfiles"])){
					$headData["scriptfiles"] = array();
				}
				$headData["scriptfiles"][] = array("strSrc"=>$strSrc, "strAttr"=>$strAttr);
			}

			// Generate script declarations
			foreach ($document->_script as $type => $content)
			{
				if (!isset($headData["scripts"])){
					$headData["scripts"] = '';
				}
				$headData["scripts"] .= $content;
			}

			// Generate script language declarations.
			if (count(Text::script()))
			{
				$lnEnd = $document->_getLineEnd();
				$tab = $document->_getTab();
				$lang = $tab . $tab . '(function() {' . $lnEnd;
				$lang .= $tab . $tab . $tab . 'var strings = ' . json_encode(Text::script()) . ';' . $lnEnd;
				$lang .= $tab . $tab . $tab . 'if (typeof Joomla == \'undefined\') {' . $lnEnd;
				$lang .= $tab . $tab . $tab . $tab . 'Joomla = {};' . $lnEnd;
				$lang .= $tab . $tab . $tab . $tab . 'Joomla.Text = strings;' . $lnEnd;
				$lang .= $tab . $tab . $tab . '}' . $lnEnd;
				$lang .= $tab . $tab . $tab . 'else {' . $lnEnd;
				$lang .= $tab . $tab . $tab . $tab . 'Joomla.Text.load(strings);' . $lnEnd;
				$lang .= $tab . $tab . $tab . '}' . $lnEnd;
				$lang .= $tab . $tab . '})();' . $lnEnd;

				if (!isset($headData["scripts"])){
					$headData["scripts"] = "";
				}
				$headData["scripts"] .= $lang;

			}
			$returnData->headData = $headData;

			// Load JEvents Config Items
			$cfg = ComponentHelper::getParams('com_jevents');

			// Create a new query object.
			$query = $db->getQuery(true);

			//Block to load:
			$blocktl = "icalevent.list_block" . $cfg->get('float_style', 1);
			// Select all records from the user profile table where key begins with "custom.".
			// Order it by the ordering field.
			$query->select($db->quoteName(array('title', 'name', 'value', 'state')));
			$query->from($db->quoteName('#__jev_defaults'));
			$query->where($db->quoteName('name') . ' = '. $db->quote($blocktl) . ' AND ' . $db->quoteName('state') . ' = 1');

			$db->setQuery($query);

			$results = $db->loadObject();

			$jeventsUIkit = $cfg->get("floatuikit", 1);

			if($results && $results->value !== '') {
				$item_bodycore = $results->value;
			} else {
				$floatStyle = $cfg->get('float_style', 1);
				if ($floatStyle && !$jeventsUIkit)
				{
					$floatStyle .= "uikit";
				}
				$item_bodycore = file_get_contents(JEV_PATH . "/views/float/defaults/icalevent.list_block" . $floatStyle . ".html");
			}

			if ($jeventsUIkit)
			{
				$item_bodycore = str_replace("uk-", "gsl-" , $item_bodycore);
			}

			include_once(JEV_VIEWS."/float/abstract/abstract.php");
			include_once(JEV_VIEWS."/float/range/view.html.php");

			$view = new FloatViewRange(array());

			if (Factory::getApplication()->input->get("option")!=="com_jevents"){
				Factory::getApplication()->input->set("option","com_jevents");
			}

			$r = 0;
			foreach ($returnData->ev_data['rows'] as $row)
			{
				// special code to insert modules once every 5 cells e.g. for adverts

				if ($r > 0 && $r % 5 === 0)
				{

					$module = Jhtml::_('content.prepare', '{loadposition jevents_float_advert}');
					if ($module && $module !== '' && $module !== '{loadposition jevents_float_advert}') {
						$returnData->ev_pdata .= '<div class="jeviso_item w' . $cfg->get('float_columns', 4) . ' style' . $cfg->get('float_style', 1) . '">' . $module . '</div>';


					}
				}

				$row = $returnData->ev_data['rows'][$r];

				$item_bodycore = JevIsotope::itembody($row, $iso_params, "", "", $view, $blocktl);
				$item_body = '<div id="jeviso_item' . $row->rp_id() . '" class="jeviso_item w' . $cfg->get('float_columns', 4) . ' ' . $row->catname() . ' style' . $cfg->get('float_style', 1) . '">';
				$item_body .= $item_bodycore;
				$item_body .= '</div>';
				$returnData->ev_pdata .= $item_body;

				$r++;
			}
		}
		else
		{
			$returnData->no_more = 1;
		}

		$returnData->returned_count = $r;

		// Day Data fetching request
		if (isset($requestObject->task) == "DayData")
		{
			$returnData->test = 'DayData';
		}

	}



	list ($usec, $sec) = explode(" ", microtime());
	$time_end = (float) $usec + (float) $sec;
	//  $returnData->datatiming =  "getFloatData time = ".round($time_end - $starttime, 4);

	if ($debug || $joomladebug){
		$log = version_compare(JVERSION, '4.0.0', 'gt') ? $db->getMonitor()->getLogs() : $db->getLog();
		if ($log){
			$timings = version_compare(JVERSION, '4.0.0', 'gt') ? $db->getMonitor()->getTimings() : $db->getTimings();

			$timing = array();
			$output = "";

			if (isset($timings[0]))
			{
				foreach ($log as $id => $query)
				{
					if (isset($timings[$id * 2 + 1]))
					{
						// Compute the query time: $timing[$k] = array( queryTime, timeBetweenQueries ).
						$timing[$id] =round(($timings[$id * 2 + 1] - $timings[$id * 2]) * 1000,2);
						// only log non-trivial queries - more than 5 milliseconds
						if ($timing[$id]>5){
							Log::add("\n".$timing[$id] ." : ".$query."\n", Log::INFO, 'float');
						}
					}
				}

			}

		}

	}

	return $returnData;

}
