<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2018 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://www.jevents.net
 */

defined('_JEXEC') or die( 'No Direct Access' );
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;

// Event repeat startdate fitler
class jevAfterdateFilter extends jevFilter
{

	var $dmap="";
	var $_date = "";

	function __construct($tablename, $filterfield, $isstring=true){
		$lang = Factory::getLanguage();
		$lang->load("plg_jevents_jevtimelimit", JPATH_SITE);
		$lang->load("plg_jevents_jevtimelimit", JPATH_ADMINISTRATOR);

		$this->fieldset=true;

		$this->valueNum=3;
		$this->filterNullValue=0;
		$this->filterNullValues[0]=1; // n/a, before, after
		$this->filterNullValues[1]=""; // the date
		$this->filterNullValues[2]=0; // true means the form is submitted

		$this->filterType="afterdate";
		$this->filterLabel="";
		$this->dmap = "rpt";
		parent::__construct($tablename,$filterfield, true);

		$plugin = PluginHelper::getPlugin("jevents","jevtimelimit");
		$this->params = new Registry($plugin ? $plugin->params: null);
		$this->dateformat = $this->params->get("dateformat","Y-m-d");
		$this->inputpopup = $this->params->get("inputpopup",1);

		/*
		// filter processing class takes care of memory w.r.t. filter module now.
		$this->filter_values[1] = Factory::getApplication()->input->getString($this->filterType.'_fvs1','');
		$this->filter_values[2] = Factory::getApplication()->input->getInt($this->filterType.'_fvs2',0);
		*/

		// Should these be ignored?
		$reg = Factory::getConfig();
		$modparams = $reg->get("jev.modparams",false);
		if ($modparams && $modparams->get("ignorefiltermodule",false)){
			$this->filter_values[1] = $this->filterNullValues[1];
			$this->filter_values[2] = $this->filterNullValues[2];
			$this->_date = $this->filter_values[1];
			return;

		}

		$this->_date = $this->filter_values[1];

	}

	function _createFilter($prefix=""){
		if (!$this->filterField ) return "";
		// first time visit
		if (isset($this->filter_values[2]) && $this->filter_values[2]==0) {
			$this->filter_values = array();
			$this->filter_values[0]=1;
			// default scenario is inactive
			$this->filter_values[1]="";
			return "";
		}
		else if ($this->filter_values[0]==1 && $this->filter_values[1]==""){
			return "";
		}
		$filter="";

		if ($this->_date!="" ){
			$parts = preg_split("/[\: \/\-]/",$this->_date );
			$parts = ArrayHelper::toInteger($parts);

			if (count($parts)<3) {
				$fulldate = date( 'Y-m-d H:i:s',strtotime("-5 seconds"));
				$this->filter_values[1]=substr($fulldate,0,10);
				$this->filter_values[2]=1;
				return  $this->dmap.".endrepeat>='$fulldate'";
			}
			else {
				$h = 0;
				$min = 0;
				switch ($this->dateformat)
				{
					case 'Y-m-d' :
						$y=$parts[0];
						$m=$parts[1];
						$d=$parts[2];
						break;
					case 'm/d/Y' :
						$y=$parts[2];
						$m=$parts[0];
						$d=$parts[1];
						break;
					case 'd/m/Y' :
					case 'd.m.Y' :
						$y=$parts[2];
						$m=$parts[1];
						$d=$parts[0];
						break;
					case 'Y-m-d H:i' :
						$y=$parts[0];
						$m=$parts[1];
						$d=$parts[2];
						$h=$parts[3];
						$min=$parts[4];
						break;
					case 'm/d/Y H:i' :
						$y=$parts[2];
						$m=$parts[0];
						$d=$parts[1];
						$h=$parts[3];
						$min=$parts[4];
						break;
					case 'd/m/Y H:i' :
					case 'd.m.Y H:i' :
						$y=$parts[2];
						$m=$parts[1];
						$d=$parts[0];
						$h=$parts[3];
						$min=$parts[4];
						break;
				}


				$fulldate = date( 'Y-m-d H:i:s', mktime($h, $min, 0, $m, $d, $y) );
				$this->filter_values[1] = $fulldate;
			}
			$date = $this->dmap.".endrepeat>='$fulldate'";
		}
		else {
			$date = "";
		}
		$filter = $date;

		return $filter;
	}

	function _createfilterHTML(){

		if (!$this->filterField) {
			return "";
		}

		$filterList = array();

		$filterList["title"] = Text::_("PLG_JEVENTS_JEVTIMELIMIT_ENDING_AFTER_LABEL");
		$filterList["html"] = "<input type='hidden' name='".$this->filterType."_fvs0' value='1'/>";

		$params = ComponentHelper::getParams( JEV_COM_COMPONENT );
		if(method_exists("JEVHelper", "getMinYear"))
		{
			$minyear =  JEVHelper::getMinYear();
			$maxyear = JEVHelper::getMaxYear();
		}
		else
		{
			$minyear = $params->get("com_earliestyear", 1970);
			$maxyear = $params->get("com_latestyear", 2150);
		}

		$parts = preg_split("/[\: \/\-]/",$this->filter_values[1] );
		$parts = ArrayHelper::toInteger($parts);
		if (count($parts)<3){
			$this->calendarValue = "";
		}
		else {
			$this->calendarValue = date($this->dateformat, strtotime($this->filter_values[1]));
		}

		$params = ComponentHelper::getParams(JEV_COM_COMPONENT);
		$minyear = JEVHelper::getMinYear();
		$maxyear = JEVHelper::getMaxYear();
		ob_start();
		?>
(function () {
		var filterType = '<?php echo $this->filterType; ?>';
		var dateval = document.getElementById(filterType + 'trigger_fv').value;
		var dateFormat = '<?php echo $this->dateformat; ?>';
		var dateselector = '.jevstartdate';
		if (dateval!=''){
			jQuery('.jevdatetask').prop('name', 'task');
			var myregexp = new RegExp('[: .\/-]');
			var parts = dateval.split(myregexp);
			
			switch (dateFormat)
			{
				case 'Y-m-d' :
					document.querySelector(dateselector).value =  parts[0]+'-'+parts[1]+'-'+parts[2];
					break;
				case 'm/d/Y' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[0]+'-'+parts[1];
					break;
				case 'd/m/Y' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[1]+'-'+parts[0];
					break;
				case 'd.m.Y' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[1]+'-'+parts[0];
					break;
				case 'Y-m-d H:i' :
					document.querySelector(dateselector).value =  parts[0]+'-'+parts[1]+'-'+parts[2]+' '+parts[3]+':'+parts[4];
					break;
				case 'm/d/Y H:i' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[0]+'-'+parts[1]+' '+parts[3]+':'+parts[4];
					break;
				case 'd/m/Y H:i' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[1]+'-'+parts[0]+' '+parts[3]+':'+parts[4];
					break;
				case 'd.m.Y H:i' :
					document.querySelector(dateselector).value =  parts[2]+'-'+parts[1]+'-'+parts[0]+' '+parts[3]+':'+parts[4];
					break;
			}
			document.getElementById(filterType + '_fvs1').value =  document.getElementById(filterType + 'trigger_fv').value;
		}
		else {

			jQuery('.jevdatetask').prop('name', 'xxxtask');
			document.querySelector(dateselector).value = '';
			document.getElementById(filterType + '_fvs1').value =  document.getElementById(filterType + 'trigger_fv').value = '';
		}
})();
		<?php
		$script =str_replace(array("\n","\t", "\r"), "", ob_get_clean());

		ob_start();

		$showtime = strpos($this->dateformat, ":i") ? 1 : 0;

		JEVHelper::loadElectricCalendar($this->filterType."trigger_fv", $this->filterType."trigger_fv", $this->calendarValue, $minyear, $maxyear, '', $script, $this->dateformat,
				//$this->inputpopup ? array('readonly'=>"readonly", "maxlength"=>"10", "size"=>"12") : array( "maxlength"=>"10", "size"=>"12")
			array( "maxlength"=>"10", "size"=>"12"),
			$showtime
		);

		$filterList["html"] .= ob_get_clean();

		$filterList["html"] .=  '<input type="hidden" name="'.$this->filterType.'_fvs1" id="'.$this->filterType.'_fvs1" value="'.$this->calendarValue.'"  readonly="readonly" maxlength="10" size="12"  />'."\n";

		$filterList["html"] .= "<input type='hidden' name='".$this->filterType."_fvs2' value='1'/>\n";
		$filterList["html"] .= "<input type='hidden' name='xxxtask' class='jevdatetask' value='range.listevents' />\n";
		$filterList["html"] .= "<input type='hidden' name='startdate' class='jevstartdate' value='".$this->filter_values[1]."' />\n";

		$script = "try {JeventsFilters.filters.push({id:'".$this->filterType."trigger_fv',value:''});} catch (e) {}\n";
		$script .= "try {JeventsFilters.filters.push({id:'".$this->filterType."_fvs1',value:''});} catch (e) {}\n";
		$script .= "try {JeventsFilters.filters.push({id:'".$this->filterType."_fvs2',value:1});} catch (e) {}\n";
		if ($this->filter_values[1] == ""){
			$script .= "jQuery(document).on('ready',function() {jQuery('#".$this->filterType."_fvs1').val('');})\n";
                        $script .= "jQuery(document).on('ready',function() {jQuery('#".$this->filterType."trigger_fv').val('');})\n";
		}
		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		return $filterList;


	}
}
