<?php
/**
* @copyright	Copyright (C) 2009 GWE Systems Ltd. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE
*/

// Set flag that this is a parent file
defined('JPATH_BASE') or die;

use Joomla\Registry\Registry;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;

function ProcessJsonRequest(&$requestObject, $returnData)
{
	if ($requestObject->error){
		return "Error";
	}

	$typeOfReCaptcha = $requestObject->type;

	switch($typeOfReCaptcha)
	{
		case "v2":
			$returnData = checkNoCaptchaV2($requestObject, $returnData);
			break;
		case "v1":
			$returnData = checkReCaptchaV1($requestObject, $returnData);
			break;
		default:
			$returnData->result = "error";
			break;
	}

	return $returnData;
}

function checkReCaptchaV1($requestObject, $returnData)
{
	require_once('anonuserlib/recaptcha.php');

	$private = false;
	$plugin = PluginHelper::getPlugin('jevents', 'jevanonuser');
	$params = new Registry( $plugin->params );

	$lang = Factory::getLanguage();
	$lang->load("plg_jevents_jevanonuser", JPATH_ADMINISTRATOR);

	$plugin = Factory::getApplication()->getParams()->get('captcha', Factory::getConfig()->get('captcha'));
	if ($plugin && (!$params->get("typeofcaptcha", '') || $params->get("typeofcaptcha", '') == "joomla" ))
	{
		$captcha = PluginHelper::getPlugin('captcha', $plugin);
		$captchaParams = new Registry( $captcha->params );
		$private = $captchaParams->get('private_key', false);
	}
	else
	{
		$private = $params->get("recaptchaprivate",false);
	}

	if($private)
	{
		$mainframe= Factory::getApplication('site');

		if (isset($requestObject->responseField) && trim($requestObject->responseField)!=="")
		{
			$response = recaptcha_check_answer($private,Factory::getApplication()->input->getString("REMOTE_ADDR","","server"), $requestObject->challengeField,$requestObject->responseField);
			if (!$response->is_valid)
			{
				$returnData->result = "error";
				$mainframe->setUserState("jevrecaptcha","error");
			}
			else
			{
				$returnData->result = "success";

				$mainframe->setUserState("jevrecaptcha","ok");
			}
		}
		else
		{
			$returnData->result = "error";
		}
	}
	else
	{
		$returnData->result = "error";
	}
	return $returnData;
}

function checkNoCaptchaV2($requestObject, $returnData)
{
	require_once('recaptchav2/jevrecaptchalib.php');

	$private = false;
	$plugin = PluginHelper::getPlugin('jevents', 'jevanonuser');
	$params = new Registry( $plugin->params );

	$lang = Factory::getLanguage();
	$lang->load("plg_jevents_jevanonuser", JPATH_ADMINISTRATOR);

	$plugin = Factory::getApplication()->getParams('com_jevents')->get('captcha', Factory::getConfig()->get('captcha'));
	if ($plugin && (!$params->get("typeofcaptcha", '') || $params->get("typeofcaptcha", '') == "joomla" ))
	{
		$captcha = PluginHelper::getPlugin('captcha', $plugin);
		$captchaParams = new Registry( $captcha->params );
		$private = $captchaParams->get('private_key', false);
	}
	else
	{
		$private = $params->get("recaptchav2private",false);
	}

	if($private)
	{
		$mainframe= Factory::getApplication('site');

		if (isset($requestObject->responseField) && trim($requestObject->responseField)!=="")
		{
			$noCaptcha = new JevReCaptcha($private);
			$response =$noCaptcha->verifyResponse(Factory::getApplication()->input->getString("REMOTE_ADDR","","server"), $requestObject->responseField);
			
			if (!$response->is_valid)
			{
				$returnData->result = "error";
				$mainframe->setUserState("jevrecaptcha","error");
			}
			else
			{
				$returnData->result = "success";

				$mainframe->setUserState("jevrecaptcha","ok");
			}
		}
		else
		{
			$returnData->result = "error";
		}
	}
	else
	{
		$returnData->result = "error";
	}
	return $returnData;

}