<?php

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;


/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: helper.php 1893 2011-04-08 09:01:06Z geraintedwards $
 * @package     JEvents
 * @subpackage  Module JEvents Calendar
 * @copyright   Copyright (C) 2006-2017 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://joomlacode.org/gf/project/jevents
 */
// no direct access
defined ( '_JEXEC' ) or die ( 'Restricted access' );

class modJeventsCategoriesHelper {

	private $params;
	 
	public function  __construct($params) {
		// setup for all required function and classes
		$file = JPATH_SITE . '/components/com_jevents/mod.defines.php';
		if (file_exists ( $file )) {
			include_once ($file);
			include_once (JEV_LIBS . "/modfunctions.php");

		} else {
			die ( "JEvents Calendar\n<br />This module needs the JEvents component" );
		}
                jimport('joomla.utilities.date');
		// load language constants
		JEVHelper::loadLanguage ( 'modcal' );

		$this->params = $params;
	}
	
	public function getAllCats($modparams, &$catids, &$catidList) {

		if (version_compare(JVERSION, "1.6.0", 'ge')){
			$db = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select('a.id');
			$query->from('#__categories AS a');
			$query->where('a.parent_id > 0');

			// Filter on extension.
			$query->where('a.extension = "com_jevents"');
			$query->where('a.published = 1');
			$query->where('a.language in (' . $db->quote(Factory::getLanguage()->getTag()) . ',' . $db->quote('*') . ')');
			$query->order('a.lft');

			$db->setQuery($query);
			$catids = $db->loadColumn();
			$catidList = implode(",",$catids);
			return;
		}
		
		$catidList = "";
		// New system
		$newcats = $modparams->get( "catidnew", false);
		if ($newcats && is_array($newcats )){
			foreach ($newcats as $newcat){
				if ( !in_array( $newcat,$catids )){
					$catids[]=$newcat;
					$catidList .= (strlen($catidList)>0?",":"").$newcat;
				}
			}				
		}
		else {			
			for($c = 0; $c < 999; $c ++) {
				$nextCID = "catid$c";
				//  stop looking for more catids when you reach the last one!
				if (! $modparams->get ( $nextCID, false ))
					break;
				if ($modparams->get ( $nextCID ) > 0 && ! in_array ( $modparams->get ( $nextCID ), $catids )) {
					$catids [] = $modparams->get ( $nextCID );
					$catidList .= (strlen ( $catidList ) > 0 ? "," : "") . $modparams->get ( $nextCID );
				}
			}
		}
		
	}
	
	public function countEventsByCat($allrows) {
		$catids  = array();
		$this->extractCatidFromTree($allrows, $catids);
		//$catids = implode(",",array_keys($allrows));
		if (count($catids)==0) return 0;
		$catids = implode(",",$catids);
		$db = Factory::getDbo ();

	    list($startdate, $enddate ) = $this->getStartEndDates();
	
	    // Note that using a $limit value of -1 the limit is ignored in the query
	   	$cfg = JEVConfig::getInstance();
		$counter = $this->countIcalEventsByRange( $catids, $startdate,$enddate,  $cfg->get('com_showrepeats'));

		return $counter;
	}

	private function extractCatidFromTree($allrows,&$catids){
		foreach ($allrows as $k => $v){
			if (isset($v->subcats) && is_array($v->subcats)  && count($v->subcats)>0){
				$this->extractCatidFromTree($v->subcats, $catids);
			}
			$catids[]=$k;
		}
	}
	
	// Allow the passing of filters directly into this function for use in 3rd party extensions etc.
	public function countIcalEventsByRange($catids, $startdate, $enddate, $showrepeats = 1) {
		$user = Factory::getUser();
		$aid = JEVHelper::getAid($user);
		list($year, $month, $day) = explode('-', $startdate);
		list($thisyear, $thismonth, $thisday) = JEVHelper::getYMD();

		$startdate = mktime( 0, 0, 0, intval($month),intval($day),intval($year) );
		$startdate = strftime('%Y-%m-%d',$startdate);

		if (strlen($startdate)==10) $startdate.= " 00:00:00";
		if (strlen($enddate)==10) $enddate.= " 23:59:59";

		$db	= Factory::getDbo();
		$lang = Factory::getLanguage();
		$langtag = $lang->getTag();
		$user = Factory::getUser();
		
		// process the new plugins
		// get extra data and conditionality from plugins
		$extrawhere =array();
		$extrajoin = array();
		$extrafields = "";  // must have comma prefix
		$extratables = "";  // must have comma prefix
		$needsgroup = false;

		$filters = jevFilterProcessing::getInstance(array("published","justmine","search"));
		$filters->setWhereJoin($extrawhere,$extrajoin);
		$needsgroup = $filters->needsGroupBy();


		Factory::getApplication()->triggerEvent('onListIcalEvents', array (& $extrafields, & $extratables, & $extrawhere, & $extrajoin, & $needsgroup));
				

		$params = ComponentHelper::getParams("com_jevents");
		if ($params->get("multicategory",0)){
			$extrajoin[] = "\n #__jevents_catmap as catmap ON catmap.evid = rpt.eventid";
			$extrajoin[] = "\n #__categories AS catmapcat ON catmap.catid = catmapcat.id";
			$extrawhere[]= " catmapcat.access " . (version_compare(JVERSION, '1.6.0', '>=') ? ' IN (' . $aid . ')' : ' <=  ' . $aid);
			
			// TODO check this - it should not be just the top level categories from the tree!
			$extrawhere[]= " catmap.catid IN(" . $catids . ")";

			$extrajoin = ( count( $extrajoin  ) ?  " \n LEFT JOIN ". implode( " \n LEFT JOIN ", $extrajoin ) : '' );
			$extrawhere = ( count( $extrawhere ) ? ' AND '. implode( ' AND ', $extrawhere ) : '' );
			
                        $showrepeats = $this->params->get('countrepeats', 1);
			if ($showrepeats == 0){
				$query = "SELECT count(distinct ev.ev_id) as evcount, catmap.catid";
			}
			else {
				$query = "SELECT count(distinct rpt.rp_id) as evcount, catmap.catid";
			}
			$query .= "\n FROM #__jevents_repetition as rpt"
			. "\n LEFT JOIN #__jevents_vevent as ev ON rpt.eventid = ev.ev_id"
			. "\n LEFT JOIN #__jevents_icsfile as icsf ON icsf.ics_id=ev.icsid "
			. "\n LEFT JOIN #__jevents_vevdetail as det ON det.evdet_id = rpt.eventdetail_id"
			. "\n LEFT JOIN #__jevents_rrule as rr ON rr.eventid = rpt.eventid"
			. $extrajoin
			. "\n WHERE 1"
                        // Sub query seems faster on sites with large numbers of events for some reason
                        . "\n AND rpt.rp_id in (SELECT rp_id FROM #__jevents_repetition as rpt2 WHERE 1 AND rpt2.endrepeat >= '$startdate' AND rpt2.startrepeat <= '$enddate')"
			//. "\n AND rpt.endrepeat >= '$startdate' AND rpt.startrepeat <= '$enddate'"

			// Must suppress multiday events that have already started
			. "\n AND NOT (rpt.startrepeat < '$startdate' AND det.multiday=0) "

			. $extrawhere
			. "\n AND ev.access " . (version_compare(JVERSION, '1.6.0', '>=') ? ' IN (' . $aid . ')' : ' <=  ' . $aid)
			. "  AND icsf.state=1 AND icsf.access " . (version_compare(JVERSION, '1.6.0', '>=') ? ' IN (' . $aid . ')' : ' <=  ' . $aid)
			;

			$query .="\n GROUP BY catmap.catid";
		}
		else {
			$extrajoin = ( count( $extrajoin  ) ?  " \n LEFT JOIN ". implode( " \n LEFT JOIN ", $extrajoin ) : '' );
			$extrawhere = ( count( $extrawhere ) ? ' AND '. implode( ' AND ', $extrawhere ) : '' );
			
			if (!$showrepeats){
				$query = "SELECT count(distinct ev.ev_id) as evcount, ev.catid";
			}
			else {
				$query = "SELECT count(distinct rpt.rp_id) as evcount, ev.catid";
			}
			$query .= "\n FROM #__jevents_repetition as rpt"
			. "\n LEFT JOIN #__jevents_vevent as ev ON rpt.eventid = ev.ev_id"
			. "\n LEFT JOIN #__jevents_icsfile as icsf ON icsf.ics_id=ev.icsid "
			. "\n LEFT JOIN #__jevents_vevdetail as det ON det.evdet_id = rpt.eventdetail_id"
			. "\n LEFT JOIN #__jevents_rrule as rr ON rr.eventid = rpt.eventid"
			. $extrajoin
			. "\n WHERE ev.catid IN(" . $catids. ")"
			. "\n AND rpt.endrepeat >= '$startdate' AND rpt.startrepeat <= '$enddate'"

			// Must suppress multiday events that have already started
			. "\n AND NOT (rpt.startrepeat < '$startdate' AND det.multiday=0) "

			. $extrawhere
			. "\n AND ev.access " . (version_compare(JVERSION, '1.6.0', '>=') ? ' IN (' . $aid . ')' : ' <=  ' . $aid)
			. "  AND icsf.state=1 AND icsf.access " . (version_compare(JVERSION, '1.6.0', '>=') ? ' IN (' . $aid . ')' : ' <=  ' . $aid)
			;

			$query .="\n GROUP BY ev.catid";
		}
		
	 	$db->setQuery($query);
		$rows = $db->loadAssocList('catid');

		//echo $db->getQuery()."<br/>";
		//var_dump($rows);
		return $rows;
	}
	
	public function getStartEndDates() {
		
		$params = $this->params;

		$jinput = Factory::getApplication()->input;
		
		$startdate = $jinput->getString( "startdate", "" );
		$enddate = $jinput->getString ( "enddate", "" );
                jimport('joomla.utilities.date');
		
		if ($startdate == "") {
			if ($params->get ( "relative", "rel" ) == "abs") {
				$startdate = $params->get ( "absstart", "" );
				list ( $startyear, $startmonth, $startday ) = explode ( "-", $startdate );
			} else if ($params->get ( "relative", "rel" ) == "strtotime") {
				$value = $params->get ( "strstart", "" );
				$value = new JevDate ( $value );
				$startdate = $value->toFormat ( "%Y-%m-%d" );
			} else {
				$value = $params->get ( "relstart", "" );
				$value = str_replace ( ",", " ", $value );
				$value = str_replace ( "y", "year", $value );
				$value = str_replace ( "d", "day", $value );
				$value = str_replace ( "w", "week", $value );
				$value = str_replace ( "m", "month", $value );
				$value = new JevDate ( $value );
				$startdate = $value->toFormat ( "%Y-%m-%d" );
			}
		}
		if ($enddate == "") {
			if ($params->get ( "relative", "rel" ) == "abs") {
				$enddate = $params->get ( "absend", "" );
			} else if ($params->get ( "relative", "rel" ) == "strtotime") {
				$value = $params->get ( "strend", "" );
				$value = new JevDate ( $value );
				$enddate = $value->toFormat ( "%Y-%m-%d" );
			} else {
				$value = $params->get ( "relend", "" );
				$value = str_replace ( ",", " ", $value );
				$value = str_replace ( "y", "year", $value );
				$value = str_replace ( "d", "day", $value );
				$value = str_replace ( "w", "week", $value );
				$value = str_replace ( "m", "month", $value );
				$value = new JevDate ( $value );
				$enddate = $value->toFormat ( "%Y-%m-%d" );
			}
		}
		return array ($startdate, $enddate );
	
	}

}
