<?php

/**
 * copyright (C) 2015 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;

// This will only be triggered in Joomla 3.4.0 or later after a change in the name convension of file packages
class JEventsExtPlusLayoutInstallerScript
{

	private $oldelement = "extplus";
	private $newelement = "JEventsExtPlusLayout";

	/**
	 * method to uninstall the component
	 *
	 * @return void
	 */
	public function uninstall($parent)
	{

		// We are a file installer, so we need to manually delete the files.
		// $parent is the class calling this method
		if (Folder::exists(JPATH_SITE."/components/com_jevents/views/extplus")){
			Folder::delete(JPATH_SITE."/components/com_jevents/views/extplus");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_cal/tmpl/extplus")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_cal/tmpl/extplus");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_latest/tmpl/extplus")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_latest/tmpl/extplus");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_legend/tmpl/extplus")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_legend/tmpl/extplus");
		}
	}

	//
	// Joomla installer functions
	//
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	public function postflight($type, $parent)
	{
		//Lower case file names means a lower case element name, lets clean that up.
		$db = Factory::getDbo();
		$db->setQuery("SELECT * FROM #__extensions WHERE element =".$db->quote(strtolower($this->newelement)));
		$extension = $db->loadobject();

		if ($extension) {
			$db->setQuery("UPDATE #__extensions SET element =".$db->quote($this->newelement)." WHERE element =".$db->quote(strtolower($this->newelement)));
			$db->execute();
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery("SELECT * FROM #__extensions WHERE element =".$db->quote($this->oldelement). " OR element=".$db->quote($this->newelement));
		$extensions = $db->loadObjectList();

		if (count($extensions)>1){
			$hasold = false;
			$hasnew = false;
			foreach ($extensions as $extension){
				if ($extension->element == $this->oldelement) {
					$hasold = $extension;
				}
				else if ($extension->element == $this->newelement) {
					$hasnew = $extension ;
				}
			}
			if ($hasold && $hasnew){
				$db->setQuery("DELETE FROM #__extensions WHERE element =".$db->quote($this->oldelement));
				$db->execute();
			}
		}

		if (File::exists(JPATH_ADMINISTRATOR."/manifests/files/extplus.xml")){
			File::delete(JPATH_ADMINISTRATOR."/manifests/files/extplus.xml");
		}

		return;

	}

}
