<?php

/**
 * copyright (C) 2015 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;

// This will only be triggered in Joomla 3.4.0 or later after a change in the name convension of file packages
class JEventsFloatLayoutInstallerScript
{

    private $oldelement = "float";
    private $newelement = "JEventsFloatLayout";


    //
    // Joomla installer functions
    //
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

    public function postflight($type, $parent)
    {

        // Joomla! broke the update call, so we have to create a workaround check.
        $db = Factory::getDbo();

	    $db->setQuery('SELECT manifest_cache FROM #__extensions WHERE element = "com_jevents" AND type = "component"');
	    $manifestcache = $db->loadResult();
	    if (!$manifestcache) {
		    \Joomla\CMS\Factory::getApplication()->enqueueMessage('This version of Float Theme is desgined for JEvents.<br/>Please install JEvents 3.4.0 or later before installing Float Theme' , "warning");
		    return false;
	    }
	    else {
		    $manifestcache = json_decode( $manifestcache, true );
		    if (version_compare($manifestcache["version"], "3.4.23", "<")){
			    \Joomla\CMS\Factory::getApplication()->enqueueMessage('This version of Float Theme is designed for JEvents 3.4.23 or later.<br/>Please upgrade JEvents to version 3.4.11 or later before installing this version of Float Theme' , "warning");
			    return false;
		    }
	    }

        $db->setQuery("SELECT * FROM #__extensions WHERE element =".$db->quote($this->oldelement). " OR element=".$db->quote($this->newelement));
        $extensions = $db->loadObjectList();

        if (count($extensions)>1){
            $hasold = false;
            $hasnew = false;
            foreach ($extensions as $extension){
                if ($extension->element == $this->oldelement) {
                    $hasold = $extension;
                }
                else if ($extension->element == $this->newelement) {
                    $hasnew = $extension ;
                }
            }
            if ($hasold && $hasnew){
                $db->setQuery("DELETE FROM #__extensions WHERE element =".$db->quote($this->oldelement));
                $db->execute();
            }
        }
	    //Lets just check we have inserted the default rows if not insert them!!
	    $db->setQuery("SELECT * FROM #__jev_defaults");
	    $defaults =$db->loadObjectList("name");
	    if (!isset($defaults['icalevent.list_block1'])){
		    $db->setQuery("INSERT INTO  #__jev_defaults set name='icalevent.list_block1',
						title=".$db->Quote("JEV_FLOAT_LAYOUT_BLOCK1").",
						subject='',
						value='',
                                                params='',
						state=0");
		    $db->execute();

		    $db->setQuery("INSERT INTO  #__jev_defaults set name='icalevent.list_block2',
						title=".$db->Quote("JEV_FLOAT_LAYOUT_BLOCK2").",
						subject='',
						value='',
                                                params='',
						state=0");
		    $db->execute();
	    }
	    if (!isset($defaults['icalevent.list_block3']))
	    {
		    $db->setQuery("INSERT INTO  #__jev_defaults set name='icalevent.list_block3',
						title=" . $db->Quote("JEV_FLOAT_LAYOUT_BLOCK3") . ",
						subject='',
						value='',
                                                params='',
						state=0");
		    $db->execute();
	    }
	    if (!isset($defaults['icalevent.list_block4']))
	    {
		    $db->setQuery("INSERT INTO  #__jev_defaults set name='icalevent.list_block4',
						title=" . $db->Quote("JEV_FLOAT_LAYOUT_BLOCK4") . ",
						subject='',
						value='',
                                                params='',
						state=0");
		    $db->execute();
	    }
	    if (File::exists(JPATH_ADMINISTRATOR."/manifests/files/float.xml")){
		    File::delete(JPATH_ADMINISTRATOR."/manifests/files/float.xml");
	    }

		return;

    }

	public function uninstall($parent)
	{

		// We are a file installer, so we need to manually delete the files.
		// $parent is the class calling this method
		if (Folder::exists(JPATH_SITE."/components/com_jevents/views/float")){
			Folder::delete(JPATH_SITE."/components/com_jevents/views/float");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_cal/tmpl/float")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_cal/tmpl/float");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_latest/tmpl/float")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_latest/tmpl/float");
		}
		if (Folder::exists(JPATH_SITE."/modules/mod_jevents_legend/tmpl/float")){
			Folder::delete(JPATH_SITE."/modules/mod_jevents_legend/tmpl/float");
		}
	}

}
