<?php

/**
 * copyright (C) 2012 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

class plgjeventsjevnotifyInstallerScript
{

	//
	// Joomla installer functions
	//
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	function install($parent)
	{
		$this->createTables();

		//Whoops! must disable auto enable for now. We need to update the database default params at the same time, or add more fallbacks in code.
		// New install, lets enable the plugin! 
		//$db = Factory::getDbo();

		//$sql = "UPDATE #__extensions SET enabled=1 WHERE element='agendaminutes'";
		//$db->setQuery($sql);
		//$db->execute();
		//echo $db->getErrorMsg();

		return true;

	}

	function uninstall($parent)
	{
		// No nothing for now, we want to keep the tables just incase they remove the plugin by accident. 

	}

	function update($parent)
	{
		$this->createTables();
		$this->updateTables();

	}

	function createTables()
	{
		$db = Factory::getDbo();
		// message type 0 = new, 1 = changed
		if (version_compare(JVERSION, "3.3", 'ge'))
		{
			$charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}
		else
		{
			$charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}
		$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_notifications(
	id int(11) NOT NULL auto_increment,
	rp_id int(11) NOT NULL default 0,
	messagetype tinyint(1) unsigned NOT NULL default 0,
	user_id int(11) NOT NULL default 0,
	attendee_id int(11) NOT NULL default 0,
	ajeattendee_id int(11) NOT NULL default 0,
	invitee_id int(11) NOT NULL default 0,
	remindee_id int(11) NOT NULL default 0,
	emailaddress varchar(250) NOT NULL default '',
	created datetime  NOT NULL default '0000-00-00 00:00:00',
	sentmessage tinyint(1) unsigned NOT NULL default 0,
	whensent datetime  NOT NULL default '0000-00-00 00:00:00',
	notificationtype int(11) NOT NULL default 0,
	subject text NOT NULL ,
	message text NOT NULL ,
	creatoremail varchar(255) NOT NULL default ''	,
	creatorname varchar(255) NOT NULL default ''	,
	m_ev int(11) NOT NULL default 0,
	PRIMARY KEY  (id),
	UNIQUE INDEX (rp_id, user_id, emailaddress),
	INDEX (rp_id)
) $charset;
SQL;
		$db->setQuery($sql);
		if (!$db->execute())
		{
		//	echo $db->getErrorMsg();
		}

		$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_notification_map(
	id int(11) NOT NULL auto_increment,
	user_id int(12) NOT NULL  default 0,
	cat_id int(12) NOT NULL  default 0,
	PRIMARY KEY (id),
	INDEX  (user_id),
	INDEX  (cat_id),
	INDEX  (user_id, cat_id)	
);
SQL;
		$db->setQuery($sql);
		$db->execute();

	}

	function updatetables()
	{
		$db = Factory::getDbo();


		if (version_compare(JVERSION, "3.3", 'ge'))
		{
			$charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}
		else
		{
			$charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}

		$sql = "SHOW COLUMNS FROM #__jev_notifications";
		$db->setQuery($sql);
		$cols = @$db->loadObjectList("Field");

		if (!array_key_exists("notificationtype", $cols))
		{
			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN notificationtype int(11) NOT NULL default 0";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("subject", $cols))
		{
			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN subject text NOT NULL  ";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("message", $cols))
		{
			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN message text NOT NULL ";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("creatoremail", $cols))
		{
			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN creatoremail varchar(255) NOT NULL default '' ";
			$db->setQuery($sql);
			@$db->execute();

			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN creatorname varchar(255) NOT NULL default '' ";
			$db->setQuery($sql);
			@$db->execute();
		}
		if (!array_key_exists("m_ev", $cols))
		{
			$sql = "ALTER TABLE #__jev_notifications ADD COLUMN m_ev int(11) NOT NULL default 0 ";
			$db->setQuery($sql);
			@$db->execute();
		}

	}

	function postflight($type, $parent)
	{
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)
		$jversion = new \Joomla\CMS\Version;
		if ($jversion->isCompatible('4.0'))
		{
			$version =  $parent->manifest->version;
		}
		else
		{
			$version =  $parent->get('manifest')->version ;
		}
		echo '<h2>' . Text::_('PLG_INST_JEVENTS_JEVNOTIFY') . ' ' . $version . ' </h2>';

		echo '<strong>';

		if ($type == "update")
		{
			echo Text::_('PLG_INST_JEVENTS_JEVNOTIFY_UPDATE') . '<br/><br/>';
		}
		else
		{
			echo Text::_('PLG_INST_JEVENTS_JEVNOTIFY_INSTALL') . '<br/><br/>';
		}
		echo '</strong><br/><br/>';

	}

}
