<?php

/**
 * JEvents Component for Joomla 2.5.x
 *
 * @version     $Id: scriptfile.php 1430 2010-11-11 14:15:05Z royceharding $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2020 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://www.jevents.net
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Installer\Installer;
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

class pkg_jeventstagsInstallerScript
{
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	// TODO enable plugins
	public function update()
	{
		$this->cleanupFilters();


		return true;
	}

	public function install($adapter)
	{
		$this->cleanupFilters();

		return true;
	}


	public function uninstall($adapter)
	{

	}

	/*
	 * enable the plugins
	 */
	function postflight($type, $parent)
	{
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)

		if ($type == 'install') {
			// enable plugin
			$db = Factory::getDbo();
			$query = "UPDATE #__extensions SET enabled=1 WHERE folder='jevents' and type='plugin' and element='jevtags'";
			$db->setQuery($query);
			$db->execute();
		}
	}

	// removes old redundance filter files
	private function cleanupFilters(){
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		$folder = "jevents/jevlocations";
		if (Folder::exists(JPATH_ROOT.'/plugins/filters') || Folder::exists(JPATH_ROOT.'/plugins/jevents/filters')) {
			$filters = array("Taglookup");
			foreach ($filters as $filter){
				if (File::exists(JPATH_ROOT.'/plugins/filters/'.$filter.".php")) {
					try {
						File::delete(JPATH_ROOT.'/plugins/filters/'.$filter.".php");
					}
					catch (Exception $e){

					}
				}

				if (File::exists(JPATH_ROOT.'/plugins/jevents/filters/'.$filter.".php")) {
					try {
						File::delete(JPATH_ROOT.'/plugins/jevents/filters/'.$filter.".php");
					}
					catch (Exception $e){

					}
				}

			}
		}
	}

}
