<?php

/**
 * @copyright (C) 2012 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Installer\Installer;

class plgjeventsjevfilesInstallerScript {

    //
    // Joomla installer functions
    //
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

    function install($parent) {
        $success = $this->createTables();
        if (!$success) {
            return false;
        }

        // Whoops! must disable auto enable for now. We need to update the database default params at the same time, or add more fallbacks in code.
        // New install, lets enable the plugin!
        $db = Factory::getDbo();

        $sql = "UPDATE #__extensions SET enabled=1 WHERE element='jevfiles'";
        $db->setQuery($sql);
	    try {
		    $db->execute();
	    }
	    catch (RuntimeException $e)
	    {
		    $error = $e->getMessage();
		    echo $error;
	    }

        return true;
    }

    function uninstall($parent) {
        // No nothing for now, we want to keep the tables just incase they remove the plugin by accident.
    }

    function update($parent) {
        $success = $this->createTables();
        if (!$success) {
            return false;
        }

        // Nothing to do for now, tables should be created on install.
    }

    function createTables() {
        $db = Factory::getDbo();
        if (version_compare(JVERSION, '3.3', 'ge')) {
            $charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
            $rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
        } else {
            $charset = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
            $rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
        }
        $sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_files(
	file_num int(2) NOT NULL  default 0,
	ev_id int(12) NOT NULL  default 0,
	evdet_id int(12) NOT NULL  default 0,
	filetype varchar(10) NOT NULL  default "",
	filetitle varchar(120) NOT NULL  default "",
	filename varchar(120) NOT NULL  default "",
	filecomment varchar(250) NOT NULL  default "",

	PRIMARY KEY (file_num,evdet_id,filetype),
	INDEX (ev_id),
        INDEX (evdet_id)
)  $charset;
SQL;
        $db->setQuery($sql);
	    try {
		    $db->execute();
	    }
	    catch (RuntimeException $e)
	    {
		    $error = $e->getMessage();
		    echo $error;
	    }

        $sql = 'SHOW COLUMNS FROM `#__jev_files`';
        $db->setQuery($sql);
        $cols = @$db->loadObjectList('Field');

        if (!array_key_exists('evdet_id', $cols))
        {
                $sql = 'ALTER TABLE #__jev_files ADD COLUMN evdet_id int(12) NOT NULL  default 0';
                $db->setQuery($sql);
                @$db->execute();

                // Now insert the missing data
                $sql = 'UPDATE #__jev_files AS jf, #__jevents_vevent AS evt  SET jf.evdet_id = evt.detail_id WHERE evt.ev_id=jf.ev_id and jf.evdet_id=0';
                $db->setQuery($sql);
                @$db->execute();

                // Drop orphan rows
                $sql = "SELECT jf.ev_id FROM #__jev_files AS jf
LEFT JOIN #__jevents_vevent AS evt  
ON evt.ev_id=jf.ev_id 
where evt.ev_id is null";
                $db->setQuery($sql);
                $orphans = $db->loadColumn();

                if (count($orphans)>0){
                   // $sql = 'DELETE FROM #__jev_files WHERE ev_id in ('.implode(",", $orphans).')';
                    //$db->setQuery($sql);
                    //@$db->execute();
                }

                // finally fix the index
                $sql = 'alter table #__jev_files drop primary key, add PRIMARY KEY (file_num,ev_id, filetype)';
                $db->setQuery($sql);
                @$db->execute();
        }

        // New combined table - should be faster with fewer queries
        $sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_files_combined(
        file_id int(12) NOT NULL auto_increment,
	ev_id int(12) NOT NULL  default 0,
	evdet_id int(12) NOT NULL  default 0,
	
        PRIMARY KEY (file_id),
        UNIQUE KEY (ev_id, evdet_id),
	INDEX (ev_id),
        INDEX (evdet_id)
)  $charset;
SQL;

        $db->setQuery($sql);
	    try {
		    $db->execute();
	    }
	    catch (RuntimeException $e)
	    {
		    $error = $e->getMessage();
		    echo $error;
	    }

        $sql = 'SHOW COLUMNS FROM `#__jev_files_combined`';
        $db->setQuery($sql);
        $cols = @$db->loadObjectList('Field');

        if (!array_key_exists('filetitle1', $cols)) {

            for ($col = 1; $col <= 30; $col++) {
                $sql = "ALTER TABLE #__jev_files_combined "
                        . "ADD COLUMN imagetitle$col   varchar(120) NOT NULL  default '', "
                        . "ADD COLUMN imagename$col    varchar(120) NOT NULL  default '', "
                        . "ADD COLUMN imagecomment$col TEXT , "
                        . "ADD COLUMN filetitle$col   varchar(120) NOT NULL  default '', "
                        . "ADD COLUMN filename$col    varchar(120) NOT NULL  default '', "
                        . "ADD COLUMN filecomment$col TEXT ";
                $db->setQuery($sql);
                @$db->execute();
            }
        }

        $sql = 'SELECT * FROM `#__jev_files_combined` LIMIT 1';
        $db->setQuery($sql);
        $hasnewdata = @$db->loadObject();
        $sql = "SELECT * FROM `#__jev_files` LIMIT 1";
        $db->setQuery($sql);
        $hasolddata = @$db->loadObject();
        if ($hasolddata && !$hasnewdata) {
            /*
            // Do we have permission to create the procedure
            $sql = "SHOW GRANTS";
            $db->setQuery($sql);
            $grants = $db->loadColumn();

            $allowed = false;
            foreach ($grants as $grant){
                if (strpos($grant, "ALL PRIVILEGES")>0 || strpos($grant, "CREATE ROUTINE")>0){
                    $allowed = true;
                }
            }
            if (!$allowed) {
                \Joomla\CMS\Factory::getApplication()->enqueueMessage( Text::_('JEV_UPGRADE_REQUIRED_CREATE_ROUTINE_PERMISSIONS'), "warning");
                return false;
            }

            $sql = <<< SQL
DROP PROCEDURE IF EXISTS convert_jevfiles;
SQL;
            $db->setQuery($sql);
            $db->execute();
            $sql = <<< SQL
CREATE PROCEDURE convert_jevfiles()
BEGIN
    DECLARE cnt INT DEFAULT 1;

    DELETE  FROM #__jev_files where filename="";

    INSERT IGNORE INTO #__jev_files_combined ( ev_id, evdet_id)
        SELECT  ev_id, evdet_id FROM #__jev_files
        WHERE filetype="image" OR filetype="file" AND ev_id IS NOT NULL GROUP BY ev_id, evdet_id;

countloop: WHILE cnt <=30 DO
        SET @qry = CONCAT('UPDATE #__jev_files_combined AS jfc
LEFT JOIN #__jev_files AS jf ON jfc.evdet_id=jf.evdet_id AND jfc.ev_id=jf.ev_id
SET jfc.imagetitle',cnt,'=jf.filetitle ,
jfc.imagename',cnt,'= jf.filename,
jfc.imagecomment',cnt,'=jf.filecomment
WHERE jf.filetype="image" AND jf.file_num=',cnt
            );

        PREPARE stmt FROM @qry;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;

        SET @qry = CONCAT('UPDATE #__jev_files_combined AS jfc
LEFT JOIN #__jev_files AS jf ON jfc.evdet_id=jf.evdet_id AND jfc.ev_id=jf.ev_id
SET jfc.filetitle',cnt,'=jf.filetitle ,
jfc.filename',cnt,'= jf.filename,
jfc.filecomment',cnt,'=jf.filecomment
WHERE jf.filetype="file" AND jf.file_num=',cnt
            );

        PREPARE stmt FROM @qry;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;

        SET cnt = cnt + 1;

    END WHILE countloop;
END;
SQL;
            // normal DB execute method can't deal with table prefix within the stored procedure so we do a trivial replacement here
            $sql = str_replace('#__', $db->getPrefix(), $sql);
            $db->setQuery($sql);
            $db->execute();
            $sql = <<< SQL
CALL convert_jevfiles();
SQL;
            $db->setQuery($sql);
            $db->execute();
            $sql = <<< SQL
DROP PROCEDURE IF EXISTS convert_jevfiles;
SQL;
            $db->setQuery($sql);
            $db->execute();
             *
             */

            // do migration using PHP - too many sites have issues with stored procedures :(

            $sql = "DELETE  FROM #__jev_files where filename=''";
            $db->setQuery($sql);
            $db->execute();

            // Initial import of reference data - no image or files yet
            $sql = "INSERT IGNORE INTO #__jev_files_combined ( ev_id, evdet_id) 
            SELECT  ev_id, evdet_id FROM #__jev_files 
            WHERE filetype='image' OR filetype='file' AND ev_id IS NOT NULL 
            GROUP BY ev_id, evdet_id;";
            $db->setQuery($sql);
            $db->execute();

            // Now loop over the images/files
            for ($cnt = 1; $cnt<=30; $cnt++){
                $sql = "UPDATE #__jev_files_combined AS jfc
LEFT JOIN #__jev_files AS jf ON jfc.evdet_id=jf.evdet_id AND jfc.ev_id=jf.ev_id 
SET jfc.imagetitle$cnt = jf.filetitle ,
jfc.imagename$cnt = jf.filename,
jfc.imagecomment$cnt = jf.filecomment
WHERE jf.filetype='image' AND jf.file_num=$cnt ";
                $db->setQuery($sql);
                $db->execute();

                $sql = "UPDATE #__jev_files_combined AS jfc
LEFT JOIN #__jev_files AS jf ON jfc.evdet_id=jf.evdet_id AND jfc.ev_id=jf.ev_id 
SET jfc.filetitle$cnt = jf.filetitle ,
jfc.filename$cnt = jf.filename,
jfc.filecomment$cnt = jf.filecomment
WHERE jf.filetype='file' AND jf.file_num=$cnt ";
                $db->setQuery($sql);
                $db->execute();

                Factory::getApplication()->enqueueMessage(Text::_("JEV_IMAGES_MIGRATED"), "warning");
            }

        }

        return true;
    }

    function postflight($type, $parent) {
        // $parent is the class calling this method
        // $type is the type of change (install, update or discover_install)
	    $jversion = new \Joomla\CMS\Version;
	    if ($jversion->isCompatible('4.0'))
	    {
		    $version =  $parent->manifest->version;
	    }
	    else
	    {
		    $version =  $parent->get('manifest')->version ;
	    }
	    echo '<h2>' . Text::_('PLG_INST_JEVENTS_JEVFILES') . ' ' . $version . ' </h2>';

        echo '<strong>';

        if ($type == 'update') {
            echo Text::_('PLG_INST_JEVENTS_JEVFILES_UPDATE') . '<br/>';
        } else {
            echo Text::_('PLG_INST_JEVENTS_JEVFILES_INSTALL') . '<br/>';
        }
        echo '</strong><br/><br/>';
    }

}
