<?php

/**
 * copyright (C) 2012 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\Installer;

class plgjeventsjevusersInstallerScript
{

	//
	// Joomla installer functions
	//

	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	public function install($parent)
	{

		$this->createTables();

		return true;

	}

	public function uninstall($parent)
	{
		// No nothing for now, we want to keep the tables just incase they remove the plugin by accident.

	}

	public function update($parent)
	{
		$this->createTables();

		$db = Factory::getDBO();

		$sql = "SHOW COLUMNS FROM `#__jev_usereventsmap`";
		$db->setQuery($sql);
		$cols     = $db->loadObjectList();
		$uptodate = false;
		foreach ($cols as $col)
		{
			if ($col->Field == "groupid")
			{
				$uptodate = true;
				break;
			}
		}
		if (!$uptodate)
		{
			$sql = "Alter table #__jev_usereventsmap ADD COLUMN groupid int(12) NOT NULL default 0 ";
			$db->setQuery($sql);
			try {
				$db->execute();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
			}

			$sql = "alter table #__jev_usereventsmap DROP PRIMARY KEY";
			$db->setQuery($sql);
			try {
				$db->execute();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
			}

			$sql = "alter table #__jev_usereventsmap ADD PRIMARY KEY (`user_id`,`evdet_id`, `groupid`)";
			$db->setQuery($sql);
			try {
				$db->execute();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
			}

		}

		return true;

	}

	private function createTables()
	{

		$db = Factory::getDBO();
		if (version_compare(JVERSION, "3.3", 'ge'))
		{
			$charset    = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}
		else
		{
			$charset    = ($db->hasUTFSupport()) ? ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ? 'CHARACTER SET utf8' : '';
		}
		$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_usereventsmap(
	user_id int(12) NOT NULL  default 0,
	evdet_id int(12) NOT NULL  default 0,
	privateevent tinyint(2) NOT NULL default 1,
	groupid int(12) NOT NULL default 0,

	PRIMARY KEY (user_id,evdet_id, groupid),
	INDEX  (evdet_id),
	INDEX  (user_id)
) $charset;
SQL;
		$db->setQuery($sql);
		try {
			$db->execute();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
		}

	}

}

