<?php

/**
 * copyright (C) 2012 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Installer\Installer;

class plgjeventsjevcustomfieldsInstallerScript
{
	//
	// Joomla installer functions
	//

	public function preflight($type, $parent) {

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}
	}

	function install($parent)
	{

		$this->update($parent);

		return true;

	}

	function uninstall($parent)
	{
		// No nothing for now, we want to keep the tables just incase they remove the plugin by accident.
	}

	function update($parent)
	{
		$this->createTables();

		$db = Factory::getDbo();
		// If upgrading then add new columns - do all the tables at once
		$sql = "SHOW COLUMNS FROM `#__jev_customfields`";
		$db->setQuery($sql);
		$cols = @$db->loadObjectList("Field");

		if (!array_key_exists("evdet_id", $cols))
		{
			$sql = "ALTER TABLE #__jev_customfields ADD COLUMN evdet_id int(11) NOT NULL default 0";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("language", $cols))
		{
			$sql = "ALTER TABLE #__jev_customfields ADD COLUMN language varchar(20) NOT NULL default '*'";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("user_id", $cols))
		{
			$sql = "ALTER TABLE #__jev_customfields ADD COLUMN user_id int(11) NOT NULL default 0";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("rp_id", $cols))
		{
			$sql = "ALTER TABLE #__jev_customfields ADD COLUMN rp_id int(11) NOT NULL default 0";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("evdet_id", $cols))
		{
			$sql = "ALTER TABLE `#__jev_customfields` ADD INDEX user_id (user_id)";
			$db->setQuery($sql);
			@$db->execute();
		}

		// If upgrading then add new columns - do all the tables at once
		$sql = "SHOW INDEX FROM `#__jev_customfields`";
		$db->setQuery($sql);
		$indexes = @$db->loadObjectList("Key_name");

		if (!array_key_exists("rp_id", $indexes))
		{
			$sql = "ALTER TABLE `#__jev_customfields` ADD INDEX rp_id (rp_id)";
			$db->setQuery($sql);
			@$db->execute();
		}

		if (!array_key_exists("detval", $indexes))
		{
			$sql = "ALTER TABLE `#__jev_customfields` ADD INDEX detval (evdet_id, value(10) )";
			$db->setQuery($sql);
			@$db->execute();
		}

                if (!array_key_exists("detlang", $indexes))
		{
			$sql = "ALTER TABLE `#__jev_customfields` ADD INDEX detlang (evdet_id,user_id, language(5))";
			$db->setQuery($sql);
			@$db->execute();
		}

		//Whoops! must disable auto enable for now. We need to update the database default params at the same time, or add more fallbacks in code.

		// New install, lets enable the plugin!
		$db = Factory::getDbo();

		$sql = "UPDATE #__extensions SET enabled=1 WHERE element='jevcustomfields'";
		$db->setQuery($sql);
		@$db->execute();

	}

	function createTables() {

				$db = Factory::getDbo();
		if (version_compare(JVERSION, "3.3", 'ge')){
			$charset = ($db->hasUTFSupport()) ?  ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ?  'CHARACTER SET utf8' : '';
		}
		else {
			$charset = ($db->hasUTFSupport()) ?  ' DEFAULT CHARACTER SET `utf8`' : '';
			$rowcharset = ($db->hasUTFSupport()) ?  'CHARACTER SET utf8' : '';
		}
				$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_customfields(
	id int(11) NOT NULL auto_increment,
	evdet_id int(11) NOT NULL default 0,
	user_id int(11) NOT NULL default 0,
	rp_id int(11) NOT NULL default 0,
	name varchar(255) NOT NULL default '',
	value text NOT NULL ,
        language varchar(20) NOT NULL default '*',
                                        
	PRIMARY KEY  (id),
	INDEX (evdet_id),
	INDEX (user_id),
	INDEX (rp_id),
	INDEX combo (name,value(10)),
	INDEX detval (evdet_id,value(10)),
	INDEX detlang (evdet_id,user_id, language(5))                                        
)  $charset;
SQL;
				$db->setQuery($sql);
		try {
			$db->execute();
		}
		catch (RuntimeException $e)
		{
			echo  $e->getMessage();
		}

        $sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_customfields_combined(
	id int(11) NOT NULL auto_increment,
	evdet_id int(11) NOT NULL default 0,
	user_id int(11) NOT NULL default 0,
	rp_id int(11) NOT NULL default 0,
        extension varchar(20) NOT NULL default 'com_jevents',
        language varchar(20) NOT NULL default '*',
                
	name varchar(255) NOT NULL default '',                
	value text NOT NULL ,
                                        
	PRIMARY KEY  (id),
	INDEX (evdet_id),
	INDEX (user_id),
	INDEX (rp_id),
	INDEX combo (name,value(10)),
	INDEX detval (evdet_id,value(10)),
	INDEX detlang (evdet_id,user_id, language(5))                                        
)  $charset;
SQL;
        $db->setQuery($sql);
		try {
			$db->execute();
		}
		catch (RuntimeException $e)
		{
			echo  $e->getMessage();
		}
    }
    function postflight($type, $parent)
    {
	    // Remove old file causes un-usable backend.
	    if(File::exists(JPATH_SITE . '/plugins/jevents/jevcustomfields/customfields/jevboolean.php')) {
		    File::delete(JPATH_SITE . '/plugins/jevents/jevcustomfields/customfields/jevboolean.php');
	    }
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)
	    $jversion = new \Joomla\CMS\Version;
	    if ($jversion->isCompatible('4.0'))
	    {
		    $version =  $parent->manifest->version;
	    }
	    else
	    {
		    $version =  $parent->get('manifest')->version ;
	    }

	    echo '<h2>' . Text::_('JEV_CUSTOMFIELDS_PLUGIN') . ' ' . $version . ' </h2>';

		echo '<strong>';

		if ($type == "update") {
			echo Text::_('JEV_CUSTOMFIELDS_INSTALL_SUCCESS_1') . '<br/>';
                        echo Text::_('JEV_CUSTOMFIELDS_PLUGIN_DESC');
		} else {
			echo Text::_('JEV_CUSTOMFIELDS_INSTALL_SUCCESS_2') . '<br/>';
                        echo Text::_('JEV_CUSTOMFIELDS_PLUGIN_DESC');
		}
		echo '</strong><br/><br/>';
	}

}
/*
create temporary table tempTable (id int(11),evdet_id int(11),user_id int(11),name varchar(255),value text,
	PRIMARY KEY  (id),
	INDEX (evdet_id),
	INDEX (user_id),
	INDEX combo (name,value(10)));

insert into tempTable (id,evdet_id,user_id,name,value) (select max(id), evdet_id, user_id, name, value  from jos_jev_customfields where user_id=0 group by evdet_id, name);

SELECT cf.*, cfsub.id  from jos_jev_customfields as cf
LEFT JOIN tempTable as cfsub ON cf.evdet_id= cfsub.evdet_id AND cf.name= cfsub.name and cf.id = cfsub.id
WHERE  cf.user_id=0
ORDER BY evdet_id desc

 */
