<?php

/**
 * copyright (C) 2012 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Version;

class plgjeventsagendaminutesInstallerScript
{

	//
	// Joomla installer functions
	//
	public function preflight($type, $parent) {

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	protected function joomlaCheck()
	{
		$jversion = new Version();

		// abort if the current Joomla release is older
		if( version_compare( $jversion->getShortVersion(), $this->minimum_joomla_release, 'lt' ) )
		{
			Factory::getApplication()->enqueueMessage(Text::sprintf("PLG_INST_JEVENTS_JEVAGENDAMINUTES_LOW_JOOMLA_WARNING", $this->minimum_joomla_release), 'error');
			return false;
		}

		return true;
	}

	function install($parent)
	{

		$this->createTables();

		//Whoops! must disable auto enable for now. We need to update the database default params at the same time, or add more fallbacks in code.
		// New install, lets enable the plugin!
		//$db = Factory::getDbO();

		//$sql = "UPDATE #__extensions SET enabled=1 WHERE element='agendaminutes'";
		//$db->setQuery($sql);
		//$db->execute();
		//echo $db->getErrorMsg();

		return true;

	}

	function uninstall($parent)
	{
		// No nothing for now, we want to keep the tables just incase they remove the plugin by accident.

	}

	function update($parent)
	{
		// Nothing to do for now, tables should be created on install.

	}

	function createTables()
	{

		$db = Factory::getDbo();


		$charset = ($db->hasUTFSupport()) ?  ' DEFAULT CHARACTER SET `utf8`' : '';
		$rowcharset = ($db->hasUTFSupport()) ?  'CHARACTER SET utf8' : '';

		/**
		 * create table if it doesn't exit
		 *
		 * For now :
		 *
		 * I'm ignoring attach,comment, resources, transp, attendee, related to, rdate, request-status
		 *
		 * Separate tables for rrule and exrule
		 */
		$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_agendaminutes(
	minutes_id int(12) NOT NULL  default 0,
	agenda_id int(12) NOT NULL  default 0,
	evdet_id int(12) NOT NULL  default 0,

	PRIMARY KEY (evdet_id),
	INDEX  (minutes_id),
	INDEX  (agenda_id)
) $charset;
SQL;
		$db->setQuery($sql);
		$db->execute();
		//echo $db->getErrorMsg();

	}

	function postflight($type, $parent)
	{
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)
		$jversion = new \Joomla\CMS\Version;
		if ($jversion->isCompatible('4.0'))
		{
			$version =  $parent->manifest->version;
		}
		else
		{
			$version =  $parent->get('manifest')->version ;
		}

		echo '<h2>' . Text::_('PLG_INST_JEVENTS_JEVAGENDAMINUTES') . ' ' . $version . ' </h2>';
		echo '<strong>';

		if ($type == "update")
		{
			echo Text::_('PLG_INST_JEVENTS_JEVAGENDAMINUTES_SUCC1') . '<br/>';
		}
		else
		{
			echo Text::_('PLG_INST_JEVENTS_JEVAGENDAMINUTES_SUCC2') . '<br/>';
		}
		echo '</strong><br/><br/>';
	}

}
