<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2018 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://www.jevents.net
 */

// no direct access
defined('_JEXEC') or die('Restricted access');
// We need to use a Directory Seperator as Windows does not like the usage of / still.
if(!defined('DS')){
	define('DS',DIRECTORY_SEPARATOR);
}
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Installer\Installer;
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

class plgjeventsjevtimelimitInstallerScript
{

	var $pluginsDir;
	var $filterpath;
	var $pluginDir;
	var $plugin_filterpath;
	var $filters;

	public function __construct()
	{
		$this->pluginsDir = 'plugins/jevents';
		// needed to clean up old installations
		$this->filterpath = Folder::makeSafe($this->pluginsDir ."/filters");

		$this->pluginDir = $this->pluginsDir.'/jevtimelimit';
		$this->plugin_filterpath = Folder::makeSafe($this->pluginDir ."/filters");

		$this->filters = array('Afterdate.php', 'Beforedate.php', 'Timelimit.php');  // RSH 11/11/10 - Need a better way to do this!
	}

	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	public function update()
	{

		return true;
	}

	public function install($adapter)
	{
		return true;
	}

	public function postflight($action, $adapter)
	{

		$db = Factory::getDbo();
		$charset = ($db->hasUTFSupport()) ? 'DEFAULT CHARACTER SET `utf8`' : '';

		$sql = <<<SQL
CREATE TABLE IF NOT EXISTS #__jev_timelimits(
	evdet_id int(12) NOT NULL  default 0,
	startlimit datetime  NOT NULL default '0000-00-00 00:00:00', 
	endlimit datetime  NOT NULL default '0000-00-00 00:00:00' ,
	PRIMARY KEY (evdet_id)
)  $charset;
SQL;
		$db->setQuery($sql);
		try {
			$db->execute();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
		}

		// Clean up the old style filters folder content - only ones that are duplicated with existing ones
		// Downside is it removes customised versions of these filters
		if (Folder::exists(JPATH_ROOT . "/" . $this->plugin_filterpath) && Folder::exists(JPATH_ROOT . "/" . $this->filterpath)) {

			// Delete the main filters folder if it is empty
			$oldfilterfiles = Folder::files(JPATH_ROOT ."/". $this->filterpath, $filter = '.', $recurse = true, $fullpath = true, $exclude = array());
			$filterfiles = Folder::files(JPATH_ROOT ."/". $this->plugin_filterpath, $filter = '.', $recurse = true, $fullpath = false, $exclude = array());

			if ( (is_array($oldfilterfiles)) && (count($oldfilterfiles) > 0) ) {
				foreach ($oldfilterfiles as $oldfilterfile)
				{
					if (in_array(basename($oldfilterfile),$filterfiles))
					{
						//File::delete($oldfilterfile);
						if (!File::exists($oldfilterfile . ".keep"))
						{
							File::move($oldfilterfile, $oldfilterfile . ".keep");
						}
					}
				}
			}
		}

		return true;

	}

}
