<?php

/**
 * copyright (C) 2012 - 2022 GWE Systems Ltd - All rights reserved
 * @license GNU/GPLv3 www.gnu.org/licenses/gpl-3.0.html
 * version 3.2.16
 * */
// no direct access
defined('_JEXEC' ) or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Component\ComponentHelper;

jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

class pkg_jevlocationsInstallerScript
{

	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = Factory::getDbo ();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}

	}

	// TODO enable plugins
	public function update()
	{

		return true;
	}

	public function install($adapter)
	{
		return true;
	}


	public function uninstall($adapter)
	{

	}

	/*
	 * enable the plugins
	 */
	function postflight($type, $parent)
	{
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)

		if ($type == 'install') {
			// enable plugin
			$db = Factory::getDbo ();
			$query = "UPDATE #__extensions SET enabled=1 WHERE folder='jevents' and type='plugin' and element='jevlocations'";
			$db->setQuery($query);
			$db->execute();
		}
		else
		{
			// Is this a migration from the old configuration setup - if so we need  to copy over the google maps settings
			$compparams = ComponentHelper::getParams('com_jevlocations');

			if ($compparams->get('mapsprovider', 'not set') === 'not set')
			{
				// Mapping
				$compparams->set('googlemaps-service.maps', $compparams->get('googlemaps', 'https://maps.googleapis.com'));
				$compparams->set('googlemaps-service.directions', $compparams->get('googledirections', 'https://maps.google.com'));
				//$compparams->set('googlemaps-service.redirecttodirections' ,$compparams->get('', ''));
				$compparams->set('googlemaps-service.gregion', $compparams->get('gregion', 'US'));
				$compparams->set('googlemaps-service.googlemaps_serverkey', $compparams->get('googlemapskey', ''));
				$compparams->set('googlemaps-service.googledomain1', $compparams->get('googledomain1', ''));
				$compparams->set('googlemaps-service.googledomain2', $compparams->get('googledomain2', ''));
				$compparams->set('googlemaps-service.googledomain3', $compparams->get('googledomain3', ''));
				$compparams->set('googlemaps-service.googledomainurl1', $compparams->get('googledomainurl1', ''));
				$compparams->set('googlemaps-service.googledomainurl2', $compparams->get('googledomainurl2', ''));
				$compparams->set('googlemaps-service.googledomainurl3', $compparams->get('googledomainurl3', ''));
				$compparams->set('googlemaps-service.googledomainkey1', $compparams->get('googledomainkey1', ''));
				$compparams->set('googlemaps-service.googledomainkey1', $compparams->get('googledomainkey1', ''));
				$compparams->set('googlemaps-service.googledomainkey2', $compparams->get('googledomainkey2', ''));

				$compparams->set('googlemaps-lookandfeel.staticmap', $compparams->get('staticmap', 0));
				$compparams->set('googlemaps-lookandfeel.maptype', $compparams->get('maptype', 'ROADMAP'));
				//$compparams->set('googlemaps-lookandfeel.google_style' ,      $compparams->get('google_style', 'standard'));
				//$compparams->set('googlemaps-lookandfeel.google_style_custom',$compparams->get('google_style_custom', ''));
				$compparams->set('googlemaps-lookandfeel.reducezoom', $compparams->get('reducezoom', 3));
				$compparams->set('googlemaps-lookandfeel.zoom', $compparams->get('zoom', 10));
				$compparams->set('googlemaps-lookandfeel.directionsmapquery', $compparams->get('directionsmapquery', 'coordinates'));

				$compparams->set('googlemaps-on_edit.longitude', $compparams->get('long', ''));
				$compparams->set('googlemaps-on_edit.latitude', $compparams->get('lat', ''));
				$compparams->set('googlemaps-on_edit.zoom', $compparams->get('zoom', 10));

				// set the provider last to make sure the others are all done already
				$compparams->set('mapsprovider', 'openstreetmaps');

				$db    = Factory::getDbo ();
				$query = "UPDATE #__extensions SET params = " . $db->quote($compparams->toString()) . "WHERE type='component' and element='com_jevlocations'";
				$db->setQuery($query);
				$db->execute();

			}
		}

	}

}
