<?php

/**
 * JEvents Component for Joomla 2.5.x
 *
 * @version     RSVPRO_VERSION  -  RSVPRO_DATE
 * @package     JEvents
 * @copyright   Copyright (C) 2008-JEVENTS_COPYRIGHT GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Version;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Installer\Installer;
use Joomla\Registry\Registry;
use Joomla\CMS\Router\Route;

class pkg_rsvpproInstallerScript
{
	public function preflight($type, $parent) {

		if (version_compare(PHP_VERSION, "7.0", "<")){
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('JEvents and its addons require PHP version 7 at least to run - please upgrade your server from version ' . PHP_VERSION, 'warning');
		}

		// Joomla! broke the update call, so we have to create a workaround check.
		$db = JFactory::getDbo ();
		$db->setQuery ( "SELECT * FROM #__extensions WHERE element = 'com_jevents' AND type='component' " );
		$jevents = $db->loadObject();

		if (is_null($jevents))
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('JEvents Must be installed first to use this addon.', 'warning');
			return false;
		}

		if ((int) $jevents->enabled !== 1)
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('JEvents has been disabled, please enable it first.', 'warning');
			return false;
		}

		$this->hasJEventsInst = 1;

		$manifestcache = @json_decode($jevents->manifest_cache, true);
		if (!$manifestcache)
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('Invalid JEvents Manifest Cache entry please download a freshcopy of JEvents and reinstall ', 'warning');
			return false;
		}

		if (version_compare($manifestcache["version"], "3.6.0", "lt"))
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('A minimum of JEvents V3.6.0 is required for this addon. <br/>Please update JEvents first. Current version is ' . $manifestcache["version"], 'warning');
			return false;
		}
		// remove old stripe library files.
		try
		{
			if (Folder::exists(JPATH_PLUGINS . "/rsvppro/stripe/vendor/"))
			{
				Folder::delete(JPATH_PLUGINS . "/rsvppro/stripe/vendor/");
			}
		}
		catch (Exception $exception)
		{
		//	\Joomla\CMS\Factory::getApplication()->enqueueMessage($exception->getMessage());
		}

	}

	// TODO enable plugins
	public function update()
	{
		//return $this->checkVirtuemartVersion();
	}

	public function install($adapter)
	{
		//return $this->checkVirtuemartVersion();
	}

	private function checkVirtuemartVersion(){
		// TRAP OUT OF DATE VERSIONS OF VIRTUEMART!
		$db = Factory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_virtuemart'");
		$is_vm= $db->loadResult();

		if (!$is_vm){
			return true;
		}

		$manifest  =  JPATH_ADMINISTRATOR . "/components/com_virtuemart/virtuemart.xml";
		$manifestdata = Installer::parseXMLInstallFile($manifest);

		if (!$manifestdata){
			return true;
		}

		if (version_compare( $manifestdata ["version"] , '3.0.6', "lt")) {
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('<strong>This version of RSVP Pro is only compatible with versions of Virtuemart later than V3.0.6.<strong><br/>We have therefore installed the older version of the Virtuemart plugins.<br/>You should upgrade Joomla and Virtuemart as soon as is convenient' , "warning");
			return false;
		}

		return true;

	}

	public function uninstall($adapter)
	{

	}

	/*
	 * enable the plugins
	 */
	function postflight($type, $parent)
	{
		// $parent is the class calling this method
		// $type is the type of change (install, update or discover_install)
		?>
		<style type="text/css">
			.adminform tr th:first-child {display:none;}
			table.adminform tr td {padding:15px;}
			div.jev_install {background-color:#f4f4f4;border:1px solid #ccc; border-radius:5px; padding:10px;}
			.installed {clear:both;display:inline-block;}
			.installed ul { width:350px;padding-left:0px;border: 1px solid #ccc;border-radius: 5px;}
			.installed ul li:first-child {border-top-left-radius: 5px;border-top-right-radius: 5px;}
			.installed ul li:last-child {border-bottom-left-radius: 5px;border-bottom-right-radius: 5px;}
			.installed ul li {padding:8px;list-style-type:none;}
			.installed ul li:nth-child(odd) {background-color: #fff;}
			.installed ul li:nth-child(even) {background-color: #D6D6D6;}
			.proceed {display:inline-block; vertical-align:top;}
			div.proceed ul {text-align:left;list-style-type:none;}
			div.proceed ul li {padding:5px;background-color:#fff;border:1px solid #ccc;margin-bottom:10px;border-radius:5px;}
		</style>
				<?PHP
		if ($type == 'install') {
			// enable plugin
			$db = Factory::getDbo();
			$query = "UPDATE #__extensions SET enabled=1 WHERE folder='jevents' and type='plugin' and element='jevrsvppro'";
			$db->setQuery($query);
			$db->execute();

			$query = "UPDATE #__extensions SET enabled=1 WHERE folder='rsvppro' and type='plugin' and element='manual'";
			$db->setQuery($query);
			$db->execute();

			// clean up any old install with the wrong extension name in the package!
			$query = "DELETE FROM #__extensions WHERE type='package' AND element='com_jevrsvppro' ";
			$db->setQuery($query);
			$db->execute();

		}

		// Disable simple RSVP
		$db = Factory::getDbo();
		$query = "UPDATE #__extensions SET enabled=0 WHERE folder='jevents' and type='plugin' and element='jevrsvp'";
		$db->setQuery($query);
		$db->execute();

		//Lets load the language strings directly from the installed component admin folder, loading the english first then any other after to override to avoid untranslated strings.
		$language = Factory::getLanguage();
		$language->load('com_rsvppro', JPATH_ADMINISTRATOR, 'en-GB', true);
		$language->load('com_rsvppro', JPATH_ADMINISTRATOR, null, true);

		// Remove component unstyled text
		$jversion = new Version;
		if ($jversion->isCompatible('4.0'))
		{
			$manifest = $parent->manifest;
		}
		else
		{
			$manifest = $parent->get("manifest");
		}

		// Remove component unstyled text
		if ($this->hasJEventsInst == 0) { $inst_text = Text::_('PKG_INST_JEVENTS_RSVPPRO_SUCC2'); } else {  $inst_text = Text::_('PKG_INST_JEVENTS_RSVPPRO_SUCC1') . $manifest->version;}
		echo "<style type='text/css'>table.adminform tbody tr th:first-child {display:none;}</style>";
		echo "<div class='jev_install'>
				<div class='jev_logo'><img src='https://www.jevents.net/images/JeventsTransparent.png' /></div>
				<div class='version'><h2>". $inst_text ."</h2></div>";
			echo '<div class="installed">
						<ul>
							<li>RSVP Pro Component
							<li>RSVP Pro Plugin</li>
							<li>RSVP Pro Virtuemart Tickets Plugin</li>
							<li>Payment Processor Plugin - Manual</li>
							<li>Payment Processor Plugin - PayPal IPN</li>
							<li>Payment Processor Plugin - PayPal Express</li>
							<li>Payment Processor Plugin - Authorize.net</li>
							<li>Payment Processor Plugin - Virtuemart 3</li>
							<li>Payment Processor Plugin - HikaShop</li>
							<li>Payment Processor Plugin - Mollie</li>
							<li>Payment Processor Plugin - Stripe</li>
							<li>Payment Processor Plugin - Quickpay</li>
						</ul><br/><br/>
					</div>
					<div class="proceed">
					<ul>';
						echo "<li style='background-color: #F5EF62;'>" . Text::_('PKG_INST_JEVENTS_RSVPPRO_NOTE_PLUGINS') . "</li>";
						echo "<li>" . Text::_('PKG_INST_JEVENTS_RSVPPRO_DESC') . "</li>";
					echo "</ul>
					</div>";
		echo "</div>";

        // Clean up stray install files
        if (File::exists(JPATH_SITE."/plugins/jevent/jev_rsvppro/rsvppro/rsvp.js.zip")){
                File::delete(JPATH_SITE."/plugins/jevent/jev_rsvppro/rsvppro/rsvp.js.zip");
        }

		// Check if we need to warn about paypal express changes!
		$query = "select * from  #__extensions WHERE enabled=1 AND folder='rsvppro' and type='plugin' and element='paypalexpress'";
		$db->setQuery($query);
		$paypalExpress = $db->loadObject();
		if ($paypalExpress)
		{
			$params = new Registry($paypalExpress->params);
			if ($params->get('ppe_PayPalMode', -987654) === -987654)
			{
				Factory::getApplication()->enqueueMessage(Text::_('PKG_INST_JEVENTS_RSVPPRO_PAYPALEXPRESS_WARNING'), 'warning');
			}
		}

		// Check if we need to warn about paypal IPN changes!
		$query = "select * from  #__extensions WHERE enabled=1 AND folder='rsvppro' and type='plugin' and element='paypalipn'";
		$db = Factory::getDbo();
		$db->setQuery($query);
		$paypalIpn = $db->loadObject();
		if ($paypalIpn)
		{
			$params = new Registry($paypalIpn->params);
			if ($params->get('pp_warning', -987654) === -987654)
			{
				Factory::getApplication()->enqueueMessage(Text::sprintf('PKG_INST_JEVENTS_RSVPPRO_PAYPALIPN_WARNING', Route::_("index.php?option=com_plugins&task=plugin.edit&extension_id=") . $paypalIpn->extension_id), 'warning');
			}
		}

	}


}
