<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: mod.defines.php 1400 2009-03-30 08:45:17Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2009 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 */
// no direct access
defined('_JEXEC') or die('Restricted access');
use Joomla\CMS\Table\Table;
use Joomla\CMS\Captcha\Captcha;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Utilities\ArrayHelper;

jimport('joomla.plugin.plugin');
JLoader::register('JevJoomlaVersion', JPATH_ADMINISTRATOR . "/components/com_jevents/libraries/version.php");

class plgJEventsJevanonuser extends CMSPlugin
{

	private
		$recaptchalang = "en";

	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);

		Factory::getLanguage()->load('plg_jevents_jevanonuser', JPATH_ADMINISTRATOR);

		if( !$this->params->get('jevanonuser-use-jevents-translations',0) )
		{
			$lang = Factory::getLanguage();

			list ($tag1, $tag2) = explode("-", $lang->getTag());
			// See http://recaptcha.net/apidocs/captcha/client.html for list of supported languages
			$langs = array("en", "nl", "fr", "de", "pt", "ru", "es", "tr");

			if (in_array($tag1, $langs))
			{
				$this->recaptchalang = $tag1;
			}
		}

	}

	// This enable anon users to create (but not edit events)
	public function isEventCreator(&$isEventCreator)
	{

		$user = Factory::getUser();
		// if logged in then do not change isEventCreator
		if ($user->id > 0 && !$this->params->get("allusers", 0))
			return true;

		if (!$this->params->get("allusers", 0))
			$isEventCreator = true;
		return true;

	}

	// This enable anon users to create (but not edit events)
	public function isEventPublisher($type, &$isEventPublisher)
	{

		$user = Factory::getUser();
		// if logged in then do not change isEventPublisher
		if ($user->id > 0 || $type == "strict")
			return true;

		$isEventPublisher = $this->params->get("canpublishown", 0);

	}

	/**
	 * Custom part of form for re-captcha and name/email
	 *
	 * @param unknown_type $row
	 * @param unknown_type $customfields
	 * @return unknown
	 */
	public function onEditCustom(&$row, &$customfields)
	{

		$user = Factory::getUser();
		$evid = $row->ev_id();
		if ($user->id > 0 && $row->ev_id() == 0 && !$this->params->get("allusers", 0))
			return true;

		if ($user->id == 0 && $row->ev_id() > 0 && $row->created_by() > 0)
			return true;

		// Only setup when editing an event (they should not be able to edit a repeat !!!)
		if (Factory::getApplication()->input->getString("jevtask", "") != "icalevent.edit")
			return;

		if($user->id == 0)
		{
			$this->addJSChecks();
		}

		$anonname = "";
		$anonemail = "";
		if ($row->ev_id() > 0)
		{
			$db = Factory::getDbo();
			$db->setQuery("SELECT * FROM #__jev_anoncreator where ev_id=" . (int)$row->ev_id());
			$anonrow = $db->loadObject();
			if ($anonrow)
			{
				$anonname = $anonrow->name;
				$anonemail = $anonrow->email;
			}
		}

		$this->addDataFields($customfields, $anonname, $anonemail );

		if ($user->id == 0)
		{
			$this->addJSChecks();
			switch( $this->params->get("typeofcaptcha", 'v1') )
			{
				case "none":
					$this->showCaptchaNone();
					break;
				case "v1":
					$customfields['recaptcha'] = $this->showCaptchaV1($user);
					break;
				case "v2":
					$customfields['recaptcha'] = $this->showCaptchaV2();
					break;
				case "joomla":
					$customfields['recaptcha'] = $this->showCaptchaJoomla();
					break;
			}
		}

		return true;

	}

	/**
	 * Custom part of form for re-captcha and name/email
	 *
	 * @param unknown_type $row
	 * @param unknown_type $customfields
	 * @return unknown
	 */
	public function onEditLocationCustom(&$location, &$customfields)
	{

		$user = Factory::getUser();

		$locId = $location->loc_id;
		if ($user->id > 0 && $locId == 0)
			return true;

		if($user->id == 0)
		{
			$this->addJSChecks();
		}

		$this->addDataFields($customfields, $location->anonname, $location->anonemail );

		if ($user->id == 0)
		{
			switch( $this->params->get("typeofcaptcha", 'v1') )
			{
				case "none":
					$this->showCaptchaNone();
					break;
				case "v1":
					$customfields['recaptcha'] = $this->showCaptchaV1($user);
					break;
				case "v2":
					$customfields['recaptcha'] = $this->showCaptchaV2();
					break;
				case "joomla":
					$customfields['recaptcha'] = $this->showCaptchaJoomla();
					break;
			}
		}

		return true;

	}

	public function addDataFields(&$customfields, $anonname, $anonemail)
	{
		$user = Factory::getUser();

		if ($user->id == 0)
		{
			$label = Text::_("JEV_ANON_NAME");
			$input = '<input size="50" type="text" name="custom_anonusername" id="custom_anonusername" class="inputbox" value="' . $anonname . '" />';
			$customfield = array("label" => $label, "input" => $input);
			$customfields["anonusername"] = $customfield;

			$label = Text::_("JEV_ANON_EMAIL");
			$input = '<input size="50" type="text" name="custom_anonemail" id="custom_anonemail" class="inputbox" value="' . $anonemail . '" />';
			$customfield = array("label" => $label, "input" => $input);
			$customfields["anonemail"] = $customfield;
		}
		else if ($anonemail)
		{
			// otherwise show the creator details
			$label = Text::_("JEV_CREATOR_NAME");
			//$input = '<span style="font-decoration:italic">' . $anonname . '</span>';
			$input = '<input size="50" type="text" name="custom_anonusername" id="custom_anonusername" class="inputbox" value="' . $anonname . '" />';
			$customfield = array("label" => $label, "input" => $input);
			$customfields["anonusername"] = $customfield;

			$label = Text::_("JEV_CREATOR_EMAIL");
			//$input = '<span style="font-decoration:italic">' . $anonemail . '</span>';
			$input = '<input size="50" type="text" name="custom_anonemail" id="custom_anonemail" class="inputbox" value="' . $anonemail . '" />';
			$customfield = array("label" => $label, "input" => $input);
			$customfields["anonemail"] = $customfield;
		}

		return $customfields;
	}

	public function onBeforeSaveEvent(&$array, &$rrule)
	{
		$user = Factory::getUser();

		if ($user->id > 0)
			return true;

		// make sure self publishing respects plugin settings and also set access level to public

		$array["access"] = 1;

		if (!$this->params->get("canpublishown", 0))
		{
			$params = ComponentHelper::getParams(JEV_COM_COMPONENT);
			$params->set("jevpublishown", 0);
		}

	}

	protected function showCaptchaJoomla()
	{
		$label = Text::_("JEV_ANON_RECAPTCHA_LABEL");

		$plugin = Factory::getApplication()->getParams()->get('captcha', Factory::getConfig()->get('captcha'));

		if ($plugin === 0 || $plugin === '0' || $plugin === '' || $plugin === null)
		{
			$mainframe= Factory::getApplication();
			$mainframe->setUserState("jevrecaptcha","error");
			return '';
		}

		if (($captcha = Captcha::getInstance($plugin, array('namespace' => 'jevanon'))) == null)
		{
			return '';
		}

		$input = $captcha->display("jevanoncaptcha", "jevanoncaptcha", " required");

		$customfield = array("label" => $label, "input" => $input);

		return $customfield;
	}

	protected function showCaptchaV1($user)
	{
		$document= Factory::getDocument();
		$root = JURI::root() . "plugins/jevents/jevanonuser/anonuserlib/";
		$customfield = "";

		if ($user->id == 0)
		{
			Text::script("JEV_MISSING_NAME_OR_EMAIL",true);
			HTMLHelper::script('plg_jevents_jevanonusers/anonevent.js', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => true), array('defer' => true));

			$label = Text::_("JEV_ANON_RECAPTCHA_LABEL");

			$plugin = Factory::getApplication()->getParams()->get('captcha', Factory::getConfig()->get('captcha'));
			if (version_compare(JVERSION, "3.3.0", 'ge') && $plugin && !$this->params->get("typeofcaptcha", ''))
			{
				$customfield = $this->showCaptchaJoomla();
				$checkScript[] = "anonurlroot = '$root';";
			}

			if (!$customfield && $this->params->get("recaptchapublic", false) && $this->params->get("recaptchaprivate", false) )
			{
				$ssl = false;

				if (!defined("RECAPTCHA_API_SERVER"))
				{
					require_once(dirname(__FILE__) . '/anonuserlib/recaptcha.php');
				}
				if(Factory::getApplication()->getParams()->get('force_ssl',0) == 2)
				{
					$ssl = true;
				}

				$input = recaptcha_get_html($this->params->get("recaptchapublic", false),null,$ssl);
				$customfield = array("label" => $label, "input" => $input);

				$token = Session::getFormToken();

				$recaptchaOptions = new stdClass();

				$recaptchaOptions->theme = "clean";
				$recaptchaOptions->lang = $this->recaptchalang;

				if( $this->params->get('jevanonuser-use-jevents-translations',0) )
				{
					$customTranslations = new stdClass();
					$customTranslations->instructions_visual = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_INSTRUCTIONS_VISUAL');
					$customTranslations->instructions_audio = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_INSTRUCTIONS_AUDIO');
					$customTranslations->play_again = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_PLAY_AGAIN');
					$customTranslations->cant_hear_this = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_CANT_HEAR_THIS');
					$customTranslations->visual_challenge = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_VISUAL_CHALLENGE');
					$customTranslations->audio_challenge = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_AUDIO_CHALLENGE');
					$customTranslations->refresh_btn = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_REFRESH_BTN');
					$customTranslations->help_btn = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_HELP_BTN');
					$customTranslations->incorrect_try_again = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_INCORRECT_TRY_AGAIN');
					$customTranslations->image_alt_text = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_IMAGE_ALT_TEXT');
					$customTranslations->privacy_and_terms = Text::_('PLG_JEVENTS_JEVANONUSER_RECAPTCHA_PRIVACY_AND_TERMS');
					$recaptchaOptions->custom_translations = $customTranslations;
				}

				$checkScript[] = "anonurlroot = '$root';";
				$checkScript[] = "RecaptchaOptions = " . json_encode($recaptchaOptions);

				$mainframe= Factory::getApplication();
				$mainframe->setUserState("jevrecaptcha","error");
			}


		}

		$checkscript = implode("\n",$checkScript) . ";";
		$document->addScriptDeclaration($checkscript);

		return $customfield;
	}

	protected function showCaptchaV2()
	{
		$hl = "";

		if($this->params->get("recaptchav2forcelanguage", '0'))
		{
			$hl="?hl=" . $this->params->get("recaptchav2forcelanguage", '0');
		}

		$document = Factory::getDocument();
		$document->addScript('https://www.google.com/recaptcha/api.js'.$hl,'text/javascript',true,true);

		$label = Text::_("JEV_ANON_RECAPTCHA_LABEL");

		$html =	'<div id="jevanoncaptcha" class="g-recaptcha v2 required"' .
			' data-sitekey="' . $this->params->get('recaptchav2public', '') .
			'" data-theme="' . $this->params->get('recaptchav2theme', 'light') .
			'" data-size="' . $this->params->get('recaptchav2size', 'normal') .
			'" data-type="' . $this->params->get('recaptchav2type', 'image') .
			'"></div>';

		$customfield = array("label" => $label, "input" => $html);

		return $customfield;
	}


	protected function showCaptchaNone()
	{
		return true;
	}

	protected function addJSChecks()
	{
		Text::script('JEV_RECAPTCHA_ERROR', true);
		Text::script('JEV_RECAPTCHAV2_ERROR', true);
		Text::script('JEV_ANON_RECAPTCHA_MISSING', true);
		Text::script("JEV_MISSING_NAME_OR_EMAIL",true);
		Text::script("JEV_ANON_MISSING_NAME",true);
		Text::script("JEV_ANON_MISSING_EMAIL",true);
		HTMLHelper::script('plg_jevents_jevanonusers/anonevent.js', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => true), array('defer' => true));

		$script = 'var jevtoken = "' . Session::getFormToken() . '"';

		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		return;
	}

	/**
	 * Store custom fields at event level
	 *
	 */
	public function onStoreCustomEvent($event)
	{

		$user = Factory::getUser();
		if ($user->id > 0 && $event->ev_id == 0)
			return true;

		// do I need to reset the created_by field to 0;
		$name = Factory::getApplication()->input->getString("custom_anonusername", "");
		$email = Factory::getApplication()->input->getString("custom_anonemail", "");

		if ($event->ev_id > 0 && ($name || $email))
		{
			$db = Factory::getDbo();
			$db->setQuery("SELECT * FROM #__jev_anoncreator where ev_id=" . (int) $event->ev_id);
			$anonrow = $db->loadObject();

			if ($anonrow)
			{
				$db->setQuery("UPDATE #__jevents_vevent SET created_by=0 WHERE ev_id=" . (int) $event->ev_id);
				$db->execute();

				$event->created_by = 0;
				// place private reference to created_by in event detail in case needed by plugins
				$event->_detail->_created_by = $event->created_by;

				$creator = new TableAnonCreator($db);
				$creator->id = $anonrow->id;
				$creator->name = $name;
				$creator->email = $email;
				$creator->store();
				return true;
			}
		}

		//It's a registered user
		if ($event->ev_id > 0 && $event->created_by > 0)
			return true;

		$eventid = $event->ev_id;

		switch( $this->params->get("typeofcaptcha", 'v1') )
		{
			case "none":
				$response = $this->validCaptcha();
				break;
			case "v1":
				$response = $this->checkRecaptchaV1();
				break;
			case "v2":
				$response = $this->checkRecaptchaV2();
				break;
			case "joomla":
				$response = $this->checkCaptchaJoomla();
				break;
		}



		if (!$response->is_valid)
		{
			$this->removeEvent($eventid);

			echo "<script> alert('" . Text::_("JEV_RECAPTCHA_ERROR", true) . "'); window.history.go(-1); </script>\n";
			exit();
		}
		else
		{
			if ($name != "" && $email != "")
			{
				$this->saveAnonymousData($email, $name, $eventid);
			}

		}


		return true;

	}

	public function onBeforeSaveLocation($location)
	{
		$user = Factory::getUser();

		//No captcha for registered users
		if ($user->id > 0)
		{
			return true;
		}

		$name = "";
		$email = "";
		if(isset($location["custom_anonusername"]) && isset($location["custom_anonemail"]))
		{
			$name = $location["custom_anonusername"];
			$email = $location["custom_anonemail"];
		}

		if (($name && $email))
		{
			switch( $this->params->get("typeofcaptcha", 'v1') )
			{
				case "none":
					$response = $this->validCaptcha();
					break;
				case "v1":
					$response = $this->checkRecaptchaV1();
					break;
				case "v2":
					$response = $this->checkRecaptchaV2();
					break;
				case "joomla":
					$response = $this->checkCaptchaJoomla();
					break;
			}
		}
		else if($location['loc_id'] == 0)
		{
			if(!$email && !$name)
			{
				echo "<script> alert('" . Text::_("JEV_MISSING_NAME_AND_EMAIL", true) . "'); window.history.go(-1); </script>\n";
			}
			else if(!$email)
			{
				echo "<script> alert('" . Text::_("JEV_ANON_MISSING_EMAIL", true) . "'); window.history.go(-1); </script>\n";
			}
			else
			{
				echo "<script> alert('" . Text::_("JEV_ANON_MISSING_NAME", true) . "'); window.history.go(-1); </script>\n";
			}
			exit();
		}

		if (!$response->is_valid)
		{
			echo "<script> alert('" . Text::_("JEV_RECAPTCHA_ERROR", true) . "'); window.history.go(-1); </script>\n";
			exit();
		}

		return true;
	}
	private function validCaptcha()
	{
		$response	= new stdClass();
		$response->is_valid = true;
		return $response;
	}

	private function checkRecaptchaV1()
	{
		$captcha = false;
		$mainframe = Factory::getApplication();
		$jevrecaptcha = $mainframe->getUserState("jevrecaptcha");
		$mainframe->setUserState("jevrecaptcha","");

		if($jevrecaptcha == "ok")
		{
			$response = $this->validCaptcha();
		}
		else if ($jevrecaptcha != "error")
		{
			//Check against Joomla! Captcha plugin keys
			$plugin = Factory::getApplication()->getParams()->get('captcha', Factory::getConfig()->get('captcha'));
			if ($plugin)
			{
				$captcha = Captcha::getInstance($plugin, array('namespace' => "jevanon"));
				if ($captcha)
				{
					// Test the value.
					$value = Factory::getApplication()->input->getString("jevanoncaptcha","");
					if (!$captcha->checkAnswer($value))
					{
						$error = $captcha->getError();

						$response	= new stdClass();
						$response->is_valid = false;
					}
					else {
						$response	= new stdClass();
						$response->is_valid = true;
					}
				}
			}

			if (!$captcha)
			{
				if ($this->params->get("recaptchaprivate", false))
				{
					$mainframe = Factory::getApplication();
					$jevrecaptcha = $mainframe->getUserState("jevrecaptcha");

					$name = Factory::getApplication()->input->getString("custom_anonusername", "");
					$email = Factory::getApplication()->input->getString("custom_anonemail", "");

					if ($jevrecaptcha == "ok" && $name != "" && $email != "")
					{
						$response = new stdClass();
						$response->is_valid = true;
						return $response;
					}
					// Belt and braces

					if (!defined("RECAPTCHA_API_SERVER"))
						require_once('anonuserlib/recaptcha.php');

					$response = recaptcha_check_answer($this->params->get("recaptchaprivate", false), Factory::getApplication()->input->getString("REMOTE_ADDR", "", "server"), Factory::getApplication()->input->getString("recaptcha_challenge_field"), Factory::getApplication()->input->getString("recaptcha_response_field"));
				}
				else
				{
					//If no private key we assume they do not want recaptcha. We keep this behavior as in previous versions but it should be deprecated for v2 and future upgrades.
					$response = $this->validCaptcha();
				}
			}
		}
		else
		{
			$response = new stdClass();
			$response->is_valid = false;
		}


		return $response;
	}

	private function checkRecaptchaV2()
	{
		$mainframe = Factory::getApplication();
		$jevrecaptcha = $mainframe->getUserState("jevrecaptcha");
		$mainframe->setUserState("jevrecaptcha","");

		if($jevrecaptcha == "ok")
		{
			$response = $this->validCaptcha();
		}
		else if ($jevrecaptcha != "error")
		{
			$input = Factory::getApplication()->input;
			$response  = $input->get('g-recaptcha-response', '', 'string');
			$privatekey = $this->params->get("recaptchav2private", '');
			$remoteip   = $input->server->get('REMOTE_ADDR', '', 'string');

			require_once('recaptchav2/jevrecaptchalib.php');

			if($privatekey)
			{
				$reCaptcha = new JevReCaptcha($privatekey);
				$response  = $reCaptcha->verifyResponse($remoteip, $response);
			}
		}
		else
		{
			$response = new stdClass();
			$response->is_valid = false;
		}

		return $response;
	}

	public function checkCaptchaJoomla()
	{
		$mainframe = Factory::getApplication();
		$jevrecaptcha = $mainframe->getUserState("jevrecaptcha");
		$mainframe->setUserState("jevrecaptcha","");

		$input = Factory::getApplication()->input;

		$plugin = Factory::getApplication()->getParams()->get('captcha', Factory::getConfig()->get('captcha'));
		$captcha = Captcha::getInstance($plugin, array('namespace' => 'jevanon'));

		if($jevrecaptcha == "ok")
		{
			$response = $this->validCaptcha();
		}
		else
		{
			if (!$captcha->checkAnswer(""))
			{
				$error = $captcha->getError();

				$response = new stdClass();
				$response->is_valid = false;
			}
			else
			{
				$response = $this->validCaptcha();
			}
		}


		return $response;
	}

	private function saveAnonymousData($email, $name, $eventid)
	{
		// Store the name and email address with the event
		$db = Factory::getDbo();
		$creator = new TableAnonCreator($db);
		$creator->id = 0;
		$creator->email = $email;
		$creator->name = $name;
		$creator->ev_id = $eventid;
		$creator->store();

		return true;
	}

	private function removeEvent($eventid)
	{
		// The event has already been saved - I need to delete it!
		$db = Factory::getDbo();

		$query = "SELECT detail_id FROM #__jevents_vevent WHERE ev_id IN ($eventid)";
		$db->setQuery($query);
		$detailidstring = $db->loadResult();

		$query = "DELETE FROM #__jevents_rrule WHERE eventid IN ($eventid)";
		$db->setQuery($query);
		$db->execute();

		$query = "DELETE FROM #__jevents_repetition WHERE eventid IN ($eventid)";
		$db->setQuery($query);
		$db->execute();

		$query = "DELETE FROM #__jevents_exception WHERE eventid IN ($eventid)";
		$db->setQuery($query);
		$db->execute();

		$query = "DELETE FROM #__jevents_vevdetail WHERE evdet_id IN ($detailidstring)";
		$db->setQuery($query);
		$db->execute();

		// I also need to clean out associated custom data

		// just incase we don't have jevents plugins registered yet
		PluginHelper::importPlugin("jevents");
		$res = Factory::getApplication()->triggerEvent('onDeleteEventDetails', array($detailidstring));

		$query = "DELETE FROM #__jevents_vevent WHERE ev_id IN ($eventid)";
		$db->setQuery($query);
		$db->execute();

		// I also need to delete custom data

		// just incase we don't have jevents plugins registered yet
		PluginHelper::importPlugin("jevents");
		$res = Factory::getApplication()->triggerEvent('onDeleteCustomEvent', array(&$eventid));

		return true;
	}


	public function onAfterSaveEvent(&$row, $dryrun)
	{
		if ($dryrun)
			return;
		// Make sure created_by is NOT reset during saving process
		if ($row->ev_id > 0)
		{
			$db = Factory::getDbo();
			$db->setQuery("SELECT * FROM #__jev_anoncreator where ev_id=" . intval($row->ev_id));
			$anonrow = $db->loadObject();
		}

	}

// attach creator details to event
	public function onDisplayCustomFields(&$row)
	{
		$authorname = "";
		$authoremail = "";
		if ($row->ev_id() > 0)
		{
			$db = Factory::getDbo();
			$db->setQuery("SELECT * FROM #__jev_anoncreator where ev_id=" . intval($row->ev_id()));
			$anonrow = $db->loadObject();
			if ($anonrow)
			{
				$authorname = $anonrow->name;
				$authoremail = $anonrow->email;
			}
		}

		$row->authorname = $authorname;
		$row->authoremail = $authoremail;
	}

    public function onDisplayCustomFieldsMultiRow(&$rows = array())
    {

        if(count($rows) === 0) {
            return;
        }

        $ids = array();

        foreach ($rows as $row) {
            $ids[] = $row->ev_id();
        }

        $db = Factory::getDbo();
        $db->setQuery("SELECT * FROM #__jev_anoncreator where ev_id IN (" . implode(',', $ids) . ")");
        $anonRows = $db->loadObjectList('ev_id');

        foreach ($rows as $key => $row) {
            if (!empty($anonRows[$row->ev_id()])) {
                $rows[$key]->authorname = $anonRows[$row->ev_id()]->name;
                $rows[$key]->authoremail = $anonRows[$row->ev_id()]->email;;
            }
        }
    }

	public static function fieldNameArray($layout = 'detail')
	{

		$return = array();
		$return['group'] = Text::_("JEV_ANON_CREATOR", true);

		$labels = array();
		$values = array();

		$labels[] = Text::_("JEV_CREATOR_NAME", true);
		$values[] = "JEV_CNAME";

		$labels[] = Text::_("JEV_CREATOR_EMAIL", true);
		$values[] = "JEV_CEMAIL";

		if ($layout == "edit")
		{
			$labels = array();
			$values = array();

			$labels[] = Text::_("JEV_CREATOR_NAME", true);
			$values[] = "anonusername";

			$labels[] = Text::_("JEV_CREATOR_EMAIL", true);
			$values[] = "anonemail";

			$labels[] = Text::_("JEV_ANON_CAPTCHA_CODE", true);
			$values[] = "recaptcha";
		}

		$return['values'] = $values;
		$return['labels'] = $labels;

		return $return;

	}

	public static function substitutefield($row, $code)
	{
		if (isset($row->hidedetail) && $row->hidedetail){
			return "";
		}

		if ($code == "JEV_CNAME")
		{
			if (isset($row->authorname))
				return $row->authorname;
		}
		if ($code == "JEV_CEMAIL")
		{
			if (isset($row->authoremail) && $row->authoremail != "")
				return JHTML::_('email.cloak', $row->authoremail, 0);
		}

		return "";

	}

	/* restrict calendar choices */
	public function onSelectIcals(& $query)
	{
		if (Factory::getApplication()->input->getCmd('jevtask') === "icalevent.edit")
		{
			$cals = $this->params->get('calendars', array());
			if (!empty($cals) && count($cals))
			{
				$query = str_replace("AND ical.state = 1", "AND ical.state = 1 AND ical.ics_id IN ( " . implode(', ', $cals). " )", $query);
			}

		}
	}

	/* restrict category selection */

	public function onGetAccessibleCategories(&$cats, $reindex = true)
	{

		$user = Factory::getUser();
		// if logged in then do not change isEventCreator
		if ($user->id > 0)
			return true;

		if (is_array($cats))
			return true;

		if (Factory::getApplication()->input->getString("task") != "icalevent.edit" && Factory::getApplication()->input->getString("task") != "icalrepeat.edit"
			&& Factory::getApplication()->input->getString("jevtask") != "icalevent.edit" && Factory::getApplication()->input->getString("jevtask") != "icalrepeat.edit")
			return true;

		$anoncats = $this->params->get("cats", "");
		if (is_string($anoncats) && strlen($anoncats) > 0)
		{
			$anoncats = str_replace(" ", "", $anoncats);
			$anoncats = explode(",", $anoncats);
		}
		else if (!is_array($anoncats))
		{
			$anoncats = array();
		}
		if (count($anoncats) == 0)
			return true;

		$anoncats = ArrayHelper::toInteger($anoncats);

		$anoncats[] = -1;

		$incats = explode(",", $cats);
		$anoncats =ArrayHelper::toInteger($anoncats);

		$incats = array_intersect($anoncats, $incats);
		$cats = implode(",", $incats);

	}

	/* restrict category selection during editing phase */

	public function onGetAccessibleCategoriesForEditing(&$cats, $reindex = true)
	{
		return;
		$user = Factory::getUser();
		// if logged in then do not change isEventCreator
		if ($user->id > 0)
			return true;

		$anoncats = $this->params->get("cats", "");
		if (is_string($anoncats) && strlen($anoncats) > 0)
		{
			$anoncats = str_replace(" ", "", $anoncats);
			$anoncats = explode(",", $anoncats);
		}
		else
		{
			$anoncats = array();
		}

		if (count($anoncats) == 0)
			return true;

		if (Factory::getApplication()->input->getString("task") != "icalevent.edit" && Factory::getApplication()->input->getString("task") != "icalevent.save"  && Factory::getApplication()->input->getString("task") != "icalevent.savenew"  && Factory::getApplication()->input->getString("task") != "icalevent.apply"){
			return true;
		}

	}

}

class TableAnonCreator extends Table
{

	public
	function __construct(&$db)
	{
		parent::__construct('#__jev_anoncreator', 'id', $db);

	}

}
