<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: jevpaidsubs.php 2591 2011-09-19 09:37:10Z geraintedwards $
 * @package     JEvents
 * @copyright   Copyright (C) 2009 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
JLoader::register('JevJoomlaVersion', JPATH_ADMINISTRATOR . "/components/com_jevents/libraries/version.php");

class plgJEventsJevpaidsubs extends JPlugin
{

    // the no shipping category name
    private $catname = 'Event Tokens';
    private $pid = 0;
    private $catid = 0;

    function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);

        JFactory::getLanguage()->load('plg_jevents_jevpaidsubs', JPATH_ADMINISTRATOR);

    }

    // This enables paying users to create (but not edit events) subject to payment
    function isEventCreator(&$isEventCreator)
    {

        if ($isEventCreator) {
            // Any extra categories to be setup?
            $jevuser = JEVHelper::getAuthorisedUser();
            if ($jevuser && $jevuser->categories != "all") {

                $extracats = implode(",", $this->params->get("specialcats2", array(-1)));
                if ($extracats != "") {
                    $jevuser->categories .= "|" . str_replace(",", "|", $extracats);
                }
            }
            return;
        }

        $user = JFactory::getUser();
        // if not logged in then do not change isEventCreator - we could have anon user plugin enabled
        if ($user->id == 0)
            return true;

        $document = JFactory::getDocument();
        //No Longer needed in J1.7+
        if (!version_compare(JVERSION, "1.7.0", 'ge')) {
            JHTML::script('includes/js/joomla.javascript.js');
        }


        $orderinfo = $this->getOrderSummary();
        $this->updateUserGroup($orderinfo);

        // if no orders then leave well alone
        if (!$orderinfo)
            return;

        if ($orderinfo['eventsAllowed'] > $orderinfo["eventsUsed"]) {
            $isEventCreator = true;
        }

    }

    // This enables paying users to create (but not edit l) subject to payment
    function isLocationCreator(&$isLocationCreator)
    {

        if ($this->params->get("createlocations") && !$isLocationCreator) {
            $user = JFactory::getUser();
            // if not logged in then do not change $isLocationCreator - we could have anon user plugin enabled
            if ($user->id == 0)
                return true;

            $orderinfo = $this->getOrderSummary();
            $this->updateUserGroup($orderinfo);
            // if no orders then leave well alone
            if (!$orderinfo)
                return;

            if ($orderinfo['eventsAllowed'] > $orderinfo["eventsUsed"]) {
                $isLocationCreator = true;
            }
        }

    }

    /* restrict category selection */

    function onGetAccessibleCategories(&$cats, $reindex = true)
    {

        $user = JFactory::getUser();

        if (is_array($cats))
            return true;

        if (JRequest::getString("task") != "icalevent.edit" && JRequest::getString("task") != "icalrepeat.edit")
            return true;

        $specialcats2 = $this->params->get("specialcats2", array());
        if (count($specialcats2) == 0) {
            return true;
        }

        $specialcats2[] = -1;

        $incats = explode(",", $cats);
        JArrayHelper::toInteger($specialcats2);

        $incats = array_merge($specialcats2, $incats);
        $cats = implode(",", $incats);

    }

    /* restrict category selection during editing phase */

    function onGetAccessibleCategoriesForEditing(&$cats, $reindex = true)
    {
        // must convert from and to array
        $cats = implode(",", $cats);
        $this->onGetAccessibleCategories($cats, $reindex);
        $cats = explode(",", $cats);

    }

    public function getOrderSummary()
    {

        $results = array();
        $results['orders'] = $this->getOrders();
        if (count($results['orders']) == 0)
            return false;

        $eventsAllowed = 0;
        foreach ($results['orders'] as $order) {
            //$eventsAllowed += intval($order->product_quantity);
            $eventsAllowed += intval($order->customfield_value);
        }
        $results['eventsAllowed'] = $eventsAllowed;

        $results['eventsUsed'] = $this->getCompletedEvents();

        return $results;

    }

    public function getOrders()
    {

        // Check this user's purchases
        $user = JFactory::getUser();

        $db = JFactory::getDBO();
        // TODO custom field count!

        $sql = "SELECT ord.*, orit.product_quantity, pcf.customfield_value FROM #__virtuemart_orders as ord"
            . " \n LEFT JOIN #__virtuemart_order_items as orit ON orit.virtuemart_order_id = ord.virtuemart_order_id"
            //. " \n LEFT JOIN #__virtuemart_products as pt ON pt.virtuemart_product_id = orit.virtuemart_product_id"
            . " \n LEFT JOIN #__virtuemart_product_categories as pcat ON pcat.virtuemart_product_id = orit.virtuemart_product_id"
            . " \n LEFT JOIN #__virtuemart_product_customfields as pcf ON pcf.virtuemart_product_id = orit.virtuemart_product_id"
            // Removed option U as this applies for people mailing in payment.
            //	. " \n WHERE (orit.order_status='C' OR orit.order_status='S' OR orit.order_status='I' OR orit.order_status='U') "
            . " \n WHERE (orit.order_status='C' OR orit.order_status='S' OR orit.order_status='I') "

            . " \n AND ord.virtuemart_user_id=" . $user->id
            . " \n AND pcat.virtuemart_category_id=" . $this->params->get("paidsubscat", -1)
            . " \n AND pcf.virtuemart_custom_id=" . $this->params->get("paidsubscustomfield", -1)
            . " \n ORDER BY created_on desc";

        $db->setQuery($sql);

        // Make sure Joomfish doesn't translate this
        $orders = $db->loadObjectList();
        echo $db->getErrorMsg();

        return $orders;

    }

    /**
     * Get list of events associated with this plugin
     */
    public function getCompletedEvents()
    {

        $user = JFactory::getUser();
        $db = JFactory::getDBO();
        $db->setQuery("SELECT count(DISTINCT ev_id) FROM #__jev_subevents WHERE user_id=" . $user->id);
        $events = intval($db->loadResult());

        return $events;

    }

    // Is this user authorised to publish this event?
    function isEventPublisher($type, &$isEventPublisher)
    {

        $user = JFactory::getUser();
        // if not logged in then do not change isEventPublisher
        if ($user->id == 0 || $type == "strict")
            return true;

        $orderinfo = $this->getOrderSummary();

        // if no orders then leave well alone
        if (!$orderinfo)
            return;

        if ($orderinfo['eventsAllowed'] > $orderinfo["eventsUsed"] && !$isEventPublisher) {
            $isEventPublisher = $this->params->get("canpublishown", 0);
        }

    }

    /**
     * Custom part of form for paying users ??
     *
     * @param unknown_type $row
     * @param unknown_type $customfields
     * @return unknown
     */
    function onEditCustom(&$row, &$customfields)
    {

        $user = JFactory::getUser();

        if ($user->id > 0 && $row->ev_id() == 0)
            return true;

        if ($row->ev_id() > 0 && $row->created_by() > 0)
            return true;

        // Only setup when editing an event (they should not be able to edit a repeat !!!)
        if (JRequest::getString("jevtask", "") != "icalevent.edit")
            return;

        return true;

    }

    function onBeforeSaveEvent(&$array, &$rrule)
    {
        // make sure self publishing respects plugin settings and also set access level to public
        // TODO Make this Joomla 1.7 compatible
        //$array["access"] = 0;

        if (!$this->params->get("canpublishown", 0)) {
            $params = JComponentHelper::getParams(JEV_COM_COMPONENT);
            $params->set("jevpublishown", 0);
        }
        if ($this->params->get("prioritylevel", 0) >= 1) {

            if ($array["priority"] == 0) {
                $array["priority"] = $this->params->get("prioritylevel");
            }
        }


    }

    /**
     * Store custom fields at event level
     *
     */
    function onStoreCustomEvent($event)
    {

        $user = JFactory::getUser();
        if ($user->id === 0) {
            return true;
        }
        if (JRequest::getInt("ev_id", 0) > 0) {
            return true;
        }

        $columns = array(ev_id, user_id);
        $values = array($event->ev_id, $user->id);

        //Look for events with same id first
        $db = JFactory::getDBO();

        $query = $db->getQuery(true);

        $query
            ->select($db->quoteName($columns))
            ->from($db->quoteName('#__jev_subevents'))
            ->values(implode(',', $values))
            ->where($db->quoteName('ev_id') . '=' . $event->ev_id);
        $db->setQuery($query);
        $ex_result = $db->loadObjectList();

        if (!$ex_result) {

            //Lets insert!
            $db = JFactory::getDBO();

            $query = $db->getQuery(true);

            $query
                ->insert($db->quoteName('#__jev_subevents'))
                ->columns($db->quoteName($columns))
                ->values(implode(',', $values));
            $db->setQuery($query);
            $db->query();
        }


        return true;

    }

    public function getProducts()
    {

        // Now create the product type and parameters
        $db = JFactory::getDBO();
        $typeid = $this->pid;
        $db->setQuery("SELECT * FROM #__virtuemart_products as prod
				LEFT JOIN #__virtuemart_product_categories as pcat on prod.virtuemart_product_id=pcat.virtuemart_product_id
				where pcat.virtuemart_category_id= '" . $this->params->get("paidsubscat", -1) . "'");
        $products = $db->loadObjectList();
        return $products;

    }

    public function getCategory()
    {

        if ($this->catid == 0) {
            $db = JFactory::getDBO();
            $db->setQuery("SELECT virtuemart_category_id FROM #__virtuemart_categories where virtuemart_category_id = '" . $this->params->get("paidsubscat", -1) . "'");
            $this->catid = $db->loadResult();
        }
        return $this->catid;

    }

    private function updateUserGroup($results)
    {
        //Upgrade usergroup
        //Check if upgrade usergroup is enabled
        if ($this->params->get("upgradeuser", 1)) {
            // Now preform the usergroup insert.

            $user = JFactory::getUser();
            $user_id = $user->id;
            $usergroup = $this->params->get("jusergroup");
            $db = JFactory::getDBO();
            $db->setQuery("SELECT group_id FROM #__user_usergroup_map WHERE user_id='" . $user_id . "' AND group_id='" . $usergroup . "'");
            $db->query();
            $result = $db->loadResult();

            if ($result <> $usergroup) {
                if ($results['eventsAllowed'] > $results['eventsUsed']) {
                    $db->setQuery("INSERT INTO #__user_usergroup_map (user_id, group_id) VALUES ('$user_id','$usergroup')");
                    $db->query();
                    //JFactory::getLanguage()->load();
                    JFactory::getApplication()->enqueueMessage(JText::_('JEV_PAID_SUBS_LOGOUT_FOR_EFFECT'), 'message');
                }
            } elseif ($result == $usergroup) {

                //Check to see if we need to remove the usergroup
                if (!$results || $results['eventsAllowed'] <= $results['eventsUsed']) {
                    if ($this->params->get("removeusergroup", 1)) {
                        $user = JFactory::getUser();
                        $user_id = $user->id;
                        $db = JFactory::getDBO();
                        $db->setQuery("DELETE FROM #__user_usergroup_map WHERE user_id='" . $user_id . "' AND group_id='" . $usergroup . "'");
                        $db->query();
                    }
                }
            }
            return $results;
        } else {
            return '';
        }

    }

}
