<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2018 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        http://www.jevents.net
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;

JLoader::register('jevFilterProcessing', JPATH_SITE . "/components/com_jevents/libraries/filters.php");
JLoader::register('JevJoomlaVersion', JPATH_ADMINISTRATOR . "/components/com_jevents/libraries/version.php");
$mainframe = Factory::getApplication();  // RSH 11/10/10 - Make J!1.6 compatible
if ($mainframe->isClient('administrator'))
{
	return;
}

jimport('joomla.plugin.plugin');


class plgJEventsJevtimelimit extends CMSPlugin
{

	function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
		$lang = Factory::getLanguage();
		$lang->load("plg_jevents_jevtimelimit", JPATH_SITE);
		$lang->load("plg_jevents_jevtimelimit", JPATH_ADMINISTRATOR);
	}

	/**
	 * When editing a JEvents menu item/module can add additional menu constraints dynamically
	 *
	 */
	function onEditMenuItem(&$menudata, $value, $control_name, $name, $id, $param)
	{
		// already done this param
		if (isset($menudata[$id])) return;

		static $matchingextra = null;
		// find the parameter that matches jevtl: (if any)
		if (!isset($matchingextra))
		{
			$params = $param->getGroup('params');
			foreach ($params as $key => $element)
			{
				$val = $element->value;
				if (strpos($key, "jform_params_extras") === 0)
				{
					if (strpos($val, "jevtl:") === 0)
					{
						$matchingextra = $key;
						break;
					}
				}
			}
			if (!isset($matchingextra))
			{
				$matchingextra = false;
			}
		}

		// either we found matching extra and this is the correct id or we didn't find matching extra and the value is blank
		if (strpos($value, "jevtl:") === 0 || (($value == "" || $value == "0") && $matchingextra === false))
		{
			$matchingextra = true;
			$invalue       = trim($value);
			if ($invalue == "") $invalue = 'jevtl:-1,-1';
			$value = str_replace("jevtl:", "", $invalue);

			$input = '<input type="text"  name="jevtl"  value="' . $value . '" onchange="document.getElementById(\'jevtl\').value=\'jevtl:\'+this.value;" />';
			$input .= '<input type="hidden"  name="' . $name . '"  id="jevtl" value="' . $invalue . '" />';

			$data              = new stdClass();
			$data->name        = "jevuser";
			$data->html        = $input;
			$data->label       = "JEV_DATE_RESTRICTIONS";
			$data->description = "JEV_DATE_RESTRICTIONS_DESC";
			$data->options     = array();
			$menudata[$id]     = $data;
		}
	}

	function onEditCustom(&$row, &$customfields)
	{

		if (!$this->params->get("eventspecific", 0))
		{
			return;
		}

		$detailid = intval($row->evdet_id());
		$timelimits = false;
		if ($detailid > 0)
		{
			$db = Factory::getDbo();
			$sql = "SELECT * FROM #__jev_timelimits WHERE evdet_id = " . $detailid;
			$db->setQuery($sql);
			$timelimits = $db->loadObject();
		}


		$this->dateformat = $this->params->get("dateformat", "Y-m-d");
		$minyear          = JEVHelper::getMinYear();
		$maxyear          = JEVHelper::getMaxYear();
		$script           = '';

		$label = Text::_("JEV_TIMELIMIT_START");
		$val   = $timelimits && $timelimits->startlimit !== "0000-00-00 00:00:00" ? $timelimits->startlimit : "";

		// $val is stored in the database in MySQL format
		$showtime = strpos($this->dateformat, ":i") ? "showtime" : "";
		$val = date_create_from_format("Y-m-d H:i:s", $val);
		if (!$val)
		{
			$val = date("Y-m-d H:i:s");
			$val = date_create_from_format("Y-m-d H:i:s", $val);
		}
		$val = $val->format($this->dateformat) ;

		ob_start();
		JEVHelper::loadElectricCalendar("custom_starttimelimit", "starttimelimit", $val, $minyear, $maxyear, '', $script, $this->dateformat  ,
			array("maxlength" => "16", "size" => "16", "showtime" => $showtime), true);

		$input = ob_get_clean();

		$customfield                       = array("label" => $label, "input" => $input, "default_value" => "", "id_to_check" => "starttimelimit");
		$customfields["jevtimelimitstart"] = $customfield;

		$label = Text::_("JEV_TIMELIMIT_END");
		$val   = $timelimits && $timelimits->endlimit !== "0000-00-00 00:00:00" ? $timelimits->endlimit : "";

		// $val is stored in the database in MySQL format
		$showtime = strpos($this->dateformat, ":i") ? "showtime" : "";
		$val = date_create_from_format("Y-m-d H:i:s", $val);
		if (!$val)
		{
			$val = date("Y-m-d H:i:s");
			$val = date_create_from_format("Y-m-d H:i:s", $val);
		}
		$val = $val->format($this->dateformat) ;

		ob_start();
		JEVHelper::loadElectricCalendar("custom_endtimelimit", "endtimelimit", $val, $minyear, $maxyear, '', $script, $this->dateformat ,
			array("maxlength" => "16", "size" => "16", "showtime" => $showtime), true);

		$input = ob_get_clean();

		$customfield                     = array("label" => $label, "input" => $input, "default_value" => "", "id_to_check" => "endtimelimit");
		$customfields["jevtimelimitend"] = $customfield;

		return true;
	}

	/**
	 * Store custom fields
	 *
	 * @param iCalEventDetail $evdetail
	 */
	function onStoreCustomDetails($evdetail)
	{

		$detailid          = intval($evdetail->evdet_id);
		$jevtimelimitstart = array_key_exists("starttimelimit", $evdetail->_customFields) ? $evdetail->_customFields["starttimelimit"] : false;
		$jevtimelimitend   = array_key_exists("endtimelimit", $evdetail->_customFields) ? $evdetail->_customFields["endtimelimit"] : false;

		$db = Factory::getDbo();

		$this->dateformat = $this->params->get("dateformat", "Y-m-d");

		// Delete any constraints
		if (!$jevtimelimitstart && !$jevtimelimitend || ($jevtimelimitstart  == $jevtimelimitend))
		{
			// delete
			$sql = "DELETE FROM #__jev_timelimits WHERE evdet_id=" . $detailid;
			$db->setQuery($sql);
			$db->execute();

			return true;
		}

		if ($jevtimelimitstart)
		{
			$jevtimelimitstart = date_create_from_format($this->dateformat ,  $jevtimelimitstart);
			$jevtimelimitstart = $jevtimelimitstart->format('Y-m-d H:i:s');
		}
		else
		{
			$jevtimelimitstart = "0000-00-00 00:00:00";
		}

		if ($jevtimelimitend)
		{
			$jevtimelimitend = date_create_from_format($this->dateformat ,  $jevtimelimitend);
			$jevtimelimitend = $jevtimelimitend->format('Y-m-d H:i:s');
		}
		else
		{
			$jevtimelimitend = "0000-00-00 00:00:00";
		}


		$sql = "REPLACE INTO #__jev_timelimits SET startlimit=" . $db->quote($jevtimelimitstart) . ", endlimit=" . $db->quote($jevtimelimitend) . ",  evdet_id=" . $detailid;
			$db->setQuery($sql);
			$db->execute();

		return true;

	}


	function onListIcalEvents(& $extrafields, & $extratables, & $extrawhere, & $extrajoin, & $needsgroupdby = false)
	{
		$mainframe = Factory::getApplication();  // RSH 11/10/10 - Make J!1.6 compatible
		// do not do this since it will stop the plugin from working with newsletter components etc.
		if ($mainframe->isClient('administrator'))
		{
			// return;
		}

		// Have we specified specific people for the menu item
		$compparams = ComponentHelper::getParams(Factory::getApplication()->input->getCmd("option", "com_jevents"));

		// If loading from a module then get the modules params from the registry
		$reg       = Factory::getConfig();
		$modparams = $reg->get("jev.modparams", false);

		if ($modparams)
		{
			$compparams = $modparams;
		}

		for ($extra = 0; $extra < 20; $extra++)
		{
			$extraval = $compparams->get("extras" . $extra, false);
			if (strpos($extraval, "jevtl:") === 0)
			{
				$extraval = str_replace("jevtl:", "", $extraval);
				break;
			}
			$extraval == false;
		}

		$pastdate   = false;
		$futuredate = false;

		if ($extraval && strpos($extraval, ","))
		{
			list($past, $future) = explode(",", $extraval);
			$this->dmap = "rpt";

			JLoader::register('JevDate', JPATH_SITE . "/components/com_jevents/libraries/jevdate.php");

			if (!is_numeric($past))
			{
				if ($past != "" && $future != "" && $future != "-1")
				{
					$pastdate   = new JevDate($past);
					$futuredate = new JevDate($future);
					if (!$modparams || $modparams->get("startnow", 1))
					{
						$pastdate   = $pastdate->toFormat("%Y-%m-%d %H:%M:%S");
						$futuredate = $futuredate->toFormat("%Y-%m-%d %H:%M:%S");
					}
					else
					{
						$pastdate   = $pastdate->toFormat("%Y-%m-%d  00:00:00");
						$futuredate = $futuredate->toFormat("%Y-%m-%d 23:59:59");
					}
					$extrawhere[] = "(" . $this->dmap . ".endrepeat>='$pastdate' AND " . $this->dmap . ".startrepeat<='$futuredate')";
				}
				else if ($past != "")
				{
					$pastdate = new JevDate($past);
					if (!$modparams || $modparams->get("startnow", 1))
					{
						$pastdate = $pastdate->toFormat("%Y-%m-%d %H:%M:%S");
					}
					else
					{
						$pastdate = $pastdate->toFormat("%Y-%m-%d  00:00:00");
					}
					$extrawhere[] = $this->dmap . ".endrepeat>='$pastdate'";
				}
				else if ($future != "" && $future != "-1")
				{
					$futuredate = new JevDate($future);
					if (!$modparams || $modparams->get("startnow", 1))
					{
						$futuredate = $futuredate->toFormat("%Y-%m-%d %H:%M:%S");
					}
					else
					{
						$futuredate = $futuredate->toFormat("%Y-%m-%d 23:59:59");
					}
					$extrawhere[] = $this->dmap . ".startrepeat<='$futuredate'";
				}
				//echo array_slice($extrawhere, -1)[0];
			}
			else if ($past >= 0 && $future >= 0)
			{
				$pastdate     = new JevDate("-$past days");
				$pastdate     = $pastdate->toFormat("%Y-%m-%d 00:00:00");
				$futuredate   = new JevDate("+$future days");
				$futuredate   = $futuredate->toFormat("%Y-%m-%d 23:59:59");
				$extrawhere[] = "(" . $this->dmap . ".endrepeat>='$pastdate' AND " . $this->dmap . ".startrepeat<='$futuredate')";
			}
			else if ($past >= 0)
			{
				$pastdate     = new JevDate("-$past days");
				$pastdate     = $pastdate->toFormat("%Y-%m-%d 00:00:00");
				$extrawhere[] = $this->dmap . ".endrepeat>='$pastdate'";
			}
			else if ($future >= 0)
			{
				$futuredate   = new JevDate("+$future days");
				$futuredate   = $futuredate->toFormat("%Y-%m-%d 23:59:59");
				$extrawhere[] = $this->dmap . ".startrepeat<='$futuredate'";
			}
		}
		$reg->set("jev.timelimit.past", $pastdate);
		$reg->set("jev.timelimit.future", $futuredate);

		// And the normal filters too!
		if (!(int) Factory::getApplication()->input->getInt('filter_reset', 0))
		{
			$pluginsDir = JPATH_ROOT . '/plugins/jevents';
			$filters    = jevFilterProcessing::getInstance(array("timelimit", "beforedate", "afterdate", "startingafterdate", "endingbeforedate"), $pluginsDir . "/filters/");

			$filters->setWhereJoin($extrawhere, $extrajoin);
		}
		else
		{
			if (Factory::getApplication()->input->post->getString('startdate', 0))
			{
				Factory::getApplication()->input->set('startdate', "");
			}
			if (Factory::getApplication()->input->post->getString('enddate', 0))
			{
				Factory::getApplication()->input->set('enddate', "");
			}
		}

		if (!$this->params->get("eventspecific", 0))
		{
			return true;
		}

		$extrajoin[] = "#__jev_timelimits as tl ON tl.evdet_id = det.evdet_id";
		//$extrajoin[] = "#__jev_timelimits as tl ON tl.evdet_id = ev.detail_id";

		$now = JevDate::getDate("+0 seconds");
		$db = Factory::getDbo();
		$now = $db->Quote($now->toMySql());

		$extrawhere[] = "( tl.evdet_id IS NULL OR tl.startlimit = '0000-00-00 00:00:00' OR tl.startlimit <= " . $now . ") ";
		$extrawhere[] = "( tl.evdet_id IS NULL OR tl.endlimit = '0000-00-00 00:00:00' OR tl.endlimit >= " . $now . ") ";

		return true;

	}

	function onListEventsById(& $extrafields, & $extratables, & $extrawhere, & $extrajoin)
	{
		$mainframe = Factory::getApplication();  // RSH 11/10/10 - Make J!1.6 compatible
		if ($mainframe->isClient('administrator'))
		{
			return;
		}
		$pluginsDir = JPATH_ROOT . '/plugins/jevents';
		if (!(int) Factory::getApplication()->input->getInt('filter_reset', 0))
		{

			$filters = jevFilterProcessing::getInstance(array("timelimit", "beforedate", "afterdate", "startingafterdate", "endingbeforedate"), $pluginsDir . "/filters/");

			$filters->setWhereJoin($extrawhere, $extrajoin);
		}

		if (!$this->params->get("eventspecific", 0))
		{
			return true;
		}

		$extrajoin[] = "#__jev_timelimits as tl ON tl.evdet_id = det.evdet_id";
		//$extrajoin[] = "#__jev_timelimits as tl ON tl.evdet_id = ev.detail_id";

		$now = JevDate::getDate("+0 seconds");
		$db = Factory::getDbo();
		$now = $db->Quote($now->toMySql());

		$extrawhere[] = "( tl.evdet_id IS NULL OR tl.startlimit = '0000-00-00 00:00:00' OR tl.startlimit <= " . $now . ") ";
		$extrawhere[] = "( tl.evdet_id IS NULL OR tl.endlimit = '0000-00-00 00:00:00' OR tl.endlimit >= " . $now . ") ";

		return true;
	}

	static function fieldNameArray($layout = 'detail')
	{
		if ($layout == "edit")
		{
			$return = array();
			$labels = array();
			$values = array();

			//Factory::getLanguage()->load( 'plg_jevents_jevmetatags',JPATH_ADMINISTRATOR );

			$return['group'] = Text::_("JEV_TIMELIMIT", true);

			$labels[] = Text::_("JEV_TIMELIMIT_START");
			$values[] = "jevtimelimitstart";

			$labels[] = Text::_("JEV_TIMELIMIT_END");
			$values[] = "jevtimelimitend";

			$return['values'] = $values;
			$return['labels'] = $labels;

			return $return;
		}

		return array();
	}

	function onJevFinderIndexing(&$rows) {

		// get all the attendance id info in one go
		$eventids = array(0);
		foreach ($rows as & $row) {
			if (empty($row))
			{
				continue;
			}
			$eventids[] = (int) isset($row->_detail_id) ? $row->_detail_id : $row->ev_id();
		}
		$db = Factory::getDbo();
		$sql = "SELECT * FROM #__jev_timelimits WHERE evdet_id in (" . implode(",", $eventids) . ")";
		$db->setQuery($sql);
		$rsvpdataArray = $db->loadObjectList('evdet_id');

		foreach ($rows as & $row) {
			if (empty($row))
			{
				continue;
			}
			if (isset($row->hidedetail) && $row->hidedetail) {
				continue;
			}

			$eventid = (int) isset($row->_detail_id) ? $row->_detail_id : $row->ev_id();
			if ($eventid > 0 && array_key_exists($eventid, $rsvpdataArray)) {
				$row->timelimits = $rsvpdata = $rsvpdataArray[$eventid];
			}
		}
		unset($row);
	}
}
