/*!
 * Packery layout mode PACKAGED v1.1.3
 * sub-classes Packery
 * http://packery.metafizzy.co
 */

/*!
 * classie v1.0.1
 * class helper functions
 * from bonzo https://github.com/ded/bonzo
 * MIT license
 * 
 * classie.has( elem, 'my-class' ) -> true/false
 * classie.add( elem, 'my-new-class' )
 * classie.remove( elem, 'my-unwanted-class' )
 * classie.toggle( elem, 'my-class' )
 */

/*jshint browser: true, strict: true, undef: true, unused: true */
/*global define: false, module: false */

( function( window ) {



// class helper functions from bonzo https://github.com/ded/bonzo

    function classReg( className ) {
        return new RegExp("(^|\\s+)" + className + "(\\s+|$)");
    }

// classList support for class management
// altho to be fair, the api sucks because it won't accept multiple classes at once
    var hasClass, addClass, removeClass;

    if ( 'classList' in document.documentElement ) {
        hasClass = function( elem, c ) {
            return elem.classList.contains( c );
        };
        addClass = function( elem, c ) {
            elem.classList.add( c );
        };
        removeClass = function( elem, c ) {
            elem.classList.remove( c );
        };
    }
    else {
        hasClass = function( elem, c ) {
            return classReg( c ).test( elem.className );
        };
        addClass = function( elem, c ) {
            if ( !hasClass( elem, c ) ) {
                elem.className = elem.className + ' ' + c;
            }
        };
        removeClass = function( elem, c ) {
            elem.className = elem.className.replace( classReg( c ), ' ' );
        };
    }

    function toggleClass( elem, c ) {
        var fn = hasClass( elem, c ) ? removeClass : addClass;
        fn( elem, c );
    }

    var classie = {
        // full names
        hasClass: hasClass,
        addClass: addClass,
        removeClass: removeClass,
        toggleClass: toggleClass,
        // short names
        has: hasClass,
        add: addClass,
        remove: removeClass,
        toggle: toggleClass
    };

// transport
    if ( typeof define === 'function' && define.amd ) {
        // AMD
        define( 'classie/classie',classie );
    } else if ( typeof exports === 'object' ) {
        // CommonJS
        module.exports = classie;
    } else {
        // browser global
        window.classie = classie;
    }

})( window );

/**
 * Rect
 * low-level utility class for basic geometry
 */

( function( window, factory ) {

    // universal module definition
    if ( typeof define == 'function' && define.amd ) {
        // AMD
        define( 'packery/js/rect',factory );
    } else if ( typeof exports == 'object' ) {
        // CommonJS
        module.exports = factory();
    } else {
        // browser global
        window.Packery = window.Packery || {};
        window.Packery.Rect = factory();
    }

}( window, function factory() {


// -------------------------- Packery -------------------------- //

// global namespace
    var Packery = window.Packery = function() {};

// -------------------------- Rect -------------------------- //

    function Rect( props ) {
        // extend properties from defaults
        for ( var prop in Rect.defaults ) {
            this[ prop ] = Rect.defaults[ prop ];
        }

        for ( prop in props ) {
            this[ prop ] = props[ prop ];
        }

    }

// make available
    Packery.Rect = Rect;

    Rect.defaults = {
        x: 0,
        y: 0,
        width: 0,
        height: 0
    };

    /**
     * Determines whether or not this rectangle wholly encloses another rectangle or point.
     * @param {Rect} rect
     * @returns {Boolean}
     **/
    Rect.prototype.contains = function( rect ) {
        // points don't have width or height
        var otherWidth = rect.width || 0;
        var otherHeight = rect.height || 0;
        return this.x <= rect.x &&
            this.y <= rect.y &&
            this.x + this.width >= rect.x + otherWidth &&
            this.y + this.height >= rect.y + otherHeight;
    };

    /**
     * Determines whether or not the rectangle intersects with another.
     * @param {Rect} rect
     * @returns {Boolean}
     **/
    Rect.prototype.overlaps = function( rect ) {
        var thisRight = this.x + this.width;
        var thisBottom = this.y + this.height;
        var rectRight = rect.x + rect.width;
        var rectBottom = rect.y + rect.height;

        // http://stackoverflow.com/a/306332
        return this.x < rectRight &&
            thisRight > rect.x &&
            this.y < rectBottom &&
            thisBottom > rect.y;
    };

    /**
     * @param {Rect} rect - the overlapping rect
     * @returns {Array} freeRects - rects representing the area around the rect
     **/
    Rect.prototype.getMaximalFreeRects = function( rect ) {

        // if no intersection, return false
        if ( !this.overlaps( rect ) ) {
            return false;
        }

        var freeRects = [];
        var freeRect;

        var thisRight = this.x + this.width;
        var thisBottom = this.y + this.height;
        var rectRight = rect.x + rect.width;
        var rectBottom = rect.y + rect.height;

        // top
        if ( this.y < rect.y ) {
            freeRect = new Rect({
                x: this.x,
                y: this.y,
                width: this.width,
                height: rect.y - this.y
            });
            freeRects.push( freeRect );
        }

        // right
        if ( thisRight > rectRight ) {
            freeRect = new Rect({
                x: rectRight,
                y: this.y,
                width: thisRight - rectRight,
                height: this.height
            });
            freeRects.push( freeRect );
        }

        // bottom
        if ( thisBottom > rectBottom ) {
            freeRect = new Rect({
                x: this.x,
                y: rectBottom,
                width: this.width,
                height: thisBottom - rectBottom
            });
            freeRects.push( freeRect );
        }

        // left
        if ( this.x < rect.x ) {
            freeRect = new Rect({
                x: this.x,
                y: this.y,
                width: rect.x - this.x,
                height: this.height
            });
            freeRects.push( freeRect );
        }

        return freeRects;
    };

    Rect.prototype.canFit = function( rect ) {
        return this.width >= rect.width && this.height >= rect.height;
    };

    return Rect;

}));

/**
 * Packer
 * bin-packing algorithm
 */

( function( window, factory ) {

    // universal module definition
    if ( typeof define == 'function' && define.amd ) {
        // AMD
        define( 'packery/js/packer',[ './rect' ], factory );
    } else if ( typeof exports == 'object' ) {
        // CommonJS
        module.exports = factory(
            require('./rect')
        );
    } else {
        // browser global
        var Packery = window.Packery = window.Packery || {};
        Packery.Packer = factory( Packery.Rect );
    }

}( window, function factory( Rect ) {


// -------------------------- Packer -------------------------- //

    /**
     * @param {Number} width
     * @param {Number} height
     * @param {String} sortDirection
     *   topLeft for vertical, leftTop for horizontal
     */
    function Packer( width, height, sortDirection ) {
        this.width = width || 0;
        this.height = height || 0;
        this.sortDirection = sortDirection || 'downwardLeftToRight';

        this.reset();
    }

    Packer.prototype.reset = function() {
        this.spaces = [];
        this.newSpaces = [];

        var initialSpace = new Rect({
            x: 0,
            y: 0,
            width: this.width,
            height: this.height
        });

        this.spaces.push( initialSpace );
        // set sorter
        this.sorter = sorters[ this.sortDirection ] || sorters.downwardLeftToRight;
    };

// change x and y of rect to fit with in Packer's available spaces
    Packer.prototype.pack = function( rect ) {
        for ( var i=0, len = this.spaces.length; i < len; i++ ) {
            var space = this.spaces[i];
            if ( space.canFit( rect ) ) {
                this.placeInSpace( rect, space );
                break;
            }
        }
    };

    Packer.prototype.placeInSpace = function( rect, space ) {
        // place rect in space
        rect.x = space.x;
        rect.y = space.y;

        this.placed( rect );
    };

// update spaces with placed rect
    Packer.prototype.placed = function( rect ) {
        // update spaces
        var revisedSpaces = [];
        for ( var i=0, len = this.spaces.length; i < len; i++ ) {
            var space = this.spaces[i];
            var newSpaces = space.getMaximalFreeRects( rect );
            // add either the original space or the new spaces to the revised spaces
            if ( newSpaces ) {
                revisedSpaces.push.apply( revisedSpaces, newSpaces );
            } else {
                revisedSpaces.push( space );
            }
        }

        this.spaces = revisedSpaces;

        this.mergeSortSpaces();
    };

    Packer.prototype.mergeSortSpaces = function() {
        // remove redundant spaces
        Packer.mergeRects( this.spaces );
        this.spaces.sort( this.sorter );
    };

// add a space back
    Packer.prototype.addSpace = function( rect ) {
        this.spaces.push( rect );
        this.mergeSortSpaces();
    };

// -------------------------- utility functions -------------------------- //

    /**
     * Remove redundant rectangle from array of rectangles
     * @param {Array} rects: an array of Rects
     * @returns {Array} rects: an array of Rects
     **/
    Packer.mergeRects = function( rects ) {
        for ( var i=0, len = rects.length; i < len; i++ ) {
            var rect = rects[i];
            // skip over this rect if it was already removed
            if ( !rect ) {
                continue;
            }
            // clone rects we're testing, remove this rect
            var compareRects = rects.slice(0);
            // do not compare with self
            compareRects.splice( i, 1 );
            // compare this rect with others
            var removedCount = 0;
            for ( var j=0, jLen = compareRects.length; j < jLen; j++ ) {
                var compareRect = compareRects[j];
                // if this rect contains another,
                // remove that rect from test collection
                var indexAdjust = i > j ? 0 : 1;
                if ( rect.contains( compareRect ) ) {
                    // console.log( 'current test rects:' + testRects.length, testRects );
                    // console.log( i, j, indexAdjust, rect, compareRect );
                    rects.splice( j + indexAdjust - removedCount, 1 );
                    removedCount++;
                }
            }
        }

        return rects;
    };


// -------------------------- sorters -------------------------- //

// functions for sorting rects in order
    var sorters = {
        // top down, then left to right
        downwardLeftToRight: function( a, b ) {
            return a.y - b.y || a.x - b.x;
        },
        // left to right, then top down
        rightwardTopToBottom: function( a, b ) {
            return a.x - b.x || a.y - b.y;
        }
    };


// --------------------------  -------------------------- //

    return Packer;

}));
/**
 * Packery Item Element
 **/

( function( window, factory ) {

    // universal module definition

    if ( typeof define == 'function' && define.amd ) {
        // AMD
        define( 'packery/js/item',[
                'get-style-property/get-style-property',
                'outlayer/outlayer',
                './rect'
            ],
            factory );
    } else if ( typeof exports == 'object' ) {
        // CommonJS
        module.exports = factory(
            require('desandro-get-style-property'),
            require('outlayer'),
            require('./rect')
        );
    } else {
        // browser global
        window.Packery.Item = factory(
            window.getStyleProperty,
            window.Outlayer,
            window.Packery.Rect
        );
    }

}( window, function factory( getStyleProperty, Outlayer, Rect ) {


// -------------------------- Item -------------------------- //

    var transformProperty = getStyleProperty('transform');

// sub-class Item
    var Item = function PackeryItem() {
        Outlayer.Item.apply( this, arguments );
    };

    Item.prototype = new Outlayer.Item();

    var protoCreate = Item.prototype._create;
    Item.prototype._create = function() {
        // call default _create logic
        protoCreate.call( this );
        this.rect = new Rect();
        // rect used for placing, in drag or Packery.fit()
        this.placeRect = new Rect();
    };

// -------------------------- drag -------------------------- //

    Item.prototype.dragStart = function() {
        this.getPosition();
        this.removeTransitionStyles();
        // remove transform property from transition
        if ( this.isTransitioning && transformProperty ) {
            this.element.style[ transformProperty ] = 'none';
        }
        this.getSize();
        // create place rect, used for position when dragged then dropped
        // or when positioning
        this.isPlacing = true;
        this.needsPositioning = false;
        this.positionPlaceRect( this.position.x, this.position.y );
        this.isTransitioning = false;
        this.didDrag = false;
    };

    /**
     * handle item when it is dragged
     * @param {Number} x - horizontal position of dragged item
     * @param {Number} y - vertical position of dragged item
     */
    Item.prototype.dragMove = function( x, y ) {
        this.didDrag = true;
        var packerySize = this.layout.size;
        x -= packerySize.paddingLeft;
        y -= packerySize.paddingTop;
        this.positionPlaceRect( x, y );
    };

    Item.prototype.dragStop = function() {
        this.getPosition();
        var isDiffX = this.position.x != this.placeRect.x;
        var isDiffY = this.position.y != this.placeRect.y;
        // set post-drag positioning flag
        this.needsPositioning = isDiffX || isDiffY;
        // reset flag
        this.didDrag = false;
    };

// -------------------------- placing -------------------------- //

    /**
     * position a rect that will occupy space in the packer
     * @param {Number} x
     * @param {Number} y
     * @param {Boolean} isMaxYContained
     */
    Item.prototype.positionPlaceRect = function( x, y, isMaxYOpen ) {
        this.placeRect.x = this.getPlaceRectCoord( x, true );
        this.placeRect.y = this.getPlaceRectCoord( y, false, isMaxYOpen );
    };

    /**
     * get x/y coordinate for place rect
     * @param {Number} coord - x or y
     * @param {Boolean} isX
     * @param {Boolean} isMaxOpen - does not limit value to outer bound
     * @returns {Number} coord - processed x or y
     */
    Item.prototype.getPlaceRectCoord = function( coord, isX, isMaxOpen ) {
        var measure = isX ? 'Width' : 'Height';
        var size = this.size[ 'outer' + measure ];
        var segment = this.layout[ isX ? 'columnWidth' : 'rowHeight' ];
        var parentSize = this.layout.size[ 'inner' + measure ];

        // additional parentSize calculations for Y
        if ( !isX ) {
            parentSize = Math.max( parentSize, this.layout.maxY );
            // prevent gutter from bumping up height when non-vertical grid
            if ( !this.layout.rowHeight ) {
                parentSize -= this.layout.gutter;
            }
        }

        var max;

        if ( segment ) {
            segment += this.layout.gutter;
            // allow for last column to reach the edge
            parentSize += isX ? this.layout.gutter : 0;
            // snap to closest segment
            coord = Math.round( coord / segment );
            // contain to outer bound
            // contain non-growing bound, allow growing bound to grow
            var mathMethod;
            if ( this.layout.options.isHorizontal ) {
                mathMethod = !isX ? 'floor' : 'ceil';
            } else {
                mathMethod = isX ? 'floor' : 'ceil';
            }
            var maxSegments = Math[ mathMethod ]( parentSize / segment );
            maxSegments -= Math.ceil( size / segment );
            max = maxSegments;
        } else {
            max = parentSize - size;
        }

        coord = isMaxOpen ? coord : Math.min( coord, max );
        coord *= segment || 1;

        return Math.max( 0, coord );
    };

    Item.prototype.copyPlaceRectPosition = function() {
        this.rect.x = this.placeRect.x;
        this.rect.y = this.placeRect.y;
    };

// -----  ----- //

// remove element from DOM
    Item.prototype.removeElem = function() {
        this.element.parentNode.removeChild( this.element );
        // add space back to packer
        this.layout.packer.addSpace( this.rect );
        this.emitEvent( 'remove', [ this ] );
    };

// -----  ----- //

    return Item;

}));

/*!
 * Packery v1.4.1
 * bin-packing layout library
 *
 * Licensed GPLv3 for open source use
 * or Flickity Commercial License for commercial use
 *
 * http://packery.metafizzy.co
 * Copyright 2015 Metafizzy
 */

( function( window, factory ) {

    // universal module definition
    if ( typeof define == 'function' && define.amd ) {
        // AMD
        define( 'packery/js/packery',[
                'classie/classie',
                'get-size/get-size',
                'outlayer/outlayer',
                './rect',
                './packer',
                './item'
            ],
            factory );
    } else if ( typeof exports == 'object' ) {
        // CommonJS
        module.exports = factory(
            require('desandro-classie'),
            require('get-size'),
            require('outlayer'),
            require('./rect'),
            require('./packer'),
            require('./item')
        );
    } else {
        // browser global
        window.Packery = factory(
            window.classie,
            window.getSize,
            window.Outlayer,
            window.Packery.Rect,
            window.Packery.Packer,
            window.Packery.Item
        );
    }

}( window, function factory( classie, getSize, Outlayer, Rect, Packer, Item ) {


// ----- Rect ----- //

// allow for pixel rounding errors IE8-IE11 & Firefox; #227
    Rect.prototype.canFit = function( rect ) {
        return this.width >= rect.width - 1 && this.height >= rect.height - 1;
    };

// -------------------------- Packery -------------------------- //

// create an Outlayer layout class
    var Packery = Outlayer.create('packery');
    Packery.Item = Item;

    Packery.prototype._create = function() {
        // call super
        Outlayer.prototype._create.call( this );

        // initial properties
        this.packer = new Packer();

        // Left over from v1.0
        this.stamp( this.options.stamped );

        // create drag handlers
        var _this = this;
        this.handleDraggabilly = {
            dragStart: function() {
                _this.itemDragStart( this.element );
            },
            dragMove: function() {
                _this.itemDragMove( this.element, this.position.x, this.position.y );
            },
            dragEnd: function() {
                _this.itemDragEnd( this.element );
            }
        };

        this.handleUIDraggable = {
            start: function handleUIDraggableStart( event ) {
                _this.itemDragStart( event.currentTarget );
            },
            drag: function handleUIDraggableDrag( event, ui ) {
                _this.itemDragMove( event.currentTarget, ui.position.left, ui.position.top );
            },
            stop: function handleUIDraggableStop( event ) {
                _this.itemDragEnd( event.currentTarget );
            }
        };

    };


// ----- init & layout ----- //

    /**
     * logic before any new layout
     */
    Packery.prototype._resetLayout = function() {
        this.getSize();

        this._getMeasurements();

        // reset packer
        var packer = this.packer;
        // packer settings, if horizontal or vertical
        if ( this.options.isHorizontal ) {
            packer.width = Number.POSITIVE_INFINITY;
            packer.height = this.size.innerHeight + this.gutter;
            packer.sortDirection = 'rightwardTopToBottom';
        } else {
            packer.width = this.size.innerWidth + this.gutter;
            packer.height = Number.POSITIVE_INFINITY;
            packer.sortDirection = 'downwardLeftToRight';
        }

        packer.reset();

        // layout
        this.maxY = 0;
        this.maxX = 0;
    };

    /**
     * update columnWidth, rowHeight, & gutter
     * @private
     */
    Packery.prototype._getMeasurements = function() {
        this._getMeasurement( 'columnWidth', 'width' );
        this._getMeasurement( 'rowHeight', 'height' );
        this._getMeasurement( 'gutter', 'width' );
    };

    Packery.prototype._getItemLayoutPosition = function( item ) {
        this._packItem( item );
        return item.rect;
    };


    /**
     * layout item in packer
     * @param {Packery.Item} item
     */
    Packery.prototype._packItem = function( item ) {
        this._setRectSize( item.element, item.rect );
        // pack the rect in the packer
        this.packer.pack( item.rect );
        this._setMaxXY( item.rect );
    };

    /**
     * set max X and Y value, for size of container
     * @param {Packery.Rect} rect
     * @private
     */
    Packery.prototype._setMaxXY = function( rect ) {
        this.maxX = Math.max( rect.x + rect.width, this.maxX );
        this.maxY = Math.max( rect.y + rect.height, this.maxY );
    };

    /**
     * set the width and height of a rect, applying columnWidth and rowHeight
     * @param {Element} elem
     * @param {Packery.Rect} rect
     */
    Packery.prototype._setRectSize = function( elem, rect ) {
        var size = getSize( elem );
        var w = size.outerWidth;
        var h = size.outerHeight;
        // size for columnWidth and rowHeight, if available
        // only check if size is non-zero, #177
        if ( w || h ) {
            w = this._applyGridGutter( w, this.columnWidth );
            h = this._applyGridGutter( h, this.rowHeight );
        }
        // rect must fit in packer
        rect.width = Math.min( w, this.packer.width );
        rect.height = Math.min( h, this.packer.height );
    };

    /**
     * fits item to columnWidth/rowHeight and adds gutter
     * @param {Number} measurement - item width or height
     * @param {Number} gridSize - columnWidth or rowHeight
     * @returns measurement
     */
    Packery.prototype._applyGridGutter = function( measurement, gridSize ) {
        // just add gutter if no gridSize
        if ( !gridSize ) {
            return measurement + this.gutter;
        }
        gridSize += this.gutter;
        // fit item to columnWidth/rowHeight
        var remainder = measurement % gridSize;
        var mathMethod = remainder && remainder < 1 ? 'round' : 'ceil';
        measurement = Math[ mathMethod ]( measurement / gridSize ) * gridSize;
        return measurement;
    };

    Packery.prototype._getContainerSize = function() {
        if ( this.options.isHorizontal ) {
            return {
                width: this.maxX - this.gutter
            };
        } else {
            return {
                height: this.maxY - this.gutter
            };
        }
    };


// -------------------------- stamp -------------------------- //

    /**
     * makes space for element
     * @param {Element} elem
     */
    Packery.prototype._manageStamp = function( elem ) {

        var item = this.getItem( elem );
        var rect;
        if ( item && item.isPlacing ) {
            rect = item.placeRect;
        } else {
            var offset = this._getElementOffset( elem );
            rect = new Rect({
                x: this.options.isOriginLeft ? offset.left : offset.right,
                y: this.options.isOriginTop ? offset.top : offset.bottom
            });
        }

        this._setRectSize( elem, rect );
        // save its space in the packer
        this.packer.placed( rect );
        this._setMaxXY( rect );
    };

// -------------------------- methods -------------------------- //

    function verticalSorter( a, b ) {
        return a.position.y - b.position.y || a.position.x - b.position.x;
    }

    function horizontalSorter( a, b ) {
        return a.position.x - b.position.x || a.position.y - b.position.y;
    }

    Packery.prototype.sortItemsByPosition = function() {
        var sorter = this.options.isHorizontal ? horizontalSorter : verticalSorter;
        this.items.sort( sorter );
    };

    /**
     * Fit item element in its current position
     * Packery will position elements around it
     * useful for expanding elements
     *
     * @param {Element} elem
     * @param {Number} x - horizontal destination position, optional
     * @param {Number} y - vertical destination position, optional
     */
    Packery.prototype.fit = function( elem, x, y ) {
        var item = this.getItem( elem );
        if ( !item ) {
            return;
        }

        // prepare internal properties
        this._getMeasurements();

        // stamp item to get it out of layout
        this.stamp( item.element );
        // required for positionPlaceRect
        item.getSize();
        // set placing flag
        item.isPlacing = true;
        // fall back to current position for fitting
        x = x === undefined ? item.rect.x: x;
        y = y === undefined ? item.rect.y: y;

        // position it best at its destination
        item.positionPlaceRect( x, y, true );

        this._bindFitEvents( item );
        item.moveTo( item.placeRect.x, item.placeRect.y );
        // layout everything else
        this.layout();

        // return back to regularly scheduled programming
        this.unstamp( item.element );
        this.sortItemsByPosition();
        // un set placing flag, back to normal
        item.isPlacing = false;
        // copy place rect position
        item.copyPlaceRectPosition();
    };

    /**
     * emit event when item is fit and other items are laid out
     * @param {Packery.Item} item
     * @private
     */
    Packery.prototype._bindFitEvents = function( item ) {
        var _this = this;
        var ticks = 0;
        function tick() {
            ticks++;
            if ( ticks != 2 ) {
                return;
            }
            _this.emitEvent( 'fitComplete', [ item ] );
        }
        // when item is laid out
        item.on( 'layout', function() {
            tick();
            return true;
        });
        // when all items are laid out
        this.on( 'layoutComplete', function() {
            tick();
            return true;
        });
    };

// -------------------------- resize -------------------------- //

// debounced, layout on resize
    Packery.prototype.resize = function() {
        // don't trigger if size did not change
        var size = getSize( this.element );
        // check that this.size and size are there
        // IE8 triggers resize on body size change, so they might not be
        var hasSizes = this.size && size;
        var innerSize = this.options.isHorizontal ? 'innerHeight' : 'innerWidth';
        if ( hasSizes && size[ innerSize ] == this.size[ innerSize ] ) {
            return;
        }

        this.layout();
    };

// -------------------------- drag -------------------------- //

    /**
     * handle an item drag start event
     * @param {Element} elem
     */
    Packery.prototype.itemDragStart = function( elem ) {
        this.stamp( elem );
        var item = this.getItem( elem );
        if ( item ) {
            item.dragStart();
        }
    };

    /**
     * handle an item drag move event
     * @param {Element} elem
     * @param {Number} x - horizontal change in position
     * @param {Number} y - vertical change in position
     */
    Packery.prototype.itemDragMove = function( elem, x, y ) {
        var item = this.getItem( elem );
        if ( item ) {
            item.dragMove( x, y );
        }

        // debounce
        var _this = this;
        // debounce triggering layout
        function delayed() {
            _this.layout();
            delete _this.dragTimeout;
        }

        this.clearDragTimeout();

        this.dragTimeout = setTimeout( delayed, 40 );
    };

    Packery.prototype.clearDragTimeout = function() {
        if ( this.dragTimeout ) {
            clearTimeout( this.dragTimeout );
        }
    };

    /**
     * handle an item drag end event
     * @param {Element} elem
     */
    Packery.prototype.itemDragEnd = function( elem ) {
        var item = this.getItem( elem );
        var itemDidDrag;
        if ( item ) {
            itemDidDrag = item.didDrag;
            item.dragStop();
        }
        // if elem didn't move, or if it doesn't need positioning
        // unignore and unstamp and call it a day
        if ( !item || ( !itemDidDrag && !item.needsPositioning ) ) {
            this.unstamp( elem );
            return;
        }
        // procced with dragged item

        classie.add( item.element, 'is-positioning-post-drag' );

        // save this var, as it could get reset in dragStart
        var onLayoutComplete = this._getDragEndLayoutComplete( elem, item );

        if ( item.needsPositioning ) {
            item.on( 'layout', onLayoutComplete );
            item.moveTo( item.placeRect.x, item.placeRect.y );
        } else if ( item ) {
            // item didn't need placement
            item.copyPlaceRectPosition();
        }

        this.clearDragTimeout();
        this.on( 'layoutComplete', onLayoutComplete );
        this.layout();

    };

    /**
     * get drag end callback
     * @param {Element} elem
     * @param {Packery.Item} item
     * @returns {Function} onLayoutComplete
     */
    Packery.prototype._getDragEndLayoutComplete = function( elem, item ) {
        var itemNeedsPositioning = item && item.needsPositioning;
        var completeCount = 0;
        var asyncCount = itemNeedsPositioning ? 2 : 1;
        var _this = this;

        return function onLayoutComplete() {
            completeCount++;
            // don't proceed if not complete
            if ( completeCount != asyncCount ) {
                return true;
            }
            // reset item
            if ( item ) {
                classie.remove( item.element, 'is-positioning-post-drag' );
                item.isPlacing = false;
                item.copyPlaceRectPosition();
            }

            _this.unstamp( elem );
            // only sort when item moved
            _this.sortItemsByPosition();

            // emit item drag event now that everything is done
            if ( itemNeedsPositioning ) {
                _this.emitEvent( 'dragItemPositioned', [ item ] );
            }
            // listen once
            return true;
        };
    };

    /**
     * binds Draggabilly events
     * @param {Draggabilly} draggie
     */
    Packery.prototype.bindDraggabillyEvents = function( draggie ) {
        draggie.on( 'dragStart', this.handleDraggabilly.dragStart );
        draggie.on( 'dragMove', this.handleDraggabilly.dragMove );
        draggie.on( 'dragEnd', this.handleDraggabilly.dragEnd );
    };

    /**
     * binds jQuery UI Draggable events
     * @param {jQuery} $elems
     */
    Packery.prototype.bindUIDraggableEvents = function( $elems ) {
        $elems
            .on( 'dragstart', this.handleUIDraggable.start )
            .on( 'drag', this.handleUIDraggable.drag )
            .on( 'dragstop', this.handleUIDraggable.stop );
    };

    Packery.Rect = Rect;
    Packery.Packer = Packer;

    return Packery;

}));

/*!
 * Packery layout mode v1.1.3
 * sub-classes Packery
 * http://packery.metafizzy.co
 */

/*jshint browser: true, strict: true, undef: true, unused: true */

( function( window, factory ) {

    // universal module definition
    if ( typeof define == 'function' && define.amd ) {
        // AMD
        define( [
                'isotope/js/layout-mode',
                'packery/js/packery',
                'get-size/get-size'
            ],
            factory );
    } else if ( typeof exports == 'object' ) {
        // CommonJS
        module.exports = factory(
            require('isotope-layout/js/layout-mode'),
            require('packery'),
            require('get-size')
        );
    } else {
        // browser global
        factory(
            window.Isotope.LayoutMode,
            window.Packery,
            window.getSize
        );
    }

}( window, function factor( LayoutMode, Packery, getSize ) {


// -------------------------- helpers -------------------------- //

// extend objects
    function extend( a, b ) {
        for ( var prop in b ) {
            a[ prop ] = b[ prop ];
        }
        return a;
    }

// -------------------------- masonryDefinition -------------------------- //

    // create an Outlayer layout class
    var PackeryMode = LayoutMode.create('packery');

    // save on to these methods
    var _getElementOffset = PackeryMode.prototype._getElementOffset;
    // var layout = PackeryMode.prototype.layout;
    var _getMeasurement = PackeryMode.prototype._getMeasurement;

    // sub-class Masonry
    extend( PackeryMode.prototype, Packery.prototype );

    // set back, as it was overwritten by Packery
    PackeryMode.prototype._getElementOffset = _getElementOffset;
    // PackeryMode.prototype.layout = layout;
    PackeryMode.prototype._getMeasurement = _getMeasurement;

    // set packery in _resetLayout
    var _resetLayout = PackeryMode.prototype._resetLayout;
    PackeryMode.prototype._resetLayout = function() {
        this.packer = this.packer || new Packery.Packer();
        _resetLayout.apply( this, arguments );
    };

    var _getItemLayoutPosition = PackeryMode.prototype._getItemLayoutPosition;
    PackeryMode.prototype._getItemLayoutPosition = function( item ) {
        // set packery rect
        item.rect = item.rect || new Packery.Rect();
        return _getItemLayoutPosition.call( this, item );
    };

    // HACK copy over isOriginLeft/Top options
    var _manageStamp = PackeryMode.prototype._manageStamp;
    PackeryMode.prototype._manageStamp = function() {
        this.options.isOriginLeft = this.isotope.options.isOriginLeft;
        this.options.isOriginTop = this.isotope.options.isOriginTop;
        _manageStamp.apply( this, arguments );
    };

    PackeryMode.prototype.needsResizeLayout = function() {
        // don't trigger if size did not change
        var size = getSize( this.element );
        // check that this.size and size are there
        // IE8 triggers resize on body size change, so they might not be
        var hasSizes = this.size && size;
        var innerSize = this.options.isHorizontal ? 'innerHeight' : 'innerWidth';
        return hasSizes && size[ innerSize ] != this.size[ innerSize ];
    };

    return PackeryMode;
}));
