<?php
/**
 * copyright (C) 2008-2017 GWE Systems Ltd - All rights reserved
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Session\Session;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Component\ComponentHelper;

/**
 * HTML View class for the module  frontend
 *
 * @static
 */
include_once(JPATH_SITE."/modules/mod_jevents_latest/tmpl/default/latest.php");
JLoader::register('JevRegistry', JPATH_SITE . "/components/com_jevents/libraries/registry.php");

class FloatModLatestView extends DefaultModLatestView
{

	var $oldVersion = false;

	function __construct($params, $modid)
	{
		JLoader::register('JEventsHelper', JEV_ADMINPATH . "helpers/jevents.php");

		$compparams = ComponentHelper::getParams("com_jevents");

		$floatStyle = (int) $compparams->get('float_style', 1);
		if ( $floatStyle !== 3 && $floatStyle !== 4 && $compparams->get("floatold", 0) && !$params->get("floatuikit", 0))
		{
			include_once("latest-old.php");
			$this->oldVersion = new FloatModLatestViewOld($params, $modid);
		}

		// if using the layout editor then we can skip these styles!
		if ($floatStyle !== 3 && $floatStyle !== 4 && $params->get("uselayouteditor", 0))
		{
			// using uikit style so remove references to w3, w4 etc.
			$css = $params->get("modlatest_customcss", false);
			$css = str_replace("jeviso_module .jeviso_item.w", "jeviso_module .jeviso_item.notw", $css);
			$params->set("modlatest_customcss", $css);
		}

		if ($floatStyle == 4)
		{
			JHtml::_('stylesheet','https://cdnjs.cloudflare.com/ajax/libs/simple-line-icons/2.4.1/css/simple-line-icons.min.css');
		}
		parent::__construct($params, $modid);

		if ($compparams->get("floatmoduleuseformatstring", 0) || !$params->get("uselayouteditor", 0))
		{
			$this->floatmoduleuseformatstring = 1;
		}
		else
		{
			$this->floatmoduleuseformatstring = 0;
		}

		return;
	}

	public function displayLatestEvents(){

		if ($this->oldVersion)
		{
			return $this->oldVersion->displayLatestEvents();
		}

		$params = ComponentHelper::getParams("com_jevents");
		$this->jeventsUIkit = $params->get("floatuikit", 1) && ($params->get('float_style', 1) == 3);

		/*
		 With thanks to https://w3bits.com/css-grid-masonry/
		 and see https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template-columns
		 */

		JLoader::register('JevIsotope', JPATH_LIBRARIES . "/jevents/jevisotope/jevisotope.php");

		if ($this->jeventsUIkit)
		{
			HTMLHelper::stylesheet('media/com_jevents/css/uikit.gsl.css', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false));
			HTMLHelper::script('media/com_jevents/js/uikit.js', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false), array('defer' => true));
			HTMLHelper::script('media/com_jevents/js/uikit-icons.js', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false), array('defer' => true));
			HTMLHelper::stylesheet('com_jevents/float/modulefloat.css', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false));
		}
		else
		{
			if (is_array($this->template) && isset($this->template[0]))
			{
				HTMLHelper::stylesheet('com_jevents/float/modulefloat.css', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false));
			}
		}

		$cfg = JEVConfig::getInstance();

		$jsonData = Factory::getApplication()->input->getString('json', '{"direction" : 0}');
		$jsonData = json_decode($jsonData);
		$this->getLatestEventsData();

		$content = "";

		if ($jsonData->direction == 0)
		{
			$content .= "<div class='isotope jevisotope " . ($this->jeventsUIkit ? " gsl-scope " : "" ) . "'><div id='jeviso_main'>"; //
			$content .= "<div class='mod_events_latest_data'  id='mod_events_latest_" . $this->_modid . "_data'>"; // latest isotope line 33
		}

		if(isset($this->eventsByRelDay) && count($this->eventsByRelDay))
		{

			if (method_exists($this, 'getNavigationIcons'))
			{
				$content .= $this->getNavigationIcons();
			}

			$usesComponentLayouts = false;
			if (is_array($this->template) && isset($this->template[0]))
			{
				$usesComponentLayouts = true;
			}
			if (($params->get('float_style', 1) == 3) && $usesComponentLayouts)
			{
				$header = $this->getModuleHeader('<div id="jeviso_module" class="jeviso_day"><div class="jeviso_itemcontainer jeviso_container {COLUMNS+1}@xl {COLUMNS}@l {COLUMNS-1}@m {COLUMNS-2}@s uk-grid" itemscope itemtype="http://schema.org/Event">');
				$colCount = isset($this->template[0]->params) ? $this->template[0]->params->get('columnsL', 3) : 3;
				for ($inc = 0; $inc <=12; $inc ++)
				{
					if (($params->get('float_style', 1) == 3))
					{

						if ($inc == 0)
						{
							$header = str_replace("{COLUMNS}", "uk-child-width-1-$colCount", $header);
							$header = str_replace("{COLUMNS}", "uk-child-width-1-$colCount", $header);
						}
						else
						{
							$header = str_replace("{COLUMNS+$inc}", "uk-child-width-1-" . ($colCount + $inc), $header);
							$header = str_replace("{COLUMNS-$inc}", "uk-child-width-1-" . max(1, $colCount - $inc), $header);
						}
					}
				}
			}
			else
			{
				$header = $this->getModuleHeader('<div id="jeviso_module" class="jeviso_day"><div class="jeviso_itemcontainer jeviso_container " itemscope itemtype="http://schema.org/Event">');
			}


			if ($this->jeventsUIkit)
			{
				$header = str_replace("uk-", "gsl-" , $header);
			}
			// Set the column count for the child width style setting
			$params = ComponentHelper::getParams("com_jevents");
			$header = str_replace('{COLCOUNT}', $params->get('float_columns', 5), $header);
			$content .= $header;

			// Now to display these events, we just start at the smallest index of the $this->eventsByRelDay array
			// and work our way up.

			// initialize name of com_jevents module and task defined to view
			// event detail.  Note that these could change in future com_event
			// component revisions!!  Note that the '$this->itemId' can be left out in
			// the link parameters for event details below since the event.php
			// component handler will fetch its own id from the db menu table
			// anyways as far as I understand it.

			$task_events = 'icalrepeat.detail';

			$this->processFormatString();

			foreach($this->eventsByRelDay as $relDay => $daysEvents){

				reset($daysEvents);

				// get all of the events for this day
				foreach($daysEvents as $dayEvent){

					if ($this->processTemplate($content, $dayEvent))
					{
						continue;
					}

					$eventcontent = "";

					// generate output according custom string
					foreach($this->splitCustomFormat as $condtoken) {

						if (isset($condtoken['cond'])) {
							if ( $condtoken['cond'] == 'a'  && !$dayEvent->alldayevent()) continue;
							else if ( $condtoken['cond'] == '!a' &&  $dayEvent->alldayevent()) continue;
							else if ( $condtoken['cond'] == 'e'  && !($dayEvent->noendtime() || $dayEvent->alldayevent())) continue;
							else if ( $condtoken['cond'] == '!e' &&  ($dayEvent->noendtime() || $dayEvent->alldayevent())) continue;
							else if ( $condtoken['cond'] == '!m' &&  $dayEvent->getUnixStartDate()!=$dayEvent->getUnixEndDate() ) continue;
							else if ( $condtoken['cond'] == 'm' &&  $dayEvent->getUnixStartDate()==$dayEvent->getUnixEndDate() ) continue;
						}
						foreach($condtoken['data'] as $token) {
							unset($match);
							unset($dateParm);
							$dateParm="";
							$match='';
							if (is_array($token)) {
								$match = $token['keyword'];
								$dateParm = isset($token['dateParm']) ? trim($token['dateParm']) : "";
							}
							else if (strpos($token,'${')!==false){
								$match = $token;
							}
							else {
								$eventcontent .= $token;
								continue;
							}
							$this->processMatch($eventcontent, $match, $dayEvent, $dateParm, $relDay);

						} // end of foreach
					} // end of foreach

					$floatStyle = $params->get('float_style', 1);
					if ($floatStyle && !$this->jeventsUIkit)
					{
						$floatStyle .= "uikit";
					}
					$item_bodycore = file_get_contents(JEV_PATH . "views/float/defaults/icalevent.list_block" . $floatStyle . ".html");

					$templaterow = $this->modparams->get("modlatest_templaterow") ? $this->modparams->get("modlatest_templaterow") : '%s';

					if ($params->get("floatmoduleuseformatstring", 0) || !$params->get("uselayouteditor", 0))
					{
						$item_bodycore = $eventcontent;
					}

					$item_bodycore = str_replace("%s", $item_bodycore , $templaterow);
					if ($this->jeventsUIkit)
					{
						$item_bodycore = str_replace("uk-", "gsl-" , $item_bodycore);
					}

					if (strpos($item_bodycore, "jeviso_item" . $dayEvent->rp_id()) > 0)
					{
						$item_body = $item_bodycore;
						$item_body = preg_replace("/jeviso_item w([1-9]+) /", "jeviso_item w" .  $cfg->get('float_columns', 4) . " ", $item_body);
					}
					else
					{
						$item_body = '<div  id="jeviso_item' . $dayEvent->rp_id() . '" class="jeviso_item w' . $params->get('float_columns', 4) . ' ' . $dayEvent->catname() . ' style' . $params->get('float_style', 1) . '">';
						$item_body .= $item_bodycore;
						$item_body .= '</div>';
					}

					$content   .= $item_body;

					$firstTime=false;
				} // end of foreach
			} // end of foreach

			$content .= $this->getModuleFooter('</div></div>');

			if (($params->get('float_style', 1) == 4) && $usesComponentLayouts)
			{
				$colCount = isset($this->template[0]->params) ? $this->template[0]->params->get('columnsL', 3) : 3;

				$width  = " col-xl-" . intval(12/($colCount+1) );
				$width .= " col-lg-" . intval(12/$colCount);
				$width .= " col-md-" . intval(12/max($colCount-1, 1));
				$width .= " col-sm-" . intval(12/max($colCount-2, 1));
				$width .= " col-xs-" . intval(12/max($colCount-3, 1));

				$content = str_replace("{COLUMNS}", $width , $content);
			}

		}
		else if ($this->modparams->get("modlatest_NoEvents", 1)){
			if (method_exists($this, 'getNavigationIcons'))
			{
				if ($jsonData->direction == 0)
				{
					$content .= $this->getNavigationIcons(false, false, false);
				}
				else
				{
					$content .= $this->getNavigationIcons($jsonData->direction > 0, $jsonData->direction < 0, false);
				}
			}
			$header = $this->getModuleHeader('<div id="jeviso_module" class="jeviso_day"><div class="jeviso_itemcontainer jeviso_container uk-child-width-1-4@l uk-child-width-1-3@m uk-child-width-1-1@s uk-grid" itemscope itemtype="http://schema.org/Event">');

			if ($this->jeventsUIkit)
			{
				$header = str_replace("uk-", "gsl-" , $header);
			}
			$content .= $header;
			$content .= Text::_('JEV_NO_EVENTS');
			$content .= $this->getModuleFooter('</div></div>');
		}

		preg_match("#jeviso_item w([1-9]*) style3#", $content, $match);

		//$content = str_replace("jeviso_item w3 style3", "jeviso_item gsl-width-1-3 style3", $content);
		$colCount = count($match) == 2 ? $match[1] : $cfg->get('float_columns', 4);

		$style = <<< STYLE
.jeviso_container {
  display: grid;
  grid-gap: 1em;                                                    /* [1] Add some gap between rows and columns */
  grid-template-columns: repeat($colCount, minmax(350px,1fr));      /* [2] Make columns adjust according to the available viewport */
  grid-auto-rows: 1px;                                              /* [3] Set the height for implicitly-created row track  - 1px means the rows/tracks are 1px high*/
}
.jeviso_container .jeviso_item
{
	min-height:100px;
	height:max-content;
	padding:10px;
	grid-row-end: span 100;
}

STYLE;
		// grandfather old layouts using w3 etc.
		$hasW = false;
		if (count($match) ==2 && isset($match[1]))
		{
			$style = "";
			$hasW = true;
		}

		$style .= <<< STYLE
.jeviso_itemcontainer .jeviso_item .jfloat_item
{
	min-height:100px;
	height:max-content;
}

STYLE;
		Factory::getDocument()->addStyleDeclaration($style);

		$callink_HTML = '<div class="mod_events_latest_callink">'
			.$this->getCalendarLink()
			. '</div>'	;

		if ($this->linkToCal == 1) $content = $callink_HTML . $content;
		if ($this->linkToCal == 2) $content .= $callink_HTML;

		if ($this->displayRSS){
			if (version_compare(JVERSION, "1.6.0", 'ge')) {
				$rssimg = JURI::root() . "media/system/images/livemarks.png";
			}
			else {
				$rssimg = JURI::root() . "images/M_images/livemarks.png";
			}

			$callink_HTML = '<div class="mod_events_latest_rsslink">'
				.'<a href="'.$this->rsslink.'" title="'.Text::_("RSS_FEED").'" target="_blank">'
				.'<img src="'.$rssimg.'" alt="'.Text::_("RSS_FEED").'" />'
				.Text::_("SUBSCRIBE_TO_RSS_FEED")
				. '</a>'
				. '</div>';
			$content .= $callink_HTML;
		}
		if ($jsonData->direction == 0)
		{
			$content .= "</div>";  // close latest isotope line 33
			$content .= "</div></div>";
		}

		$width = $params->get('float_columns', 4);

		// if not called from gwe_fetchlatestevents.php
		if (Factory::getApplication()->input->getString('file', '') !== 'fetchlatestevents')
		{
			// Navigation
			static $scriptloaded = false;
			if (!$scriptloaded ){

				$root = JURI::root();
				$token= Session::getFormToken();
				$script = <<<SCRIPT

function fetchMoreFloatEvents(modid, direction)
{        
                      
		var url = "{$root}index.php?option=com_jevents&ttoption=com_jevents&typeaheadtask=gwejson&file=fetchlatestevents&path=module&folder=mod_jevents_latest&token={$token}";
		const xhr = new XMLHttpRequest();
		xhr.open("POST", url);
		xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded;charset=UTF-8");
		xhr.onload = function () {
			try {
				if (xhr.status != 200) {
					alert('error ' + xhr.statusText);
					return;
				}

				var returnData = JSON.parse(xhr.responseText);

				document.getElementById("mod_events_latest_" + modid + "_data").innerHTML = returnData.html;
				setTimeout(waitForImages, 100);
                try {
                    document.getElementById("mod_events_latest_"+modid+"_data").parentNode.scrollIntoView({block: "start", behavior: "smooth"});
                }
                catch (e) {
                }

  			    jQuery("#mod_events_latest_" + modid + "_data").trigger('jevisomoreevents');
				return;
			} catch (e) {
				alert('bad response = ' + e + " :: " + xhr.responseText );
			}
		}

		xhr.onerror = function () {
			alert('error ' + xhr.statusText);

		}
		xhr.send('json=' + JSON.stringify({'modid':modid, 'direction':direction}));	
}

/**
 * Apply spanning to all the masonry items
 *
 * Loop through all the items and apply the spanning to them using
 * `resizeMasonryItem()` function.
 *
 * @uses resizeMasonryItem
 */
function resizeAllMasonryItems(){
	// Get all item class objects in one list
	var allItems = document.querySelectorAll('_data div.jeviso_container .jeviso_item');

	/*
	 * Loop through the above list and execute the spanning function to
	 * each list-item (i.e. each masonry item)
	 */
	for(var i=0;i<allItems.length;i++){
		resizeMasonryItem(allItems[i]);
	}
}

/**
 * Set appropriate spanning to any masonry item
 *
 * Get different properties we already set for the masonry, calculate 
 * height or spanning for any cell of the masonry grid based on its 
 * content-wrapper's height, the (row) gap of the grid, and the size 
 * of the implicit row tracks.
 *
 * @param item Object A brick/tile/cell inside the masonry
 */
function resizeMasonryItem(item){
  /* Get the grid object, its row-gap, and the size of its implicit rows */
  var grid = document.querySelector('_data div.jeviso_container');
  if (!grid)
  {
    return;
  }
  var rowGap = parseInt(window.getComputedStyle(grid).getPropertyValue('grid-row-gap'));
  var rowHeight = parseInt(window.getComputedStyle(grid).getPropertyValue('grid-auto-rows'));

  /*
   * Spanning for any brick = S
   * Grid's row-gap = G
   * Size of grid's implicitly create row-track = R
   * Height of item content = H
   * Net height of the item = H1 = H + G
   * Net height of the implicit row-track = T = G + R
   * S = H1 / T
   */
  //var rowSpan = Math.ceil((item.querySelector('.masonry-content').getBoundingClientRect().height+rowGap)/(rowHeight+rowGap));
  var rowSpan = Math.ceil((item.getBoundingClientRect().height+rowGap)/(rowHeight+rowGap));

  /* Set the spanning as calculated above (S) */
  item.style.gridRowEnd = 'span '+rowSpan;
}		
/**
 * Resize the items when all the images inside the masonry grid 
 * finish loading. This will ensure that all the content inside our
 * masonry items is visible.
 *
 * @uses ImagesLoaded
 * @uses resizeMasonryItem
 */
function waitForImages() {
return;
  var allItems = document.querySelectorAll('_data div.jeviso_container .jeviso_item');
  for(var i=0;i<allItems.length;i++){
     var item =  allItems[i];
     var img = item.querySelector('img'); 
     if (img) 
     {
	     img.addEventListener('load', function() {
	        resizeMasonryItem(item);
	     });
	     if (img.naturalHeight > 0)
	     {
	        resizeMasonryItem(item);
	     }
     }
     else 
     {
	        resizeMasonryItem(item);     
     }
  }
}

/* Resize all the grid items on the load and resize events */
var masonryEvents = ['load', 'resize'];
masonryEvents.forEach( function(event) {
  window.addEventListener(event, resizeAllMasonryItems);
} );
		
/* Do a resize once more when all the images finish loading */
document.addEventListener('DOMContentLoaded', function() {waitForImages()});
	
SCRIPT;
				if ($hasW)
				{
					$width  = $cfg->get('float_columns', 4);
					$script .= <<< SCRIPT
		function jeviso_listmodule(id) {			
			var container = jQuery('#mod_events_latest_' + id + '_data div.jeviso_container');
			container.trigger('jevisolist');
			
			container.find('.jeviso_item').removeClass('w$width');
			container.find('.jeviso_item').addClass('w1 listv');
			/* Reset layout for the changes to take effect */
			try {
				container.isotope('layout');
			}
			catch (e)
			{
			}
		}

		function jeviso_gridmodule(id) {
			var container = jQuery('#mod_events_latest_' + id + '_data div.jeviso_container');
			container.trigger('jevisogrid');
			
			container.find('.jeviso_item').addClass('w$width');
			container.find('.jeviso_item').removeClass('w1');
			container.find('.jeviso_item').removeClass('listv');
			/* Reset layout for the changes to take effect */
			try {
				container.isotope('layout');
			}
			catch (e)
			{
			}
		}
SCRIPT;
				}
				else
				{
					$script .= <<< SCRIPT
var childWidths = [];

function jeviso_listmodule(id) {			
	var container = document.querySelector('#mod_events_latest_' + id + '_data div.jeviso_container');
	if (container)
	{
		container.style.display = "block";
	}	
	container = document.querySelector('#mod_events_latest_' + id + '_data div.jeviso_itemcontainer');
	if (container)
	{
		if (childWidths == "")
		{ 
		  childWidthList = container.classList;
		  for (c=0; c < childWidthList.length; c++)
		  {
		    if (childWidthList[c].indexOf('gsl-child-width') == 0 || childWidthList[c].indexOf('uk-child-width') == 0)
		    {
		        childWidths.push(childWidthList[c]);
		    }	    
		  }
		}
		container.classList.add('gsl-child-width-1-1');
		container.classList.add('uk-child-width-1-1');
	
		for (c=0; c < childWidths.length; c++)
		{
			container.classList.remove(childWidths[c]);
		}

	}
}

function jeviso_gridmodule(id) {
	var container = document.querySelector('#mod_events_latest_' + id + '_data div.jeviso_container');
	if (container)
	{
		container.style.display = "grid";
	}
	container = document.querySelector('#mod_events_latest_' + id + '_data div.jeviso_itemcontainer');
	if (container)
	{
		container.classList.remove('gsl-child-width-1-1');
		container.classList.remove('uk-child-width-1-1');
		for (c=0; c < childWidths.length; c++)
		{
			container.classList.add(childWidths[c]);
		}
	}
}
		
SCRIPT;
				}
				Factory::getDocument()->addScriptDeclaration($script);

			}

		}
		return $content;
	} // end of function

	protected function getNavigationIcons($showprevious = true, $shownext = true, $showtoggle = true) {
		$cfg = JEVConfig::getInstance();

		$registry = JevRegistry::getInstance("jevents");
		$params = $registry->get("jevents.moduleparams", new Registry);
		$content = "";
		$prev = "";
		$next = "";
		if ($params->get("showNavigation",0))
		{
			$page = (int) Factory::getApplication()->getUserState("jevents.moduleid" . $this->_modid . ".page", 0);
			if ($page > 0 || $params->get("modlatest_Mode", 0) != 2)
			{
				$prev = '<button class="jeviso_prev uk-button uk-button-default uk-button-small" onclick="fetchMoreFloatEvents(' . $this->_modid . ',-1);return false;"> &lt; </button>';
			}
			$next = '<button class="jeviso_next uk-button uk-button-default uk-button-small " onclick="fetchMoreFloatEvents(' . $this->_modid . ',1);return false;"> &gt; </button>';
		}

		if (!$showprevious)
		{
			$prev = "";
		}
		if (!$shownext)
		{
			$next = "";
		}
		$viewswitch = "";
		if ($cfg->get('float_dyn_switch', 1) == 1 && (int) $cfg->get('float_style', 1) == 3)
		{
			$lang = Factory::getLanguage();
			$lang->load("files_jeventsfloatlayout");

			$list =  Text::_('JEV_FLOAT_LIST_VIEW');
			$grid =  Text::_('JEV_FLOAT_GRID_VIEW');
$viewswitch = <<< VIEWSWITCH
<div class="jeviso_view_switch uk-buton-group uk-inline-block">
	<button class="uk-button uk-button-default uk-button-small " onclick="jeviso_listmodule($this->_modid);">$list</button>
	<button class="uk-button uk-button-default uk-button-small" onclick="jeviso_gridmodule($this->_modid);">$grid</button>
</div>
VIEWSWITCH;
		}
		if (!$showtoggle)
		{
			$viewswitch = '<div class="jeviso_view_switch uk-buton-group"></div>';
		}

		if (!empty($viewswitch) || !empty($next)  || !empty($prev))
		{
			$buttons = <<< LINKS
			<div class="jeviso_display_header uk-margin-small-bottom">
				$viewswitch
				<div class="date nv_links uk-buton-group  uk-inline-block uk-float-right">
					$prev
					$next
				</div>
			</div>
LINKS;
			if ($this->jeventsUIkit)
			{
				$buttons = str_replace("uk-", "gsl-" , $buttons);
			}
			$content .= $buttons;
		}
		return $content;
	}

	public function  processTemplate(&$content, $dayEvent)
	{
		if (is_array($this->template) && isset($this->template[0]))
		{
			ob_start();
			$eventtemplate = DefaultLoadedFromTemplate(false, 'module.latest_event.' . $this->_modid, $dayEvent, 0, false, true,  $this->floatmoduleuseformatstring);
			if (!$eventtemplate)
			{
				DefaultLoadedFromTemplate(false, 'module.latest_event', $dayEvent, 0, false, true, true);
			}
			$eventcontent = ob_get_clean();
			if ($this->jeventsUIkit)
			{
				$eventcontent = str_replace("uk-", "gsl-" , $eventcontent);
			}

			$content .= $eventcontent;
			return true;
		}
		return false;

	}

} // end of class
