<?php
/*
  Plugin Name: Remote Joomla Module Loader
  Plugin URI: https://www.jevents.net/
  Description: A plugin that loads a Joomla module to display on a wordpress site
  Version: 1.0
  Author: Geraint Edwards
  Author URI: http://www.jevents.net
  License: GPL
 */

function register_remotejoomlamodule_widget()
{
	register_widget('RemoteJoomlaModule_Widget');

}

add_action('widgets_init', 'register_remotejoomlamodule_widget');

class RemoteJoomlaModule_Widget extends WP_Widget
{

	private $debug = true;

	function __construct( $id_base, $name, $widget_options = array(), $control_options = array())
	{
		parent::__construct(
				'remotejoomlamodule_widget', // Base ID
				__('Load Remote Joomla Module', 'text_domain'), // Widget Name
				array('description' => __('Display Joomla Module on WP Site', 'text_domain'),)
		);

		// this is loaded after wp_enqueue_scripts
		//add_action('wp_head', array($this, "getscript"));
		// this is first
		add_action('wp_enqueue_scripts', array($this, 'scripts'));

	}

	public
			function _register()
	{
		return parent::_register();

	}

	private
			$widget_content = array();

	function scripts()
	{

		// Trick to get the customised scripts earlier than the call to widgets can do!
		$settings = $this->get_settings();
		foreach ($settings as $number => $instance)
		{
			if (is_active_widget(false, $this->id_base . "-" . $number, $this->id_base))
			{
				if (strrpos($instance["joomlaurl"], "/") !== strlen($instance["joomlaurl"])-1)
				{
					$instance["joomlaurl"] .= "/";
				}
				if (intval($instance['modid'])>0){
					//$url = $instance['joomlaurl'] . "modules/mod_remoteloader/remotemodule/secretmodule.php?loadjs=1&loadcss=1&asJSON=1&modid=" . $instance['modid'];
					$url = $instance['joomlaurl'] . "index.php?option=com_jevents&typeaheadtask=gwejson&json={}&file=secretmodule&path=module&folder=mod_remoteloader%2Fremotemodule&loadjs=1&loadcss=1&asJSON=1&modid=" . $instance['modid'];
					if ($this->debug) {
						$url .=  "&XDEBUG_SESSION_START=netbeans-xdebug";
					}
				}
				else {
					//$url = $instance['joomlaurl'] . "modules/mod_remoteloader/remotemodule/secretmodule.php?loadjs=1&loadcss=1&asJSON=1&itemid=" . $instance['itemid'];
					$url = $instance['joomlaurl'] . "index.php?option=com_jevents&typeaheadtask=gwejson&json={}&file=secretmodule&path=module&folder=mod_remoteloader%2Fremotemodule&loadjs=1&loadcss=1&asJSON=1&itemid=" . $instance['itemid'];
					if ($this->debug) {
						$url .=  "&XDEBUG_SESSION_START=netbeans-xdebug";
					}
				}
				//echo $url."<br/>";
				$this->widget_content[$this->id_base . "-" . $number] = wp_remote_retrieve_body(wp_remote_get($url));
		
				if ($this->widget_content[$this->id_base . "-" . $number])
				{
					$this->widget_content[$this->id_base . "-" . $number] = json_decode($this->widget_content[$this->id_base . "-" . $number]);
					$content = $this->widget_content[$this->id_base . "-" . $number];

					/*
					  static $cssloaded = false;
					  if (!$cssloaded)
					  {
					  // We need some inline CSS
					  // This makes sure that the positioning is also good for right-to-left languages
					  $x = is_rtl() ? 'left' : 'right';

					  $style = "
					  #dolly {
					  float: $x;
					  padding-$x: 15px;
					  padding-top: 5px;
					  margin: 0;
					  font-size: 11px;
					  }
					  ";
					  wp_add_inline_style("remotemodulestyle", $style);
					  }
					 */
					if (isset($content->head->stylesheets))
					{
						$count = 0;
						foreach ($content->head->stylesheets as $ss)
						{
							// eliminate double //
							if (strpos($ss->strSrc, '/') === 0) {
								$ss->strSrc = substr($ss->strSrc, 1);
							}
							$src = strpos($ss->strSrc, 'http') === 0 ? $ss->strSrc : $instance['joomlaurl'] . $ss->strSrc;
							wp_enqueue_style('remotemodulestyle' . $number . "-" . $count, $src, array(), null);
							$count++;
						}
					}
					wp_enqueue_style('remotemodulestyle', plugins_url("/css/remotemodule.css", __FILE__));

					if (isset($content->head->scriptfiles))
					{
						$count = 0;
						foreach ($content->head->scriptfiles as $sf)
						{
							// When adding scripts from source file WP always adds it to the footer AFTER the inline scripts !!!
							// So do not call this from the widget method!
							// eliminate double //
							if (strpos($sf->strSrc, '/') === 0) {
								$sf->strSrc = substr($sf->strSrc, 1);
							}
							$src = strpos($sf->strSrc, 'http') === 0 ? $sf->strSrc : $instance['joomlaurl'] . $sf->strSrc;
							wp_enqueue_script('remotemodulesscriptfile' . $number . "-" . $count, $src, array(), null);
							/*
							  add_action('wp_footer', function () use ($sf, $instance) {
							  $script = "<script type='text/javascript' src='" . $instance['joomlaurl'] . $sf->strSrc . "'  >x</script>\n";
							  echo $script;
							  });
							 */
							$count++;
						}
					}

					if (isset($content->head->scripts))
					{
						$count = 0;
						foreach ($content->head->scripts as $sf)
						{
							/*
							  add_action('wp_footer', function () use ($sf) {
							  echo "<script type='text/javascript' >" . $sf->content . "</script>\n";
							  });
							 */
							add_action('wp_head', function () use ($sf) {
								echo "<script type='text/javascript' >" . $sf->content . "</script>\n";
							});
						}
					}
				}
			}
			/*
			  // Add Json Callback script too!
			  $domain = $instance['joomlaurl'];
			  $jsoncallback = <<< JSON
			  function joomlaJsonCallBack()
			  {
			  alert(0);

			  };
			  JSON;
			  add_action('wp_head', function () use ($jsoncallback) {
			  echo "<script type='text/javascript' >" . $jsoncallback . "</script>\n";
			  });

			 */
		}

	}

	public
			function getscript()
	{
		//$instance = $this->get_settings()[$this->number];
		//echo '<script type="text/javascript" >alert(456);</script>';

	}

	public
			function form($instance)
	{

		$title = isset($instance['title']) ? $instance['title'] : "";
		$joomlaurl = isset($instance['joomlaurl']) ? $instance['joomlaurl'] : "";
		$modid = isset($instance['modid']) ? $instance['modid'] : "";
		$itemid = isset($instance['itemid']) ? $instance['itemid'] : "";
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Widget Title:'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>">
		</p>

		<p>
			<label for="<?php echo $this->get_field_id('joomlaurl'); ?>"><?php _e('Joomla URL:'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('joomlaurl'); ?>" name="<?php echo $this->get_field_name('joomlaurl'); ?>" type="text" value="<?php echo esc_attr($joomlaurl); ?>">
		</p>

		<p>
			<label for="<?php echo $this->get_field_id('modid'); ?>"><?php _e('Module Id:'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('modid'); ?>" name="<?php echo $this->get_field_name('modid'); ?>" type="text" value="<?php echo esc_attr($modid); ?>">
		</p>

		<p>
			<label for="<?php echo $this->get_field_id('itemid'); ?>"><?php _e('Menu Item Id:'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('itemid'); ?>" name="<?php echo $this->get_field_name('itemid'); ?>" type="text" value="<?php echo esc_attr($itemid); ?>">
		</p>

		<?php

	}

// array $instance Stores values from database.
	public
			function widget($args, $instance)
	{

		if (strrpos($instance["joomlaurl"], "/") !== strlen($instance["joomlaurl"]))
		{
			$instance["joomlaurl"] .= "/";
		}
		if (intval($instance['modid'])>0){
			//$url = $instance['joomlaurl'] . "modules/mod_remoteloader/remotemodule/secretmodule.php?loadjs=1&loadcss=1&asJSON=1&modid=" . $instance['modid'];
			$url = $instance['joomlaurl'] . "index.php?option=com_jevents&typeaheadtask=gwejson&json={}&file=secretmodule&path=module&folder=mod_remoteloader%2Fremotemodule&loadjs=1&loadcss=1&asJSON=1&modid=" . $instance['modid'];
			if ($this->debug) {
				$url .=  "&XDEBUG_SESSION_START=netbeans-xdebug";
			}
		}
		else {
			//$url = $instance['joomlaurl'] . "modules/mod_remoteloader/remotemodule/secretmodule.php?loadjs=1&loadcss=1&asJSON=1&itemid=" . $instance['itemid'];
			$url = $instance['joomlaurl'] . "index.php?option=com_jevents&typeaheadtask=gwejson&json={}&file=secretmodule&path=module&folder=mod_remoteloader%2Fremotemodule&loadjs=1&loadcss=1&asJSON=1&itemid=" . $instance['itemid'];
			if ($this->debug) {
				$url .=  "&XDEBUG_SESSION_START=netbeans-xdebug";
			}
		}

		//echo $url."<br/>";

		if ($this->widget_content[$this->id])
		{
			$content = $this->widget_content[$this->id];

			$title = apply_filters('widget_title', $instance['title']);

			echo $args['before_widget'];

			if (!empty($title))
			{
				echo $args['before_title'] . $title . $args['after_title'];
			}

			echo $content->data;

			echo $args['after_widget'];
		}

	}

	public
			function update($new_instance, $old_instance)
	{
		$instance = array();
		$instance['title'] = (!empty($new_instance['title']) ) ? strip_tags($new_instance['title']) : '';
		$instance['joomlaurl'] = (!empty($new_instance['joomlaurl']) ) ? strip_tags($new_instance['joomlaurl']) : '';
		$instance['modid'] = (!empty($new_instance['modid']) ) ? intval($new_instance['modid']) : "";
		$instance['itemid'] = (!empty($new_instance['itemid']) ) ? intval($new_instance['itemid']) : "";

		return $instance;

	}

}

// Experimental code
//
// Create the Manage RJM the Options submenus
//add_action('admin_menu', 'rjm_create_remotemodule_submenu');

function rjm_create_remotemodule_submenu()
{
	$rjm_remotemodule_page_id = get_option('rjm_remotemodule_page');
	if (!$rjm_remotemodule_page_id || !get_page($rjm_remotemodule_page_id))
	{
		add_action('admin_notices', "rjm_explain_remotemodule_page_missing");
	}

	if (function_exists('add_submenu_page'))
	{
		 $assetRoot = plugins_url('',plugin_basename(__FILE__)).'/';
		 // need to configure permissions - called capacilities in WP
		add_object_page(__('Remote Module', 'rjm'), __('Remote Module', 'rjm'), 'edit_posts', 'remotejoomlamodule', 'rjm_remotemodule_page', $assetRoot . 'img/calendar-16.png');
		// Add a submenu to the custom top-level menu:
		// edit event also needs just "add" as capability, otherwise you will not be able to edit own created events
		$plugin_page = add_submenu_page('remotejoomlamodule', __('Edit'), __('Edit'), 'edit_posts', 'rjm-manager', 'rjm_events_page');
		$plugin_page = add_submenu_page('remotejoomlamodule', __('Locations', 'rjm'), __('Locations', 'rjm'), 'edit_posts', 'rjm-locations', "rjm_locations_page");
	}

}

// To enable activation through the activate function
register_activation_hook(__FILE__, 'rjm_install');
// when deactivation is needed
register_deactivation_hook(__FILE__, 'rjm_uninstall');

function rjm_install()
{
	// Create RJM page if necessary
	$rjm_remotemodule_page = get_option('rjm_remotemodule_page');

	if ($rjm_remotemodule_page != "")
	{
		if (!get_page($rjm_remotemodule_page))
			rjm_create_remotemodule_page();
	} else
	{
		rjm_create_remotemodule_page();
	}
	//echo "installed RJM";

}

function rjm_create_remotemodule_page()
{
	global $wpdb;
	$postarr = array(
		'post_title' => wp_strip_all_tags(__('RemoteJoomla', 'rj')),
		'post_content' => __("This page is used by Remote Joomla Module. Don't change it, don't use it in your menu's, don't delete it. ' accordingly.", 'rj'),
		'post_type' => 'page',
		'post_status' => 'publish',
		'comment_status' => 'closed',
		'ping_status' => 'closed'
	);
	if ($int_post_id = wp_insert_post($postarr))
	{
		update_option('rjm_remotemodule_page', $int_post_id);
	}

}

function rjm_uninstall()
{
	rjm_delete_remotemodule_page();

}

function rjm_delete_remotemodule_page()
{
	$rjm_remotemodule_page_id = get_option('rjm_remotemodule_page');
	if ($rjm_remotemodule_page_id)
		wp_delete_post($rjm_remotemodule_page_id);

}

/*



//
// Create the Joomla and Options submenus
add_action('admin_menu','create_remotejoomla_submenu');
function create_remotejoomla_submenu () {
   $remotejoomla_page_id = get_option('remotejoomla_page');
   if (!$events_page_id || !get_page($events_page_id))
      add_action('admin_notices', "rjm_explain_events_page_missing");

   if(function_exists('add_submenu_page')) {
      add_object_page(__('Events', 'rjm'),__('Events', 'rjm'),get_option('rjm_cap_list_events'),'events-manager','rjm_events_page', $assetRoot.'images/calendar-16.png');
      // Add a submenu to the custom top-level menu:
      // edit event also needs just "add" as capability, otherwise you will not be able to edit own created events
      $plugin_page = add_submenu_page('events-manager', __('Edit'),__('Edit'),get_option('rjm_cap_list_events'),'events-manager','rjm_events_page');
      //add_action( 'admin_head-'. $plugin_page, 'rjm_admin_events_script' );
      //add_action( 'admin_head-'. $plugin_page, 'rjm_admin_event_boxes' );
      $plugin_page = add_submenu_page('events-manager', __('Add new', 'rjm'), __('Add new','rjm'), get_option('rjm_cap_add_event'), 'rjm-new_event', "rjm_new_event_page");
      //add_action( 'admin_head-'. $plugin_page, 'rjm_admin_events_script' );
      //add_action( 'admin_head-'. $plugin_page, 'rjm_admin_event_boxes' );
      $plugin_page = add_submenu_page('events-manager', __('Locations', 'rjm'), __('Locations', 'rjm'), get_option('rjm_cap_add_locations'), 'rjm-locations', "rjm_locations_page");
      if (get_option('rjm_categories_enabled')) {
         $plugin_page = add_submenu_page('events-manager', __('Event Categories','rjm'),__('Categories','rjm'), get_option('rjm_cap_categories'), "rjm-categories", 'rjm_categories_page');
      }
      $plugin_page = add_submenu_page('events-manager', __('Templates','rjm'),__('Templates','rjm'), get_option('rjm_cap_templates'), "rjm-templates", 'rjm_templates_page');
      if (get_option('rjm_rsvp_enabled')) {
         $plugin_page = add_submenu_page('events-manager', __('People', 'rjm'), __('People', 'rjm'), get_option('rjm_cap_people'), 'rjm-people', "rjm_people_page");
         $plugin_page = add_submenu_page('events-manager', __('Pending Approvals', 'rjm'), __('Pending Approvals', 'rjm'), get_option('rjm_cap_approve'), 'rjm-registration-approval', "rjm_registration_approval_page");
         $plugin_page = add_submenu_page('events-manager', __('Change Registration', 'rjm'), __('Change Registration', 'rjm'), get_option('rjm_cap_registrations'), 'rjm-registration-seats', "rjm_registration_seats_page");
         if (get_option('rjm_rsvp_mail_notify_is_active')) {
            $plugin_page = add_submenu_page('events-manager', __('Send Mails', 'rjm'), __('Send Mails', 'rjm'), get_option('rjm_cap_send_mails'), 'rjm-send-mails', "rjm_send_mails_page");
         }
         $plugin_page = add_submenu_page('events-manager', __('Form Fields','rjm'),__('Form Fields','rjm'), get_option('rjm_cap_forms'), "rjm-formfields", 'rjm_formfields_page');
      }
      $plugin_page = add_submenu_page('events-manager', __('Cleanup', 'rjm'), __('Cleanup', 'rjm'), get_option('rjm_cap_cleanup'), 'rjm-cleanup', "rjm_cleanup_page");

      # just in case: make sure the Settings page can be reached if something is not correct with the security settings
      if (get_option('rjm_cap_settings') =='')
         $cap_settings=DEFAULT_CAP_SETTINGS;
      else
         $cap_settings=get_option('rjm_cap_settings');
      $plugin_page = add_submenu_page('events-manager', __('Events Made Easy Settings','rjm'),__('Settings','rjm'), $cap_settings, "rjm-options", 'rjm_options_page');
      //add_action( 'admin_head-'. $plugin_page, 'rjm_admin_options_script' );
      // do some option checking after the options have been updated
      // add_action( 'load-'. $plugin_page, 'rjm_admin_options_save');
   }
}
*/